package com.asda.android.app.base

import android.content.Context
import android.os.Build
import android.util.Log
import androidx.core.app.NotificationCompat
import com.asda.android.R
import com.asda.android.designlibrary.view.dialog.AsdaNotification
import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.utils.Utils
import com.asda.android.utils.view.NotificationUtils
import com.urbanairship.AirshipConfigOptions
import com.urbanairship.Autopilot
import com.urbanairship.UAirship
import com.urbanairship.push.PushMessage
import com.urbanairship.push.notifications.ActionsNotificationExtender
import com.urbanairship.push.notifications.AirshipNotificationProvider
import com.urbanairship.push.notifications.NotificationArguments
import com.urbanairship.push.notifications.NotificationResult
import com.urbanairship.util.NotificationIdGenerator

class AirshipAutoPilot : Autopilot() {

    override fun createAirshipConfigOptions(context: Context): AirshipConfigOptions? {
        val isDev = Utils.isDevBuild()
        return AirshipConfigOptions.Builder()
              .setInProduction(!isDev) //determines which app key to use
              .setDevelopmentAppKey(context.getString(R.string.airship_key_dev))
              .setDevelopmentAppSecret(context.getString(R.string.airship_secret_dev))
              .setProductionAppKey(context.getString(R.string.airship_key_prod))
              .setProductionAppSecret(context.getString(R.string.airship_secret_prod))
              .setFcmSenderId(if (isDev) context.getString(R.string.fcm_sender_id_dev) else context.getString(R.string.fcm_sender_id_prod))
              .build()
    }

    override fun onAirshipReady(airship: UAirship) {
        if (AsdaServiceSettings.DEBUG) {
            Log.d(TAG, "Channel ID is " + airship.pushManager.channelId)
        }
        val listener = AirshipPushListener()
        airship.pushManager.addPushListener(listener)
        airship.pushManager.addRegistrationListener(listener)
        airship.pushManager.setNotificationListener(listener)
        airship.pushManager.notificationProvider = createAirshipConfigOptions(UAirship.getApplicationContext())?.let { NotificationProvider(UAirship.getApplicationContext(), it) }
        if (SharedPreferencesUtil.isPushEnabled(UAirship.getApplicationContext())) {
            try {
                airship.pushManager.isPushEnabled = true
                airship.pushManager.userNotificationsEnabled = true
                if (Build.VERSION.SDK_INT >= 26) {
                    NotificationUtils.setNotificationChannel(UAirship.getApplicationContext(), CHANNEL_ID,
                          UAirship.getApplicationContext().getString(R.string.airship_channel_name))
                }
            } catch (th: Throwable) {
                Log.e(TAG, "push messaging enabling failed", th)
            }

        }

        val isMsgCenterEnabled = SharedPreferencesUtil.isMessageCenterAvailable(UAirship.getApplicationContext())
        val modified = AirshipPushListener.internalSetTag(airship, TAG_MESSAGE_CENTER, isMsgCenterEnabled)
        if (isMsgCenterEnabled && modified) {
            // Attempt to refresh messages to get any automated messages
            airship.inbox.fetchMessages()
        }

    }

    class NotificationProvider(context: Context, options: AirshipConfigOptions) : AirshipNotificationProvider(context, options) {

        override fun onCreateNotificationArguments(context: Context, message: PushMessage): NotificationArguments {
            val channelId = CHANNEL_ID
            val notificationId = NotificationIdGenerator.nextID()

            return NotificationArguments.newBuilder(message)
                  .setNotificationChannelId(channelId)
                  .setNotificationId("tag", notificationId)
                  .setRequiresLongRunningTask(false)
                  .build()
        }


        override fun onCreateNotification(context: Context, arguments: NotificationArguments): NotificationResult {
            if (arguments.message.alert.isNullOrEmpty()) {
                return NotificationResult.cancel()
            }

            val builder = AsdaNotification().newInstance(context, CHANNEL_ID,
                  if (arguments.message.title.isNullOrEmpty()) context.getString(R.string.app_name) else arguments.message.title ,
                  arguments.message.alert, NotificationCompat.BigTextStyle().bigText(arguments.message.alert), true)
            builder.extend(ActionsNotificationExtender(context, arguments))

            return NotificationResult.notification(builder.build())
        }
    }

    companion object {
        private const val TAG = "AirshipAutoPilot"
        private const val TAG_MESSAGE_CENTER = "msg_center"
        private const val CHANNEL_ID = "notify_airship_message"
    }
}
