package com.asda.android.app.base


import android.app.NotificationManager
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Handler
import android.text.TextUtils
import android.util.Log
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.AsdaEngineeringAnalyticsEvent
import com.asda.android.app.main.AsdaApplication
import com.asda.android.app.main.view.SplashActivity
import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.restapi.service.data.PushNotificationEvent
import com.asda.android.service.EventUtils
import com.asda.android.service.constants.EventConstants
import com.urbanairship.UAirship
import com.urbanairship.push.*
import org.json.JSONObject

class AirshipPushListener : NotificationListener, PushListener, RegistrationListener {


    override fun onNotificationPosted(notificationInfo: NotificationInfo) {
        if (AsdaServiceSettings.DEBUG) {
            Log.d(TAG, "Received background push message")
        }
    }

    override fun onNotificationOpened(notificationInfo: NotificationInfo): Boolean {
        val pushMessage = notificationInfo.message
        val actions = pushMessage.pushBundle.getString(PushMessage.EXTRA_ACTIONS)
        var deepLink: String? = null
        var notifHandled = false
        if (actions != null) {
            try {
                val jsonObject = JSONObject(actions)
                deepLink = jsonObject.optString("^d", null)
                if (deepLink != null) {
                    val launch = Intent(Intent.ACTION_VIEW)
                    launch.data = Uri.parse(deepLink)
                    launch.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TOP)
                    UAirship.getApplicationContext().startActivity(launch)
                    notifHandled = true
                } else if (jsonObject.optString("^u", null) != null) {
                    // URL actions are handled by UA SDK itself
                    notifHandled = true
                }
            } catch (e: Exception) {
                Log.d(TAG, "deep link failed", e)
            }

        }
        if (!notifHandled) {
            val launch = Intent(Intent.ACTION_MAIN)
            launch.setClass(UAirship.getApplicationContext(), SplashActivity::class.java)
            launch.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TOP)
            launch.putExtras(pushMessage.pushBundle)

            val messageId = pushMessage.richPushMessageId
            if (!TextUtils.isEmpty(messageId)) {
                // Launch the main activity to the message in the inbox
                launch.putExtra(EXTRA_MESSAGE_ID, messageId)
                launch.putExtra(EXTRA_NAVIGATE_ITEM, INBOX_ITEM)
                dismissInboxNotification()
            }

            UAirship.getApplicationContext().startActivity(launch)
        }

        val event = AsdaEngineeringAnalyticsEvent(Anivia.MESSAGE_TAP_EVENT)
        if (deepLink != null) {
            event.putString(Anivia.MSG_DEST_KEY, deepLink)
        } else {
            event.putString(Anivia.MSG_DEST_KEY, if (pushMessage.pushBundle.containsKey(PushNotificationEvent.PUSH_NOTIFICATION_TYPE))
                pushMessage.pushBundle.getString(PushNotificationEvent.PUSH_NOTIFICATION_TYPE)
            else
                "home")
        }
        Tracker.get().trackEvent(event.putString(Anivia.MSG_ID_KEY, getAlertExcerpt(pushMessage))
                .putString(Anivia.MSG_TYPE_KEY, "push notification"))
        return true
    }

    override fun onNotificationForegroundAction(notificationInfo: NotificationInfo, notificationActionButtonInfo: NotificationActionButtonInfo): Boolean {
        return false
    }

    override fun onNotificationBackgroundAction(notificationInfo: NotificationInfo, notificationActionButtonInfo: NotificationActionButtonInfo) {
        //Implementation Not required
    }

    override fun onNotificationDismissed(notificationInfo: NotificationInfo) {
        //Implementation Not required
    }

    override fun onPushReceived(message: PushMessage, notificationPosted: Boolean) {
        if (AsdaServiceSettings.DEBUG) {
            Log.d(TAG, "Received push message")
        }
        Handler().post {
            Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.PUSH_NOTIFICATION_EVENT)
                  .putString(Anivia.NOTIF_ID_KEY, getAlertExcerpt(message)))
        }
        if (AsdaApplication.getInstance()!!.isAppInForeGround && AsdaApplication.getInstance()?.getString(R.string.notification_text_for_checkin)
                    .equals(message.pushBundle.getString(PushMessage.EXTRA_ALERT), ignoreCase = true)) {
            //send a message to refresh the Wismo cards
            EventUtils.broadcast(AsdaApplication.getInstance(), EventConstants.CHECKIN_ORDER_STATE_CHANGED)
        }
    }

    override fun onChannelCreated(s: String) {
        //Implementation Not required
    }

    override fun onChannelUpdated(s: String) {
        //Implementation Not required
    }

    override fun onPushTokenUpdated(s: String) {
        //Implementation Not required
    }

    companion object {

        private val TAG = "AirshipPushListener"
        const val TAG_HAVE_VODKA_IN_CART = "have_vodka_in_cart"
        const val TAG_HAVE_MILK_IN_CART = "have_milk_in_cart"
        const val TAG_HAVE_MARZIPAN_IN_CART = "have_marzipan_in_cart"
        const val TAG_HAVE_CREDENTIALS = "have_credentials"
        const val EXTRA_MESSAGE_ID = "message_id_arg"
        const val EXTRA_NAVIGATE_ITEM = "navigate_item_arg"
        const val INBOX_ITEM = "inbox_type"
        private const val INBOX_NOTIFICATION_ID = 9000000


        private fun getAlertExcerpt(pushMessage: PushMessage?): String {
            var msg = "unknown"
            if (pushMessage != null) {
                val alert = pushMessage.alert
                if (!TextUtils.isEmpty(alert)) {
                    msg = if (alert!!.length > 60) alert.substring(0, 60) else alert
                }
            }
            return msg
        }

        fun setUpNamedUserID(singleProfileID: String){
            UAirship.shared { it.namedUser.id = singleProfileID }
        }

        fun setTag(tag: String, present: Boolean) {
            UAirship.shared { airship -> internalSetTag(airship, tag, present) }
        }

        fun internalSetTag(airship: UAirship, tag: String?, present: Boolean): Boolean {
            var modified = false
            try {
                val pushManager = airship.pushManager
                if (tag != null && pushManager != null) {
                    val tagEditor = pushManager.editTags()
                    if (tagEditor != null) {
                        if (present) {
                            tagEditor.addTag(tag)
                        } else {
                            tagEditor.removeTag(tag)
                        }
                        tagEditor.apply()
                        modified = true
                    }
                }
            } catch (t: Throwable) {
                Log.e(TAG, "Could not set UA tag", t)
            }

            return modified
        }

        fun dismissInboxNotification() {
            val manager = AsdaApplication.getInstance()!!.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
            manager.cancel(INBOX_NOTIFICATION_ID)
        }
    }
}
