package com.asda.android.app.bookslot

import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.os.Bundle
import android.text.TextUtils
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.app.bookslotv3.view.fragment.BookSlotFragment
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.app.shop.ShelfListAdapter
import com.asda.android.app.shop.ShelfListAlternativesFragment
import com.asda.android.base.core.listener.OnSingleClickListener
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.constants.ShelfListAdapterConstants
import com.asda.android.restapi.service.NetworkCallback
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.data.BookSlotResponse
import com.asda.android.restapi.service.data.PriceChange
import com.asda.android.restapi.service.data.ReplacementsResponse
import com.asda.android.restapi.service.data.ShelfItem
import com.asda.android.restapi.service.data.bookslotv3.ReserveSlotCXOResponse
import com.asda.android.restapi.service.data.bookslotv3.ReserveSlotResponseV3
import com.asda.android.restapi.service.data.catalog.IroProductDetailsPlp
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.utils.view.ViewUtil
import io.reactivex.SingleObserver
import io.reactivex.SingleSource
import java.util.*
import kotlin.collections.ArrayList


class PreviewBookSlotPresenter : FeaturedFragment(), SingleSource<Boolean> {
    private var mContext: Activity? = null
    private var mView: View? = null
    private var progressDialog: Dialog? = null
    private var mPriceChanges = ArrayList<PriceChange>()
    private var mObserver: SingleObserver<in Boolean>? = null
    private var fulfilledByPrimaryStore: Boolean? = null

    override fun loadBundleData(bundle: Bundle?) {
        super.loadBundleData(bundle)
        if (bundle != null) {
            context?.let {
                if (FeatureSettings.isCXOMigrationEnabled(it)) {
                    val allParcelablesList = bundle.getParcelableArrayList<PriceChange>(PRICE_CHANGES_LIST)
                    if (allParcelablesList != null) {
                        mPriceChanges.addAll(allParcelablesList)
                    }
                } else {
                    val allParcelables = bundle.getParcelableArray(PRICE_CHANGES)
                    if (allParcelables != null) {
                        for (i in allParcelables.indices) {
                            mPriceChanges.add(allParcelables[i] as PriceChange)
                        }
                    }
                }

                if (bundle.containsKey(FULFILLED_BY_PRIMARY_STORE)) {
                    fulfilledByPrimaryStore = bundle.getBoolean(FULFILLED_BY_PRIMARY_STORE)
                }
            }
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mContext = context as Activity
    }

    override fun onDestroy() {
        super.onDestroy()
        if (mObserver != null) {
            mObserver?.onSuccess(false)
        }
    }

    private fun onContinueBookClick() {
        progressDialog = DialogFactory.createDialog(DialogFactory.LOADING_INDICATOR_NOT_CANCELABLE, mContext?.getString(R.string.booking_slot_progress), mContext)
        progressDialog?.show()
    }

    override fun onStop() {
        super.onStop()
        try {
            progressDialog?.dismiss()
        } catch (ignored: Exception) {
        }

    }

    override fun onCreateView(inflater: LayoutInflater, fragmentContainer: ViewGroup?, savedInstanceState: Bundle?): View {
        if (mView == null) {
            mView = inflater.inflate(R.layout.simple_recycler_view, fragmentContainer, false)
            val list = mView?.findViewById<RecyclerView>(R.id.list)
            list?.setHasFixedSize(true)
            list?.layoutManager = LinearLayoutManager(mContext)
            val header = ViewUtil.inflate<View>(mContext, R.layout.slot_book_preview_header, list, false)
            val adapter = ShelfListAdapter(mContext, getShelfItems(mContext, mPriceChanges.toTypedArray()),
                  R.layout.price_change_list_entry, false)
            adapter.addHeaderView(header)
            adapter.setItemsEnabled(false)
            adapter.setEmptyTextVisibility(View.GONE)
            adapter.setOnShelfItemClickListener { shelfItem, _ ->

                if (isIroBasedApiCallPossibleForReplacements(shelfItem, requireContext())) {
                    val items = TextUtils.join(",", shelfItem.availableReplacements!!)
                    AsdaServiceFactory.get().iroGetProductDetails(items, "basic", object : NetworkCallback<IroProductDetailsPlp>() {
                        override fun onFailure(error: Int?, data: IroProductDetailsPlp) {
                            if (context == null) return
                            AsdaDialogHelper.displayErrorDialog(data, error, context, false, BookSlotFragment.SCREEN_NAME)
                        }

                        override fun onSuccess(data: IroProductDetailsPlp?) {
                            if (context == null) return
                            if (data?.getItems()?.isNotEmpty() == true) {
                                val replacements = ArrayList<ShelfItem>()
                                for (item in data.getItems()!!)
                                    replacements.add(ShelfItem(item))

                                if (replacements.isNotEmpty()) {
                                    val fragment = ShelfListAlternativesFragment.newInstance(replacements.toTypedArray())
                                    push(fragment)
                                } else {
                                    DialogFactory.createAlertDialog(getString(R.string.shelf_list_no_alternatives), null, null, getString(R.string.ok), mContext, BookSlotFragment.SCREEN_NAME).show()
                                }
                            }
                        }
                    })
                } else
                    AsdaServiceFactory.get().getReplacementItems(shelfItem.id, object : NetworkCallback<ReplacementsResponse>() {
                        override fun onFailure(error: Int?, data: ReplacementsResponse) {
                            if (context == null) return
                            AsdaDialogHelper.displayErrorDialog(data, error, context, false, BookSlotFragment.SCREEN_NAME)
                        }

                        override fun onSuccess(data: ReplacementsResponse) {
                            if (context == null) return
                            if (data.replacements?.isNotEmpty() == true
                                  && data.replacements?.get(0)?.replacementItems?.isNotEmpty() == true) {
                                val fragment = ShelfListAlternativesFragment.newInstance(data.replacements?.get(0)?.replacementItems)
                                push(fragment)
                            } else {
                                DialogFactory.createAlertDialog(getString(R.string.shelf_list_no_alternatives), null, null, getString(R.string.ok), mContext, BookSlotFragment.SCREEN_NAME).show()
                            }
                        }
                    })
            }
            list?.adapter = adapter

            header.findViewById<View>(R.id.continue_button).setOnClickListener(object : OnSingleClickListener(this) {
                override fun onSingleClick(v: View) {
                    onContinueBookClick()
                    if (mObserver != null) {
                        mObserver?.onSuccess(true)
                    }
                }
            })

            setupSecondaryStoreMessage(header)

            /**
             * TODO: Check this:
             * The following ticket seems to indicate that we need to show the delivery surcharge message
             * in all cases, not just when amending.
             * https://jira.walmart.com/browse/ASDAA-11370
             *
             * However, the following ticket says we need this in amending case (only?)
             * https://jira.walmart.com/browse/ASDAA-11365
             *
             * So for now, I removed the check `if (CartManager.get(mContext).isAmending())` as we have the following bug filed:
             * https://jira.walmart.com/browse/ASDAA-11516
             */
            // TODO: This is obsolete until "/slot/reserve" API implements DeliverySurcharge. For now comment this.
            /*
            Spanned description = Utils.fromHtml(mContext.getString(R.string.delivery_surcharge_trolley_changes));
            deliverySurcharge.setText(description);
            deliverySurcharge.setVisibility(View.VISIBLE);
            deliverySurcharge.setOnClickListener(view -> displayMinBasketDialog(mContext));
            */
        }
        return mView!!
    }

    private fun setupSecondaryStoreMessage(view: View?) {
        val deliveryImpactView = view?.findViewById<TextView>(R.id.delivery_impact_message)
        if (FeatureSettings.isSlotSecondaryStoreEnabled(requireContext()) && fulfilledByPrimaryStore != true) {
            deliveryImpactView?.text = context?.getString(R.string.delivery_impact_secondary_store_message)
        }
    }

    private fun isIroBasedApiCallPossibleForReplacements(itemData: ShelfListItem?, context: Context): Boolean {
        return (!itemData?.availableReplacements.isNullOrEmpty() && itemData?.availableReplacements?.size?.compareTo(0) != 0)
    }

    private fun displayMinBasketDialog(activity: Activity?) {
        if (activity != null) {
            val title = activity.getString(R.string.delivery_surcharge_title)
            val message = activity.getString(R.string.delivery_surcharge_trolley_changes_desc)
            val d = DialogFactory.createAlertDialog(title, message, activity, null)
            try {
                d.show()
            } catch (ex: Exception) {

            }

        }
    }

    override fun hasActionBar(): Boolean {
        return true
    }

    override fun getActionBarTitle(): String? {
        return mContext?.getString(R.string.preview_book_slot_title)
    }

    fun popPresenterIfOnTop() {
        if (isVisible) {
            pop()
        }
    }

    override fun subscribe(observer: SingleObserver<in Boolean>) {
        mObserver = observer
    }

    companion object {
        const val TAG = "PreviewBookSlotPres-er"
        private const val PRICE_CHANGES = "price_changes"
        private const val PRICE_CHANGES_LIST = "price_changes_list"
        private const val FULFILLED_BY_PRIMARY_STORE = "fulfilled_by_primary_store"

        fun newInstance(data: BookSlotResponse): PreviewBookSlotPresenter {
            val fragment = PreviewBookSlotPresenter()
            val args = Bundle()
            args.putParcelableArray(PRICE_CHANGES, data.messages)
            fragment.arguments = args
            return fragment
        }

        fun newInstance(slotResponseV3: ReserveSlotResponseV3?): PreviewBookSlotPresenter {
            val fragment = PreviewBookSlotPresenter()
            val args = Bundle()
            if (slotResponseV3?.data != null && slotResponseV3.data?.messages != null) {
                val messages = slotResponseV3.data?.messages
                args.putParcelableArray(PRICE_CHANGES, messages)

                slotResponseV3.data?.slot_info?.isPrimary?.let {
                    args.putBoolean(FULFILLED_BY_PRIMARY_STORE, it)
                }
            }
            fragment.arguments = args
            return fragment
        }

        fun newInstance(slotResponseV3: ReserveSlotCXOResponse?): PreviewBookSlotPresenter {
            val fragment = PreviewBookSlotPresenter()
            val args = Bundle()
            if (slotResponseV3?.data?.delivery_impact != null) {
                var messages = ArrayList<PriceChange>()

                if (!slotResponseV3.data?.delivery_impact?.delistedItems.isNullOrEmpty()) {
                    populatePriceChangeMessages(slotResponseV3.data?.delivery_impact?.delistedItems!!, PriceChange.DELISTED)?.let { messages.addAll(it) }
                }
                if (!slotResponseV3.data?.delivery_impact?.promoChangedItems.isNullOrEmpty()) {
                    populatePriceChangeMessages(slotResponseV3.data?.delivery_impact?.promoChangedItems!!, PriceChange.TYPE_PROMOS_CHANGED_NAMES)?.let { messages.addAll(it) }
                }
                if (!slotResponseV3.data?.delivery_impact?.priceChangedItems.isNullOrEmpty()) {
                    populatePriceChangeMessages(slotResponseV3.data?.delivery_impact?.priceChangedItems!!, PriceChange.PRICE_CHANGED_ITEMS)?.let { messages.addAll(it) }
                }

                args.putParcelableArrayList(PRICE_CHANGES_LIST, messages)
                slotResponseV3.data?.cart?.fulfillmentDetails?.slotInfo?.isPrimary?.let {
                    args.putBoolean(FULFILLED_BY_PRIMARY_STORE, slotResponseV3.data?.cart?.fulfillmentDetails?.slotInfo?.isPrimary == true)
                }
            }
            fragment.arguments = args
            return fragment
        }

        private fun populatePriceChangeMessages(items: java.util.ArrayList<PriceChange.PriceChangeItem>, type: String? = null)
              : ArrayList<PriceChange>? {
            val priceChangeMessages = ArrayList<PriceChange>()
            val priceChange = PriceChange()
            type?.let {
                priceChange.type = type
            }

            com.asda.android.cxo.CartManager.instance?.cartResponse?.data?.cart?.cartItems?.let { cartItems ->
                val cartItemsMap = cartItems.map { it.skuId to it }.toMap()
                for (item in items) {
                    val cartItem = cartItemsMap[item.skuid]
                    item.name = cartItem?.catalogInfo?.catalogItem?.name
                    item.imageURL = RestUtils.composeScene7ImageUrl(cartItem?.catalogInfo?.catalogItem?.catalogImages?.scene7Host, cartItem?.catalogInfo?.catalogItem?.catalogImages?.scene7Id, false)
                }
            }


            priceChange.items = items.toTypedArray()
            priceChangeMessages.add(priceChange)
            return priceChangeMessages
        }

        fun getShelfItems(context: Context?, changesData: Array<PriceChange>?): ArrayList<ShelfListItem> {
            val itemsToDisplay = ArrayList<ShelfListItem>()
            if (changesData != null && changesData.isNotEmpty()) {
                val promoChangedItems = ArrayList<ShelfListItem>()
                val delistedItems = ArrayList<ShelfListItem>()
                val priceChangedItems = ArrayList<ShelfListItem>()
                val priceChangedItemsIds = HashSet<String>()
                for (priceChange in changesData) {
                    if (priceChange.items != null && priceChange.items?.size ?: 0 > 0) {
                        if (PriceChange.TYPE_PROMOS_CHANGED_NAMES.equals(priceChange.type, ignoreCase = true)) {
                            if (promoChangedItems.isEmpty()) {
                                promoChangedItems.add(ShelfListItem(context?.getString(R.string.expired_offers_subtitle), ShelfListAdapterConstants.TYPE_DEPARTMENT_TITLE))
                            }
                            priceChange.items?.let {
                                for (priceChangeItem in it) {
                                    if (!TextUtils.isEmpty(priceChangeItem.name)) {
                                        val item = ShelfListItem()
                                        item.title = priceChangeItem.name
                                        item.type = ShelfListAdapterConstants.TYPE_PROMO_CHANGED
                                        promoChangedItems.add(item)
                                    }
                                }
                                if (promoChangedItems.size > 1) {
                                    itemsToDisplay.addAll(promoChangedItems)
                                }
                            }
                        } else if (priceChange.type != null && priceChange.type?.toLowerCase(Locale.UK)?.contains(PriceChange.DELISTED) == true) {
                            priceChange.items?.let {
                                for (priceChangeItem in it) {
                                    val shelfListItem = ShelfListItem(priceChangeItem, false)
                                    shelfListItem.priceIncreased = null
                                    shelfListItem.imageURL = ShelfListAdapter.pickImageUrl(priceChangeItem.extraLargeImageURL, priceChangeItem.imageURL)
                                    delistedItems.add(shelfListItem)
                                }
                                itemsToDisplay.addAll(delistedItems)
                            }
                        } else if ((priceChange.type == PriceChange.PRICE_CHANGED_ITEMS) || (priceChange.type == PriceChange.PRICECHANGEDITEMS)) {
                            priceChange.items?.let {
                                for (priceChangeItem in it) {
                                    val listItem = ShelfListItem(priceChangeItem, true)
                                    listItem.imageURL = ShelfListAdapter.pickImageUrl(priceChangeItem.extraLargeImageURL, priceChangeItem.imageURL)
                                    if (listItem.id == null || priceChangedItemsIds.add(listItem.id)) {
                                        priceChangedItems.add(listItem)
                                    } else {
                                        Log.w(TAG, "Don't show " + listItem.title + " because it's already shown, oldPrice " + priceChangeItem.oldPrice + "->" + priceChangeItem.price)
                                    }
                                }
                                itemsToDisplay.addAll(priceChangedItems)
                            }

                        }
                    }
                }
            }
            return itemsToDisplay
        }
    }
}
