package com.asda.android.app.bookslotv3.adapter

import android.content.Context
import android.graphics.Paint
import android.graphics.Typeface
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.app.bookslotv3.BookSlotGridInterface
import com.asda.android.app.bookslotv3.BookSlotType
import com.asda.android.app.bookslotv3.validator.BookSlotValidator
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.DateFormats
import com.asda.android.base.core.convertToFormat
import com.asda.android.base.core.view.ui.gone
import com.asda.android.base.core.view.ui.visible
import com.asda.android.restapi.service.data.bookslotv3.SlotInfo
import com.asda.android.restapi.service.data.bookslotv3.SlotStatus
import com.asda.android.utils.Utils
import java.util.*

class BookSlotAdapter(private val context: Context, private val data: ArrayList<Any>,
                      private val currentFragmentType: String,
                      private val bookSlotGridListener: BookSlotGridInterface) : RecyclerView.Adapter<RecyclerView.ViewHolder>() {
    private val layoutInflater: LayoutInflater = LayoutInflater.from(context)

    private val ITEM_TYPE_HEADER = 0
    private val ITEM_TYPE_SLOT = 1

    private var viewingSlotsForBookedSlotAddress = false
    private var bookedStartAndEndTime = Pair("", "")

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        return if (viewType == ITEM_TYPE_SLOT)
            BookSlotGridItemViewHolder(layoutInflater.inflate(R.layout.book_slot_grid_item, parent, false))
        else
            BookSlotGridItemColumnHeaderViewHolder(layoutInflater.inflate(R.layout.book_slot_grid_header, parent, false))
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        val itemDataObject = data[position]
        if (itemDataObject is SlotInfo) {
            val vh = holder as BookSlotGridItemViewHolder
            val slotStatusForVoiceOver = StringBuilder()
            val slotPriceForVoiceOver = StringBuilder()
            slotPriceForVoiceOver.append(context.resources.getString(R.string.pound_amount, itemDataObject.final_slot_price))

            resetViews(vh)
            if (SlotStatus.UNAVAILABLE.type == itemDataObject.status) {
                setupUnavailableGridItem(vh, itemDataObject, slotStatusForVoiceOver, slotPriceForVoiceOver)
            } else {
                if (!FeatureSettings.isSlotsDateTimeAddressForReservedStatusEnabled(context) && SlotStatus.RESERVED.type == itemDataObject.status) {
                    setupReservedGridItem(vh, itemDataObject, slotStatusForVoiceOver)
                } else {
                    setupAvailableGridItem(vh, itemDataObject, slotStatusForVoiceOver, slotPriceForVoiceOver)
                }
            }

            vh.gridItemParent.contentDescription = context.getString(R.string.slot_grid_voice_over,
                  slotStatusForVoiceOver,
                  itemDataObject.start_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.DD_MMM_YYYY),
                  itemDataObject.start_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA),
                  itemDataObject.end_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA),
                  slotPriceForVoiceOver)

            vh.gridItemParent.setOnClickListener {
                bookSlotGridListener.onGridItemClick(itemDataObject, vh.gridItemParent)
            }
        } else if (itemDataObject is String) {
            val vh1 = holder as BookSlotGridItemColumnHeaderViewHolder
            vh1.itemText.text = itemDataObject
        }
    }

    private fun setupAvailableGridItem(vh: BookSlotGridItemViewHolder,
                                       slotInfo: SlotInfo,
                                       slotStatusForVoiceOver: StringBuilder,
                                       slotPriceForVoiceOver: StringBuilder) {
        when {
            slotInfo.isSlotFree -> {
                vh.gridItemHeader.visibility = View.GONE
                customizeFontAppearance(
                      textView = vh.gridPriceText,
                      visibility = View.VISIBLE,
                      text = context.resources.getString(R.string.free_first_upper),
                      textColorId = R.color.black,
                      typefaceValue = Typeface.BOLD,
                      isUnderlined = false
                )
                slotStatusForVoiceOver.append(context.getString(R.string.slot_available))
                slotPriceForVoiceOver.append(context.getString(R.string.free_first_upper))
            }

            isExpressEnabledInFragment(context, currentFragmentType)
                  && slotInfo.is_express_slot -> {
                setupExpressGridData(vh, slotInfo, slotStatusForVoiceOver)
            }

            slotInfo.isSameDayPremium -> {
                vh.gridItemHeader.visibility = View.GONE
                customizeFontAppearance(
                      textView = vh.gridPriceText,
                      visibility = View.VISIBLE,
                      text = context.resources.getString(R.string.pound_amount_with_asterisk, slotInfo.final_slot_price),
                      textColorId = R.color.black,
                      typefaceValue = Typeface.BOLD,
                      isUnderlined = false
                )
                slotStatusForVoiceOver.append(context.getString(R.string.slot_available))
            }

            slotInfo.hasRestrictedItem -> {
                setupNoAlcoholText(vh, slotInfo, slotStatusForVoiceOver)
                customizeFontAppearance(
                      textView = vh.gridPriceText,
                      visibility = View.VISIBLE,
                      text = context.resources.getString(R.string.pound_amount, slotInfo.final_slot_price),
                      textColorId = R.color.black,
                      typefaceValue = Typeface.BOLD,
                      isUnderlined = false
                )
            }

            else -> {
                vh.gridItemHeader.visibility = View.GONE
                customizeFontAppearance(
                      textView = vh.gridPriceText,
                      visibility = View.VISIBLE,
                      text = context.resources.getString(R.string.pound_amount, slotInfo.final_slot_price),
                      textColorId = R.color.black,
                      typefaceValue = Typeface.BOLD,
                      isUnderlined = false
                )
                slotStatusForVoiceOver.append(context.getString(R.string.slot_available))
            }
        }

        if (viewingSlotsForBookedSlotAddress && BookSlotValidator().isTimeMatch(slotInfo, bookedStartAndEndTime) && SlotStatus.RESERVED.type == slotInfo.status) {
            setupReservedGridItem(vh, slotInfo, slotStatusForVoiceOver)
        }

        if (FeatureSettings.isSlotSecondaryStoreEnabled(context) && slotInfo.isPrimary == false) {
            vh.gridPriceText.text = context.resources.getString(R.string.pound_amount_with_informational_plus, vh.gridPriceText.text)
        }
    }

    private fun setupUnavailableGridItem(vh: BookSlotGridItemViewHolder, slotInfo: SlotInfo,
                                         slotStatusForVoiceOver: StringBuilder,
                                         slotPriceForVoiceOver: StringBuilder) {
        when {
            isExpressEnabledInFragment(context, currentFragmentType)
                  && slotInfo.isExpressAndSlotUnavailable -> {
                slotStatusForVoiceOver.append(context.getString(R.string.slot_unavailable))
                slotPriceForVoiceOver.clear()
                setExpressCellView(vh, true)
                vh.gridExpressSlot.alpha = 0.5f
                vh.gridPriceText.alpha = 0.5f
                vh.gridExpressContainer?.visibility = View.GONE
                customizeFontAppearance(
                      textView = vh.gridPriceText,
                      visibility = View.VISIBLE,
                      text = context.resources.getString(R.string.pound_amount, slotInfo.final_slot_price),
                      textColorId = R.color.black,
                      typefaceValue = Typeface.BOLD,
                      isUnderlined = false
                )
            }

            slotInfo.is_express_slot
                  && isExpressEnabledInFragment(context, currentFragmentType)
                  && slotInfo.hasRestrictedItem -> {
                setupExpressGridData(vh, slotInfo, slotStatusForVoiceOver)
            }

            slotInfo.hasRestrictedItem -> {
                slotStatusForVoiceOver.append(context.getString(R.string.slot_unavailable))
                slotPriceForVoiceOver.clear()
                setupNoAlcoholText(vh, slotInfo, slotStatusForVoiceOver)
                customizeFontAppearance(
                      textView = vh.gridPriceText,
                      visibility = View.VISIBLE,
                      text = context.resources.getString(R.string.pound_amount, slotInfo.final_slot_price),
                      textColorId = R.color.black,
                      typefaceValue = Typeface.BOLD,
                      isUnderlined = false
                )
            }

            else -> {
                // If slot is UNAVAILABLE
                slotStatusForVoiceOver.append(context.getString(R.string.slot_unavailable))
                customizeFontAppearance(
                      textView = vh.gridItemHeader,
                      visibility = View.VISIBLE,
                      text = context.resources.getString(R.string.sold_out),
                      textColorId = R.color.color_gray_767676,
                      typefaceValue = Typeface.NORMAL,
                      isUnderlined = false
                )
                customizeFontAppearance(
                      textView = vh.gridPriceText,
                      visibility = View.VISIBLE,
                      text = context.resources.getString(if (currentFragmentType == BookSlotType.HOME_DELIVERY) R.string.collect_with_question else R.string.delivery_with_question),
                      textColorId = R.color.color_ocean_blue_0073b1,
                      typefaceValue = Typeface.NORMAL,
                      isUnderlined = true
                )
                slotStatusForVoiceOver.append(context.getString(R.string.slot_sold_out))
                slotPriceForVoiceOver.append(context.getString(if (currentFragmentType == BookSlotType.HOME_DELIVERY) R.string.try_collect_with_question else R.string.try_delivery_with_question))
            }
        }

        if (viewingSlotsForBookedSlotAddress && BookSlotValidator().isTimeMatch(slotInfo, bookedStartAndEndTime)) {
            setupReservedGridItem(vh, slotInfo, slotStatusForVoiceOver)
        }
    }

    private fun setupExpressGridData(vh: BookSlotGridItemViewHolder, itemDataObject: SlotInfo, slotStatusForVoiceOver: StringBuilder) {
        setExpressCellView(vh)
        vh.gridExpressContainer?.visibility = View.VISIBLE
        customizeFontAppearance(
              textView = vh.gridPriceText,
              visibility = View.VISIBLE,
              text = context.resources.getString(R.string.pound_amount, itemDataObject.final_slot_price),
              textColorId = R.color.black,
              typefaceValue = Typeface.BOLD,
              isUnderlined = false
        )
        slotStatusForVoiceOver.append(context.getString(R.string.slot_available))
        setupNoAlcoholText(vh, itemDataObject, slotStatusForVoiceOver)
    }

    private fun setupReservedGridItem(vh: BookSlotGridItemViewHolder, slotInfo: SlotInfo, slotStatusForVoiceOver: StringBuilder) {
        customizeFontAppearance(vh.gridItemHeader, View.VISIBLE, context.resources.getString(R.string.reserved),
              R.color.white, Typeface.BOLD, false)
        customizeFontAppearance(
              textView = vh.gridPriceText,
              visibility = View.GONE,
              text = "",
              textColorId = R.color.white,
              typefaceValue = Typeface.BOLD,
              isUnderlined = false,
              shouldShowAsStrikeThrough = false
        )
        vh.gridItemParent.background = ContextCompat.getDrawable(context, R.drawable.bg_avocado_green_asda_promo_4_rounded_corners)
        if (isExpressEnabledInFragment(context, currentFragmentType) && slotInfo.is_express_slot) {
            vh.gridExpressSlot.visibility = View.GONE
            vh.gridReservedExpressSlot.visibility = View.VISIBLE
            vh.gridItemHeader.visibility = View.VISIBLE
        }
        slotStatusForVoiceOver.append(context.getString(R.string.slot_reserved))
        setupNoAlcoholText(vh, slotInfo, slotStatusForVoiceOver)
    }

    private fun setupNoAlcoholText(vh: BookSlotGridItemViewHolder, itemDataObject: SlotInfo, slotStatusForVoiceOver: StringBuilder) {
        if (itemDataObject.hasRestrictedItem) {
            vh.gridNoAlcoholText.visible()
            slotStatusForVoiceOver.append(" ").append(context.getString(R.string.no_alcohol))
            if (itemDataObject.status == SlotStatus.UNAVAILABLE.type) {
                vh.gridItemParent.alpha = 0.5f
            }
        }
    }

    private fun resetViews(vh: BookSlotGridItemViewHolder) {
        vh.gridItemParent.background = ContextCompat.getDrawable(context, R.drawable.bg_gray_asda_promo_4_rounded_corners)
        vh.gridExpressSlot.gone()
        vh.gridReservedExpressSlot.gone()
        vh.gridNoAlcoholText.gone()
        vh.gridExpressContainer?.alpha = 1f
        vh.gridItemParent.alpha = 1f
        vh.gridItemHeader.gone()
    }

    private fun setExpressCellView(view: BookSlotGridItemViewHolder, isUnavailable: Boolean = false) {
        view.gridExpressSlot.visibility = View.VISIBLE
        view.gridItemParent.background = ContextCompat.getDrawable(context, if (isUnavailable) {
            R.drawable.bg_express_slot_unavailable
        } else {
            R.drawable.bg_express_slot
        })
        view.gridItemHeader.visibility = View.GONE

    }

    override fun getItemCount(): Int {
        return data.size
    }

    override fun getItemViewType(position: Int): Int {
        return when {
            data[position] is String -> ITEM_TYPE_HEADER
            data[position] is SlotInfo -> ITEM_TYPE_SLOT
            else -> super.getItemViewType(position)
        }
    }

    private fun customizeFontAppearance(textView: TextView, visibility: Int, text: String,
                                        textColorId: Int, typefaceValue: Int, isUnderlined: Boolean,
                                        shouldShowAsStrikeThrough: Boolean = false) {
        textView.text = text
        textView.visibility = visibility
        textView.setTextColor(context.resources.getColor(textColorId))
        textView.setTextSize(TypedValue.COMPLEX_UNIT_PX, context.resources.getDimension(R.dimen.text_size_14dp))
        textView.setTypeface(null, typefaceValue)
        if (isUnderlined) {
            textView.paintFlags = textView.paintFlags or Paint.UNDERLINE_TEXT_FLAG
        } else {
            textView.paintFlags = textView.paintFlags and Paint.UNDERLINE_TEXT_FLAG.inv()
        }
        if (shouldShowAsStrikeThrough) {
            textView.paintFlags = textView.paintFlags or Paint.STRIKE_THRU_TEXT_FLAG
        } else {
            textView.paintFlags = textView.paintFlags and Paint.STRIKE_THRU_TEXT_FLAG.inv()
        }
    }

    fun setViewingSlotsForBookedSlotAddress(isSameAddress: Boolean) {
        viewingSlotsForBookedSlotAddress = isSameAddress
    }


    private fun isExpressEnabledInFragment(context: Context, currentFragmentType: String): Boolean {
        if ((currentFragmentType == BookSlotType.HOME_DELIVERY
                    && FeatureSettings.isExpressHDEnabled(context)
                    && Utils.isExpressEnabled(context))
              || (currentFragmentType == BookSlotType.CLICK_AND_COLLECT && Utils.isExpressEnabled(context))) {
            return true
        }
        return false
    }

    fun setBookedStartAndEndTime(bookedStartAndEndTime: Pair<String, String>) {
        this.bookedStartAndEndTime = bookedStartAndEndTime
    }
//------------------------------------- view holders ------------------------------------------

    private class BookSlotGridItemViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        internal val gridItemHeader: TextView = itemView.findViewById(R.id.book_slot_grid_item_header)
        internal val gridPriceText: TextView = itemView.findViewById(R.id.display_text)
        internal val gridItemParent: View = itemView.findViewById(R.id.book_slot_grid_item_parent)
        internal val gridExpressSlot: ImageView = itemView.findViewById(R.id.book_slot_grid_item_express_logo)
        internal val gridReservedExpressSlot: ImageView = itemView.findViewById(R.id.reserved_express_logo)
        internal val gridExpressContainer: LinearLayout? = itemView.findViewById(R.id.express_container)
        internal val gridNoAlcoholText: TextView = itemView.findViewById(R.id.no_alcohol_text_view)
    }

    private class BookSlotGridItemColumnHeaderViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        internal val itemText: TextView = itemView.findViewById(R.id.display_text)
    }
}

