package com.asda.android.app.bookslotv3.subscription

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import com.asda.android.R
import com.asda.android.base.core.ActivityRequestCodes
import com.asda.android.app.bookslotv3.view.activity.DeliveryImpactActivity
import com.asda.android.app.recurringslot.SlotSubscriptionManager
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.designlibrary.view.dialog.AsdaAlertDialog
import com.asda.android.restapi.service.data.recurringslots.SlotSubscription
import com.asda.android.restapi.service.data.recurringslots.SubscriptionSlotStatus
import com.asda.android.restapi.service.data.recurringslots.SubscriptionsResponse
import com.asda.android.singleprofile.base.SingleProfile
import kotlinx.android.synthetic.main.edit_subscriptions_fragment.*

class EditSubscriptionsFragment : BaseFragment(), EditSubscriptionsListener {

    lateinit var viewModel: EditSubscriptionsViewModel
    lateinit var adapter: EditSubscriptionsAdapter

    companion object {
        const val TAG = "EditSubscriptionsFragment"
    }

    override fun getInternalTag(): String? {
        return TAG
    }

    override fun hasActionBar(): Boolean {
        return true
    }

    override fun getActionBarTitle(): String? {
        return context?.getString(R.string.recurring_slot)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        viewModel = ViewModelProviders.of(this).get(EditSubscriptionsViewModel::class.java)
        setupActionBar()

    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.edit_subscriptions_fragment, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initializeRecyclerView()

        setupGetSubscriptionCallObserver()
        getSubscriptions()
        setupSkipSlotObserver()
        setupCancelSlotObserver()
    }

    private fun initializeRecyclerView() {
        context?.let {
            adapter = EditSubscriptionsAdapter(it, this, viewModel.slotSubscriptions)
            edit_subscriptions_recyclerview.adapter = adapter
        }
    }

    //GET BLOCK-------------------------------------------------------------------------------------
    private fun getSubscriptions() {
        edit_subscriptions_loader.visibility = View.VISIBLE
        viewModel.getSubscriptions(SingleProfile.getProfileResponse()?.profileId ?: "", true)
    }

    private fun setupGetSubscriptionCallObserver() {
        viewModel.liveDataSubscriptionResponse.observe(this, Observer<SubscriptionsResponse> { subscriptionResponse ->
            edit_subscriptions_loader.visibility = View.GONE
            if (subscriptionResponse.errors?.isEmpty() == false) {
                showErrorDialogForGetData()
            } else {
                adapter.notifyDataSetChanged()
            }
        })
    }

    private fun showErrorDialogForGetData() {
        val asdaAlertDialog = AsdaAlertDialog.newInstance(
              leftButtonText = context?.getString(R.string.retry).orEmpty(),
              rightButtonText = context?.getString(R.string.ok).orEmpty(),
              message = context?.getString(R.string.error_get_subscriptions).orEmpty())
        asdaAlertDialog.onLeftButtonClick {
            getSubscriptions()
            asdaAlertDialog.dismiss()
        }
        asdaAlertDialog.onRightButtonClick { asdaAlertDialog.dismiss() }
        fragmentManager?.let {
            asdaAlertDialog.show(it, TAG)
        }
    }

    //SKIP BLOCK -----------------------------------------------------------------------------------
    override fun onSubscriptionSkip(skipId: String) {
        edit_subscriptions_loader.visibility = View.VISIBLE
        viewModel.onSubscriptionSkip(skipId)
    }

    private fun setupSkipSlotObserver() {
        viewModel.skipSubscriptionLiveData.observe(this, Observer { subscriptionResponse ->

            edit_subscriptions_loader.visibility = View.GONE

            if (subscriptionResponse.errors?.isEmpty() == false) {
                onSkipSlotError()
            } else {
                onSkipSlotSuccess()
            }
        })
    }

    private fun onSkipSlotError() {
        showErrorDialog(context?.getString(R.string.error_for_skip_slot))
    }

    private fun showErrorDialog(message: String?) {
        val asdaAlertDialog = AsdaAlertDialog.newInstance(
              rightButtonText = context?.getString(R.string.ok).orEmpty(),
              message = message.orEmpty())
        asdaAlertDialog.onRightButtonClick { asdaAlertDialog.dismiss() }
        fragmentManager?.let {
            asdaAlertDialog.show(it, TAG)
        }
    }

    private fun onSkipSlotSuccess() {
        refreshCart()
        SlotSubscriptionManager.refreshSubscriptionEligibility()
        val asdaAlertDialog = AsdaAlertDialog.newInstance(
              rightButtonText = context?.getString(R.string.ok).orEmpty(),
              message = context?.getString(R.string.slot_skipped).orEmpty())
        asdaAlertDialog.onRightButtonClick { asdaAlertDialog.dismiss() }
        fragmentManager?.let {
            asdaAlertDialog.show(it, TAG)
        }
    }

    private fun refreshCart() {
        com.asda.android.cxo.CartManager.instance?.refreshCart()
    }

    //CANCEL BLOCK ---------------------------------------------------------------------------------
    override fun onSubscriptionCancel(subscription: SlotSubscription) {
        edit_subscriptions_loader.visibility = View.VISIBLE
        subscription.subscription_info?.status = SubscriptionSlotStatus.CANCEL_INITIATED.name
        viewModel.onSubscriptionCancel(subscription)
    }


    private fun setupCancelSlotObserver() {
        viewModel.cancelSubscriptionLiveData.observe(this, Observer { subscriptionResponse ->

            edit_subscriptions_loader.visibility = View.GONE

            if (subscriptionResponse.errors?.isEmpty() == false) {
                onCancelSlotError()
            } else {
                onCancelSlotSuccess()
            }
        })
    }

    private fun onCancelSlotSuccess() {
        refreshCart()
        SlotSubscriptionManager.refreshSubscriptionEligibility()
        if (activity is DeliveryImpactActivity && activity != null) {
            activity?.setResult(ActivityRequestCodes.RESULT_CODE_RECURRING_SLOT_SUCCESS)
            activity?.finish()
        } else
            pop()
    }

    private fun onCancelSlotError() {
        showErrorDialog(context?.getString(R.string.error_for_cancel_slot))
    }

}

