package com.asda.android.app.cart

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import androidx.fragment.app.FragmentPagerAdapter
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import androidx.viewpager.widget.ViewPager
import com.asda.android.R
import com.asda.android.analytics.info.BYGPageViewInfo
import com.asda.android.app.base.FragmentConfig
import com.asda.android.app.cart.CartFragment.OnFragmentOpenedListener
import com.asda.android.app.cart.model.APIErrorEventModel
import com.asda.android.app.cart.model.P13NInfoModel
import com.asda.android.app.cart.model.PageEventModel
import com.asda.android.app.cart.viewmodel.CartViewModel
import com.asda.android.app.dynamic.DynamicComponent
import com.asda.android.app.dynamic.DynamicPageFragment
import com.asda.android.app.dynamic.PersonalizationComponent
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.shop.MerchandisingHandler
import com.asda.android.app.shop.ShopFragment
import com.asda.android.app.shop.productdetail.view.ProductManager
import com.asda.android.app.view.WrapperPresenter
import com.asda.android.base.core.response.BaseStateResponse
import com.asda.android.base.core.statemanagement.type.State
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.base.core.view.fragments.PresenterFragment
import com.asda.android.base.core.view.ui.DynamicHeightViewPager
import com.asda.android.base.core.view.ui.PresenterStack
import com.asda.android.base.interfaces.ICartManager
import com.asda.android.products.ui.AsdaProductsConfig
import com.asda.android.recipes.view.fragments.RecipesTrolleyFragment
import com.asda.android.restapi.app.shop.PromotionInfo
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.service.data.Merchandising
import com.asda.android.restapi.service.data.direct.PersonalizationResponse
import com.asda.android.restapi.service.data.direct.PersonalizationResponseV2
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.CartManager
import com.asda.recipes.RecipeActivity
import com.facebook.litho.ComponentContext
import com.facebook.litho.LithoView
import com.facebook.litho.widget.Recycler
import com.facebook.litho.widget.RecyclerBinder
import com.google.android.material.tabs.TabLayout


class CartPagerFragment : BaseFragment(), OnFragmentOpenedListener {

    private val TAG = "CartPagerFragment"
    private var adapter: ViewPagerAdapter? = null
    private var viewPager: androidx.viewpager.widget.ViewPager? = null
    private var viewModel: CartViewModel? = null
    private lateinit var mLithoView: LithoView
    private var mComponentContext: ComponentContext? = null
    private var mRecyclerBinder: RecyclerBinder? = null
    private lateinit var mOffersLithoView: LithoView

    private val pageEventModel = PageEventModel()
    private val cartListener = object : ICartManager.CartListener() {
        override fun onChanged() {
            if ((Authentication.getInstance() as Authentication).isLoggedIn) {
                adapter?.refreshTrolleyFragment()
            } else {
                viewPager?.setCurrentItem(0, false)
            }
        }
    }

    override fun getInternalTag(): String? = TAG

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        CartManager.get(context).addCartListener(cartListener)
        viewModel = ViewModelProviders.of(activity!!).get(CartViewModel::class.java)

    }

    override fun onDestroy() {
        CartManager.get(context).removeCartListener(cartListener)
        super.onDestroy()
        if (mComponentContext != null) {
            mComponentContext = null
        }
        if (mRecyclerBinder != null) {
            mRecyclerBinder = null
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        val containerView = inflater.inflate(R.layout.cart, container, false)

        (containerView?.findViewById(R.id.pager) as ViewPager?)?.let { pager ->
            fragmentManager?.let { fragmentManager ->
                context?.let {
                    val tabLayout = containerView?.findViewById(R.id.sliding_tabs) as TabLayout?
                    adapter = ViewPagerAdapter(fragmentManager, it)
                    pager.adapter = adapter
                    viewPager = pager
                    tabLayout?.setupWithViewPager(pager)
                    tabLayout?.visibility = View.VISIBLE
                    adapter?.setOnCartFragmentOpenedListener(this)
                }
            }
        }

        mLithoView = containerView.findViewById(R.id.cart_p13n_lithoview)
        mOffersLithoView = containerView.findViewById(R.id.cart_offers_lithoview)
        return containerView
    }

    fun setCartFragmentListener(listener: CartFragmentListener) {
        adapter?.setCartFragmentListener(listener)
    }

    private fun makePersonalizationCall() {
        viewModel?.invokePersonalizationApi()
        viewModel?.invokePersonalizationOffersApi()
    }

    private fun initContent() {
        if (mComponentContext == null) {
            mComponentContext = ComponentContext(context)
        }
        if (mRecyclerBinder == null) {
            mRecyclerBinder = RecyclerBinder.Builder().build(mComponentContext)
        }

        val recyclerComponent = Recycler.create(mComponentContext)
                  .binder(mRecyclerBinder)
                  .nestedScrollingEnabled(false)
                  .build()

        mLithoView.setComponent(recyclerComponent)
        mOffersLithoView.setComponent(recyclerComponent)

    }

    private val mActionHandler = object : DynamicPageFragment.OnActionHandler {
        override fun onFragmentPushed(fragment: BaseFragment, breadCrumb: String) {
            /**
             * This method can be used to put functionality that is to be invoked when a fragment is pushed.
             */
        }

        override fun onSimilarClicked(item: ShelfListItem) {
            /**
             * This method can be used to put functionality that is to be invoked when similar fragment is clicked.
             */
        }

        override fun onMerchandisingClicked(banner: Merchandising.Banner, view: ImageView) {
            try {
                MerchandisingHandler.handleMerchandisingEvent(activity,
                      getCurrentFragmentManager(),
                      banner,
                      view,
                      banner.getTrackingCode())
            } catch (ignored: IllegalArgumentException) {
                Log.e(TAG, ignored.message)
            }

        }

        override fun onDynamicPageFragmentPushed(fragment: DynamicPageFragment) {
            push(fragment)
        }

        override fun onShelfListItemClicked(shelfItem: ShelfListItem) {
            context?.let {
                AsdaProductsConfig.productManager.launchProductDetail(null, it, shelfItem, false, false, true, true, null)
            }
        }

        override fun onPromotionItemClicked(promotionInfo: PromotionInfo) {

            if (context is AsdaActivity) {
                val activity = context as AsdaActivity
                val fragmentSwitcher = activity.fragmentSwitcher
                fragmentSwitcher?.let{
                    val activeFragment = it.topFragment
                    var currentPresenterStack: PresenterStack? = null
                    if (activeFragment is PresenterFragment) {
                        currentPresenterStack = activeFragment.presenterStack
                    }
                    if (currentPresenterStack != null) {
                        activity.drawerToggle.closeCart(false)
                        val linkSaveFragment = ProductManager.getLinkSaveFragment(activity, promotionInfo.promotionId, promotionInfo.displayText)
                        val wrapperPresenter = WrapperPresenter(activity, linkSaveFragment)
                        wrapperPresenter.markAsSecondaryTask()
                        currentPresenterStack.pushPresenter(wrapperPresenter, false, promotionInfo.displayText)
                    } else {
                        activity.drawerToggle.closeCart(false)
                        val bundle = Bundle()
                        bundle.putSerializable(ShopFragment.ARG_PROMOTION, promotionInfo.data)
                        it.switchToFragment(FragmentConfig.TAG_SHOP, bundle)
                    }
                }
            }
        }

    }

    private fun updatePersonalizationContents() {
        mLithoView.setVisibility(View.GONE)
        viewModel?.getPersonalizationResponseLiveData()?.observe(this, Observer { response ->

            if (response.getState() == State.SUCCESS) {
                response.getData()?.let {
                    val personalizationResponse: PersonalizationResponse = response.getData()!!

                    val pageInfo = viewModel?.p13nInfoFormatter?.format(personalizationResponse)
                    pageInfo?.let {
                        if (pageInfo.isNotEmpty()) {
                            val viewInfo = viewModel?.getPersonalizationInfo(personalizationResponse)
                            val component = PersonalizationComponent.create(mComponentContext)
                                  .handler(mActionHandler)
                                  .responseData(personalizationResponse)
                                  .viewInfo(viewInfo)
                                  .build()

                            mRecyclerBinder?.insertItemAt(0, DynamicComponent.create(mComponentContext).removePadding(true).build())

                            if (mRecyclerBinder?.itemCount == 0) {
                                mRecyclerBinder?.updateItemAt(0, component)
                            } else {
                                mLithoView.setComponentAsync(component)
                            }
                            mLithoView.setVisibility(View.VISIBLE)
                            trackP13NImpressionEvent(pageInfo)
                        }
                    }
                    updateUIForError(personalizationResponse, mLithoView)
                    trackAPIErrorEvent(response)
                }
                response.setState(State.INIT)
                response.getError()?.let {
                    mLithoView.setVisibility(View.GONE)
                }
            }
        })
    }

    private fun updateOffersPersonalizationContents() {
        mOffersLithoView.setVisibility(View.GONE)
        viewModel?.getOffersResponseLiveData()?.observe(this, Observer { response ->

            if(response.getState() == State.SUCCESS) {
                response.getData()?.let {
                    val personalizationResponse: PersonalizationResponse = response.getData()!!

                    val pageInfo = viewModel?.p13nInfoFormatter?.format(personalizationResponse)
                    pageInfo?.let {
                        if (pageInfo.isNotEmpty()) {
                            val viewInfo = viewModel?.getPersonalizationInfo(personalizationResponse)
                            val component = PersonalizationComponent.create(mComponentContext)
                                  .handler(mActionHandler)
                                  .responseData(personalizationResponse)
                                  .viewInfo(viewInfo)
                                  .build()

                            mRecyclerBinder?.insertItemAt(0, DynamicComponent.create(mComponentContext).removePadding(true).build())
                            if (mRecyclerBinder?.itemCount == 0) {
                                mRecyclerBinder?.updateItemAt(0, component)

                            } else {
                                mOffersLithoView.setComponentAsync(component)
                            }
                            mOffersLithoView.setVisibility(View.VISIBLE)
                            trackP13NImpressionEvent(pageInfo)
                        }
                    }

                    updateUIForError(personalizationResponse, mOffersLithoView)
                    trackAPIErrorEvent(response)
                }
                response.setState(State.INIT)
                response.getError()?.let {
                    mOffersLithoView.setVisibility(View.GONE)
                }
            }
        })

    }

    private fun updateUIForError(personalizationResponse: PersonalizationResponse, lithoView: LithoView) {
        personalizationResponse.errors?.let {
            if (it.isNotEmpty()) {
                lithoView.visibility = View.GONE
            }
        }
    }

    private fun trackP13NImpressionEvent(pageViewInfo: Array<BYGPageViewInfo?>) {
        pageEventModel.setHasPersonalization(true)
        pageEventModel.setInfo(pageViewInfo)
        viewModel?.execute(P13NInfoModel(pageEventModel))
    }

    private fun trackAPIErrorEvent(response:BaseStateResponse<PersonalizationResponse>) {
        val personalizationResponse = response.getData()!!
        personalizationResponse.errors?.let {
            if (it.isNotEmpty()) {
                val error = it.get(0)
                viewModel?.execute(APIErrorEventModel(pageEventModel, AsdaServiceFactory.get().getPersonalizationAPIEndPoint(
                      if (personalizationResponse.isFromV2Api) PersonalizationResponseV2()
                      else response.getData()),
                      error.errorCode, error.errorMessage))
            }
        }
    }

    override fun onFragmentOpened(opened: Boolean) {
        mLithoView.visibility = View.GONE
        mOffersLithoView.visibility = View.GONE
        if (opened) {
            context?.let {
                if (viewModel?.cartCampaignValidator?.validate(it) == true) {
                    initContent()
                    makePersonalizationCall()
                    updatePersonalizationContents()
                    updateOffersPersonalizationContents()
                }
            }
        }
    }

}

private class ViewPagerAdapter(manager: FragmentManager, val context: Context) : FragmentPagerAdapter(manager) {
    private val TAB_COUNT = 2
    private var listener: CartFragmentListener? = null
    private var trolleyFragment: RecipesTrolleyFragment? = null
    private var onFragmentOpenedlistener: CartFragment.OnFragmentOpenedListener? = null
    private var currentPosition = -1

    override fun getCount() = TAB_COUNT

    override fun getItem(position: Int): androidx.fragment.app.Fragment = when (position) {
        0 -> CartFragment()
        1 -> RecipesTrolleyFragment().apply {
            onBrowseRecipeClicked = { startActivity(Intent(context, RecipeActivity::class.java)) }
        }
        else -> Fragment()
    }

    override fun getPageTitle(position: Int): CharSequence = when (position) {
        0 -> context.getString(R.string.cart_tab_departments)
        1 -> context.getString(R.string.cart_tab_recipes)
        else -> ""
    }

    override fun instantiateItem(container: ViewGroup, position: Int): Any {
        val fragment = super.instantiateItem(container, position)
        if (fragment is CartFragment) {
            listener?.onCartFragmentCreated(fragment)
            fragment.setOnFragmentOpenedListener(onFragmentOpenedlistener)
        }
        if (fragment is RecipesTrolleyFragment) {
            trolleyFragment = fragment
        }
        return fragment
    }

    fun setCartFragmentListener(listener: CartFragmentListener) {
        this.listener = listener
    }

    fun setOnCartFragmentOpenedListener(listener: OnFragmentOpenedListener) {
         onFragmentOpenedlistener = listener;
    }
    fun refreshTrolleyFragment() {
        trolleyFragment?.refreshItems()
    }

    override fun setPrimaryItem(container: ViewGroup, position: Int, `object`: Any) {
        super.setPrimaryItem(container, position, `object`)
        if (position != currentPosition) {
            var fragment: Fragment? = null
            when(position) {
                0-> {fragment = `object` as CartFragment }
                1-> {fragment = `object` as RecipesTrolleyFragment}
            }
            if (container is DynamicHeightViewPager) {
               fragment?.view?.let {
                   currentPosition = position
                   container.measureCurrentView(it)
               }
            }

        }
    }
}

interface CartFragmentListener {
    fun onCartFragmentCreated(fragment: CartFragment)
}
