package com.asda.android.app.dp.adapters

import android.content.Context
import android.text.SpannableString
import android.text.SpannableStringBuilder
import android.text.TextUtils
import android.text.style.StrikethroughSpan
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.app.dp.formatter.DPListSortFormatter
import com.asda.android.app.dp.formatter.DPPromoEndDateFormatter
import com.asda.android.app.dp.model.DeliveryPassItemModel
import com.asda.android.app.dp.parser.DeliveryPassAdapterItemParser
import com.asda.android.base.core.view.adapter.HeaderRecyclerViewAdapter
import com.asda.android.restapi.service.data.DeliveryPassList
import com.asda.android.service.base.Authentication
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtil

class DeliveryPassListAdapter(val callback: (String?, DeliveryPassItemModel?) -> Unit) :
    HeaderRecyclerViewAdapter() {

    private var items: Array<DeliveryPassItemModel>? = null
    private var mDPOfferId: String? = null
    private var mDPOfferPromoPrice: String? = null
    private var mDPOfferWasPrice: String? = null
    private val dpListSortFormatter: DPListSortFormatter by lazy { DPListSortFormatter() }
    private val deliveryPassAdapterItemParser: DeliveryPassAdapterItemParser by lazy { DeliveryPassAdapterItemParser() }
    private val dpPromoEndDateFormatter: DPPromoEndDateFormatter by lazy { DPPromoEndDateFormatter() }

    companion object {
        private const val TYPE_ITEM = 1
    }


    fun updateData(deliveryPassList: DeliveryPassList, headerView: View, footerView: View) {
        items = deliveryPassAdapterItemParser.parse(deliveryPassList)
        dpListSortFormatter.format(items)
        updateHeaderView(headerView)
        updateFooterView(footerView)
        notifyDataSetChanged()
    }

    private fun updateFooterView(footerView: View) {
        val view = footerView
        if ((Authentication.getInstance() as Authentication).hasDPOffer()) {
            val offer = (Authentication.getInstance() as Authentication).deliveryPassOffers
            offer?.let {
                if (it.isNotEmpty() && it[0]?.sku?.isNotEmpty() == true) {
                    val firstDP = it[0]
                    mDPOfferId = firstDP.sku
                    mDPOfferPromoPrice = firstDP.promoPrice
                    mDPOfferWasPrice = firstDP.originalPrice
                    val exclusiveOfferExpiration =
                        view.findViewById<View>(R.id.exclusive_offer_expiration) as TextView
                    exclusiveOfferExpiration.text = exclusiveOfferExpiration
                        .context
                        .getString(
                            R.string.dp_exclusive_offer_expire_list,
                            dpPromoEndDateFormatter.format(firstDP.promoEndDate)
                        )
                    exclusiveOfferExpiration.visibility = View.VISIBLE
                }
            }
        }
        setFooterView(view)
    }

    private fun updateHeaderView(headerView: View) {
        addHeaderView(headerView)
    }

    override fun getItemViewType(position: Int): Int {
        if (isHeaderPosition(position)) {
            return TYPE_LIST_HEADERS - position
        } else if (isFooterPosition(position)) {
            return TYPE_LIST_FOOTER
        }
        return TYPE_ITEM
    }

    override fun getItemCount(): Int {
        var count = super.getItemCount()
        count += items?.size ?: 0
        return count
    }

    fun getItem(position: Int): DeliveryPassItemModel? {
        items?.let {
            return it[position]
        } ?: run { return null }
    }

    override fun getItemId(position: Int): Long {
        return 0
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder? {
        val viewHolder = super.onCreateViewHolder(parent, viewType)
        if (viewHolder != null) {
            return viewHolder
        }

        val holder = DeliveryPassViewHolder(
            ViewUtil.inflate(parent.context, R.layout.delivery_pass_list_item, parent),
            TYPE_ITEM
        )

        holder.oneTimePrice.setOnClickListener { v ->
            val position = translatePositionToBase(holder.adapterPosition)
            val deliveryPass = getItem(position)
            deliveryPass?.let {
                callback.invoke(it.oneTimeId, deliveryPass)
            }
        }
        holder.monthlyPrice.setOnClickListener { v ->
            val position = translatePositionToBase(holder.adapterPosition)
            val deliveryPass = getItem(position)
            deliveryPass?.let {
                callback.invoke(it.monthlyId, deliveryPass)

            }
        }
        return holder
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, recyclerPosition: Int) {
        if ((holder as BaseViewHolder).viewType == TYPE_ITEM) {
            val viewHolder = holder as DeliveryPassViewHolder
            val position = translatePositionToBase(recyclerPosition)
            val deliveryPass = getItem(position)

            deliveryPass?.let {
                viewHolder.txtDPName.text = it.name.orEmpty()

                viewHolder.txtMidWeekDays.visibility =
                    if (it.weekUsage!!.isEmpty()) View.GONE else View.VISIBLE

                viewHolder.txtMidWeekDays.text = it.weekUsage

                viewHolder.txtDuration.text = viewHolder.txtDuration.context.resources
                    .getQuantityString(R.plurals.n_months, it.durationMonths, it.durationMonths)

                viewHolder.txtMinSpend.text = viewHolder.txtMinSpend.context
                    .getString(R.string.dp_min_spend, it.dpMinSpendAmt)

                setMonthlyDetailsData(viewHolder, it)

                setOneTimeDetailsData(viewHolder, it)
            }
        }
    }

    private fun setOneTimeDetailsData(
        viewHolder: DeliveryPassViewHolder,
        itemModel: DeliveryPassItemModel
    ) {
        val oneTimeEmpty = TextUtils.isEmpty(itemModel.oneTimePrice)

        viewHolder.oneTimeDash.visibility = if (oneTimeEmpty) View.VISIBLE else View.GONE

        viewHolder.oneTimeFreeTrial.visibility =
            if (!oneTimeEmpty && itemModel.isOneTimeFreeTrial) View.VISIBLE else View.GONE

        viewHolder.oneTimePrice.visibility = if (oneTimeEmpty) View.GONE else View.VISIBLE

        viewHolder.oneTimePrice.text = Utils.addPoundIfNecessary(itemModel.oneTimePrice)

        val hasOneTimeOffer = itemModel.oneTimeId != null && itemModel.oneTimeId == mDPOfferId

        viewHolder.oneTimeExclusive.visibility =
            if (!itemModel.isOneTimeFreeTrial && hasOneTimeOffer) View.VISIBLE else View.GONE

        viewHolder.oneTimeOffer.visibility =
            if (!itemModel.isOneTimeFreeTrial && hasOneTimeOffer) View.VISIBLE else View.GONE

        if (hasOneTimeOffer) {
            viewHolder.oneTimeOffer.text = getStrikeThruPrice(viewHolder.oneTimeOffer.context)
            viewHolder.oneTimePrice.text = Utils.addPoundIfNecessary(mDPOfferPromoPrice)
        }
    }

    private fun setMonthlyDetailsData(
        viewHolder: DeliveryPassViewHolder,
        itemModel: DeliveryPassItemModel
    ) {
        val monthlyEmpty = TextUtils.isEmpty(itemModel.monthlyPrice)

        viewHolder.monthlyDash.visibility = if (monthlyEmpty) View.VISIBLE else View.GONE

        viewHolder.monthlyFreeTrial.visibility =
            if (!monthlyEmpty && itemModel.isMonthlyFreeTrial) View.VISIBLE else View.GONE

        viewHolder.monthlyPrice.visibility = if (monthlyEmpty) View.GONE else View.VISIBLE

        viewHolder.monthlyPrice.text =
            if (TextUtils.isEmpty(itemModel.monthlyPrice)) "- -" else Utils.addPoundIfNecessary(
                itemModel.monthlyPrice
            )

        val hasMonthlyOffer = itemModel.monthlyId != null && itemModel.monthlyId == mDPOfferId

        viewHolder.monthlyExclusive.visibility =
            if (!itemModel.isMonthlyFreeTrial && hasMonthlyOffer) View.VISIBLE else View.GONE

        viewHolder.monthlyOffer.visibility =
            if (!itemModel.isMonthlyFreeTrial && hasMonthlyOffer) View.VISIBLE else View.GONE

        if (hasMonthlyOffer) {
            viewHolder.monthlyOffer.text = getStrikeThruPrice(viewHolder.monthlyOffer.context)
            viewHolder.monthlyPrice.text = Utils.addPoundIfNecessary(mDPOfferPromoPrice)
        }
    }

    private fun getStrikeThruPrice(context: Context): CharSequence? {
        val wasPriceStyled =
            SpannableString(
                context.getString(
                    R.string.dp_was,
                    Utils.addPoundIfNecessary(mDPOfferWasPrice)
                )
            )
        wasPriceStyled.setSpan(
            StrikethroughSpan(),
            4,
            wasPriceStyled.length,
            SpannableStringBuilder.SPAN_INCLUSIVE_INCLUSIVE
        )
        return wasPriceStyled
    }

    private inner class DeliveryPassViewHolder constructor(itemView: View, viewType: Int) :
        BaseViewHolder(itemView, viewType) {
        val txtDPName: TextView = itemView.findViewById<View>(R.id.txt_dp_name) as TextView
        val txtMidWeekDays: TextView =
            itemView.findViewById<View>(R.id.txt_midweek_days) as TextView
        val txtDuration: TextView = itemView.findViewById<View>(R.id.txt_duration) as TextView
        val txtMinSpend: TextView = itemView.findViewById<View>(R.id.txt_minimum_spend) as TextView
        val monthlyDash: TextView = itemView.findViewById<View>(R.id.monthly_price_dash) as TextView
        val monthlyPrice: TextView =
            itemView.findViewById<View>(R.id.monthly_price_button) as TextView
        val monthlyFreeTrial: TextView =
            itemView.findViewById<View>(R.id.monthly_free_trial) as TextView
        val monthlyExclusive: TextView =
            itemView.findViewById<View>(R.id.monthly_exclusive) as TextView
        val monthlyOffer: TextView = itemView.findViewById<View>(R.id.monthly_offer) as TextView
        val oneTimeDash: TextView = itemView.findViewById<View>(R.id.onetime_price_dash) as TextView
        val oneTimePrice: TextView =
            itemView.findViewById<View>(R.id.onetime_price_button) as TextView
        val oneTimeFreeTrial: TextView =
            itemView.findViewById<View>(R.id.onetime_free_trial) as TextView
        val oneTimeExclusive: TextView =
            itemView.findViewById<View>(R.id.onetime_exclusive) as TextView
        val oneTimeOffer: TextView = itemView.findViewById<View>(R.id.onetime_offer) as TextView


    }
}
