package com.asda.android.app.dp.parser

import android.util.Log
import androidx.annotation.VisibleForTesting
import com.asda.android.app.dp.model.DeliveryPassItemModel
import com.asda.android.base.core.parser.IParser
import com.asda.android.restapi.service.base.IAuthentication
import com.asda.android.restapi.service.data.DeliveryPass
import com.asda.android.restapi.service.data.DeliveryPassList
import com.asda.android.service.base.Authentication
import com.asda.android.utils.ISO8601Duration
import java.util.*

class DeliveryPassAdapterItemParser : IParser<DeliveryPassList?, Array<DeliveryPassItemModel>?> {

    companion object {
        const val TAG = "DPAdapterItemFormatter"
    }

    override fun parse(input: DeliveryPassList?): Array<DeliveryPassItemModel>? {
        var result: Array<DeliveryPassItemModel>? = null
        input?.deliveryPassPromotions?.let { deliveryPassPromotions ->
            val list = ArrayList<DeliveryPassItemModel>()
            val userFreeTrialEligible = getAuthentication().isDPFreeTrialEligible
            for (deliveryPass in deliveryPassPromotions) {
                try {
                    val installments =
                        getInstallments(deliveryPass)

                    val durationMonths = getDurationInMonths(deliveryPass, installments)
                    val model = findSimilar(list, durationMonths, deliveryPass)
                    if (model == null) {
                        createDeliveryPassItemModel(
                            list,
                            deliveryPass,
                            durationMonths,
                            userFreeTrialEligible
                        )
                    } else {
                        updateExistingModelData(
                            list, deliveryPass, durationMonths, installments,
                            userFreeTrialEligible, model
                        )
                    }
                } catch (exception: Exception) {
                    Log.w(TAG, "Error parsing DP", exception)
                }

            }
            result = list.toTypedArray()
        }

        return result
    }

    private fun updateExistingModelData(
        list: ArrayList<DeliveryPassItemModel>,
        deliveryPass: DeliveryPass,
        durationMonths: Int,
        installments: Int,
        userFreeTrialEligible: Boolean,
        model: DeliveryPassItemModel
    ) {
        val hasInstallments = installments > 0
        if (hasInstallments && model.monthlyId.isNullOrEmpty()) {
            model.monthlyPrice = deliveryPass.price
            model.monthlyId = deliveryPass.dpSkuId
            model.isMonthlyFreeTrial =
                deliveryPass.isFreeTrialEligible && userFreeTrialEligible && getAuthentication().isDPFreeTrialSku(
                    deliveryPass.dpSkuId
                )
            model.isInstallmentType = true
        } else if (!hasInstallments && model.oneTimeId.isNullOrEmpty()) {
            model.oneTimePrice = deliveryPass.price
            model.oneTimeId = deliveryPass.dpSkuId
            model.isOneTimeFreeTrial =
                deliveryPass.isFreeTrialEligible && userFreeTrialEligible && getAuthentication().isDPFreeTrialSku(
                    deliveryPass.dpSkuId
                )
            model.isInstallmentType = false
        } else {
            // If got here, handle as a different pass
            createDeliveryPassItemModel(list, deliveryPass, durationMonths, false)
        }
    }

    private fun getDurationInMonths(deliveryPass: DeliveryPass?, installments: Int): Int {
        return (ISO8601Duration(deliveryPass?.passDuration).months
              * if (installments != 0) installments else 1)
            .toInt()
    }

    private fun getInstallments(deliveryPass: DeliveryPass?): Int {
        return if (deliveryPass?.totalInstallments.isNullOrEmpty()) 0
        else Integer.parseInt(deliveryPass?.totalInstallments!!)
    }

    @VisibleForTesting
    fun findSimilar(
        list: ArrayList<DeliveryPassItemModel>,
        durationMonths: Int,
        deliveryPass: DeliveryPass
    ): DeliveryPassItemModel? {
        var similar: DeliveryPassItemModel? = null
        if (list.isNotEmpty()) {
            for (model in list) {
                if (model.durationMonths == durationMonths && model.dpMinSpendAmt.equals(
                        deliveryPass.dpMinSpendAmt,
                        true
                    ) &&
                    model.isMidWeekDp.equals(
                        deliveryPass.isMidWeekDP,
                        true
                    ) && model.isDPPlus == deliveryPass.isDPPlus &&
                    model.midWeekDPDays.equals(deliveryPass.midWeekDPDays, true)
                ) {
                    similar = model
                    break
                }
            }
        }
        return similar
    }

    @VisibleForTesting
    fun createDeliveryPassItemModel(
        list: ArrayList<DeliveryPassItemModel>,
        deliveryPass: DeliveryPass,
        durationMonths: Int,
        isUserFreeTrialEligible: Boolean
    ) {
        val model: DeliveryPassItemModel = DeliveryPassItemModel()
        model.name = getNameForDP(deliveryPass)
        model.desc = deliveryPass.description
        model.dpMinSpendAmt = deliveryPass.dpMinSpendAmt
        model.isMidWeekDp = deliveryPass.isMidWeekDP
        model.midWeekDPDays = deliveryPass.midWeekDPDays
        model.isDPPlus = deliveryPass.isDPPlus

        model.durationMonths = durationMonths

        if ("true".equals(deliveryPass.isMidWeekDP, ignoreCase = true)) {
            model.weekUsage = convertToUserFriendlyMidWeek(deliveryPass.midWeekDPDays)
        } else {
            model.weekUsage = ""
        }

        model.isInstallmentType =
            deliveryPass.totalInstallments?.isNotEmpty() == true && Integer.parseInt(deliveryPass.totalInstallments!!) > 0

        if (model.isInstallmentType) {
            model.monthlyPrice = deliveryPass.price
            model.isMonthlyFreeTrial = isUserFreeTrialEligible && deliveryPass.isFreeTrialEligible
            model.monthlyId = deliveryPass.dpSkuId
        } else {
            model.oneTimePrice = deliveryPass.price
            model.isOneTimeFreeTrial = isUserFreeTrialEligible && deliveryPass.isFreeTrialEligible
            model.oneTimeId = deliveryPass.dpSkuId
        }


        list.add(model)
    }

    @VisibleForTesting
    fun getNameForDP(deliveryPass: DeliveryPass): String? {
        if (deliveryPass.name == null) return null

        return when {
            deliveryPass.isDPPlus -> "Delivery Pass Plus"
            deliveryPass.name!!.contains("Anytime") -> "Anytime Pass"
            deliveryPass.name!!.contains("Midweek") -> "Mid-week Pass"
            else -> deliveryPass.name
        }
    }


    @VisibleForTesting
    fun convertToUserFriendlyMidWeek(midWeekDPDays: String?): String? {
        if ("TUE,WED,THU" == midWeekDPDays) {
            return "(Tues-Thur)"
        }
        return midWeekDPDays
    }

    @VisibleForTesting
    fun getAuthentication(): IAuthentication {
        return Authentication.getInstance()
    }

}
