package com.asda.android.app.dp.viewmodel


import android.app.Application
import android.content.Context
import android.text.SpannableString
import android.text.SpannableStringBuilder
import android.text.Spanned
import android.text.TextUtils
import android.text.style.StrikethroughSpan
import android.util.Log
import androidx.annotation.VisibleForTesting
import androidx.core.text.HtmlCompat
import androidx.databinding.Bindable
import androidx.databinding.Observable
import androidx.databinding.ObservableField
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.asda.android.R
import com.asda.android.app.account.UserAccountDetails
import com.asda.android.app.dp.OneTimeLiveDataEvent
import com.asda.android.app.dp.actionstates.DPCheckoutLoadContentState
import com.asda.android.app.dp.actionstates.DPCheckoutState
import com.asda.android.app.dp.actionstates.DPConfirmOrderState
import com.asda.android.app.dp.callbacks.CardinalInitCallbackImpl
import com.asda.android.app.dp.model.DeliveryPassItemModel
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.restapi.service.NetworkCallback
import com.asda.android.restapi.service.base.RxFailure
import com.asda.android.restapi.service.data.*
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.three_ds.ThreeDsUtils
import com.asda.android.three_ds.helper.ThreeDS
import com.asda.android.utils.Utils
import io.reactivex.ObservableSource
import pangea.getEncryptCreditCardService
import java.text.SimpleDateFormat
import java.util.*

class DPCheckoutViewModel(application: Application) : BaseDPViewModel(application), Observable {
    @get:Bindable
    var isFreeTrial: Boolean = false
        private set
    @get:Bindable
    var isOffer: Boolean = false
        private set
    @get:Bindable
    var price: String? = null
        private set
    @get:Bindable
    var name: String? = null
        private set
    @get:Bindable
    var minSpend: String? = null
        private set
    private var mPromoOfferText: Spanned? = null
    private lateinit var mDPModel: DeliveryPassItemModel
    val mNextPayment = ObservableField<CharSequence>("")
    val mFreeTrialPeriod = ObservableField<CharSequence>("")
    val mTotal = ObservableField<CharSequence>("")
    val mIsAutoRenew = ObservableField(false)

    private val dpCheckoutLoadContentState =
        MutableLiveData<OneTimeLiveDataEvent<DPCheckoutLoadContentState>>()
    val dpCheckoutLoadContentStateLiveData: LiveData<OneTimeLiveDataEvent<DPCheckoutLoadContentState>>
        get() = dpCheckoutLoadContentState

    private val dpConfirmOrderState = MutableLiveData<OneTimeLiveDataEvent<DPConfirmOrderState>>()
    val dpConfirmOrderStateLiveData: LiveData<OneTimeLiveDataEvent<DPConfirmOrderState>>
        get() = dpConfirmOrderState


    private val dpCheckoutState = MutableLiveData<OneTimeLiveDataEvent<DPCheckoutState>>()
    val dpCheckoutStateLiveData: LiveData<OneTimeLiveDataEvent<DPCheckoutState>>
        get() = dpCheckoutState

    val context: Context
        get() = getApplication<Application>().applicationContext

    val promoOfferText: CharSequence?
        @Bindable
        get() = mPromoOfferText


    fun init(dPId: String, itemModel: DeliveryPassItemModel) {
        mDPModel = itemModel
        val isMonthly = dPId == itemModel.monthlyId
        isFreeTrial = if (isMonthly) itemModel.isMonthlyFreeTrial else itemModel.isOneTimeFreeTrial
        val offers = (Authentication.getInstance() as Authentication).deliveryPassOffers
        isOffer =
            offers != null && offers.isNotEmpty() && offers[0].sku != null && offers[0].sku == if (isMonthly) itemModel.monthlyId else itemModel.oneTimeId

        if (isOffer) {
            val wasPriceStyled = SpannableString(
                HtmlCompat.fromHtml(
                    context.getString(
                        R.string.dp_checkout_offer,
                        Utils.addPoundIfNecessary(offers!![0].originalPrice)
                    ),
                    HtmlCompat.FROM_HTML_MODE_LEGACY
                )
            )
            wasPriceStyled.setSpan(
                StrikethroughSpan(),
                wasPriceStyled.toString().lastIndexOf('£') + 1,
                wasPriceStyled.length,
                SpannableStringBuilder.SPAN_INCLUSIVE_INCLUSIVE
            )
            mPromoOfferText = wasPriceStyled
        } else {
            mPromoOfferText = null
        }

        if (isFreeTrial) {
            val format = SimpleDateFormat("dd-MMM-yyyy", Locale.UK)
            format.timeZone = Utils.UK.TIME_ZONE
            val calendar = Utils.getCurrentTimeInUK()
            calendar.add(Calendar.DAY_OF_MONTH, 30)
            val endDate = format.format(calendar.time)
            mFreeTrialPeriod.set(
                HtmlCompat.fromHtml(
                    context.getString(
                        R.string.dp_free_trial_period,
                        format.format(Date()),
                        endDate
                    ),
                    HtmlCompat.FROM_HTML_MODE_LEGACY
                )
            )
            mNextPayment.set(
                HtmlCompat.fromHtml(
                    context.getString(
                        R.string
                            .dp_free_trial_next_payment, endDate
                    ), HtmlCompat.FROM_HTML_MODE_LEGACY
                )
            )
        }

        setPriceDetails(itemModel, isMonthly, offers)

        minSpend = context.getString(
            R.string.dp_min_spend,
            Utils.removeTrailingZeros(itemModel.dpMinSpendAmt)
        )

        name = getDPName(itemModel)
    }

    private fun setPriceDetails(
        itemModel: DeliveryPassItemModel, isMonthly: Boolean,
        offers: Array<out UserViewResponse.PromotionalOffers>?
    ) {
        if (isMonthly) {
            var price: String? = null
            offers?.let {
                price = Utils.addPoundIfNecessary(
                    if (isOffer) it[0].promoPrice else itemModel.monthlyPrice
                )
            }
            this.price = context.resources.getQuantityString(
                R.plurals.dp_monthly_payment,
                itemModel.durationMonths,
                Utils.removeTrailingZeros(price),
                itemModel.durationMonths
            )
            mTotal.set(context.getString(R.string.dp_total, price))
        } else {
            var price: String? = null
            offers?.let {
                price = Utils.addPoundIfNecessary(
                    if (isOffer) it[0].promoPrice else itemModel.oneTimePrice
                )
            }
            this.price =
                context.getString(R.string.dp_onetime_payment, Utils.removeTrailingZeros(price))
            mTotal.set(context.getString(R.string.dp_total, price))
        }
    }

    private fun getDPName(itemModel: DeliveryPassItemModel): String {
        return if (itemModel.isDPPlus)
            context.resources.getQuantityString(
                R.plurals.dp_plus_buy_title,
                itemModel.durationMonths,
                itemModel.name,
                itemModel.durationMonths
            )
        else
            context.resources.getQuantityString(
                R.plurals.dp_buy_title,
                itemModel.durationMonths,
                context.getString(if ("true" == itemModel.isMidWeekDp) R.string.midweek else R.string.anytime),
                itemModel.durationMonths
            )
    }

    fun setFreeTrialData(checkoutResponse: CheckoutDeliveryPassResponse?) {
        if (isFreeTrial && checkoutResponse != null && checkoutResponse.freeTrialDuration != null && !TextUtils.isEmpty(
                checkoutResponse.freeTrialDuration!!.startDate
            )
        ) {
            val format = SimpleDateFormat("dd-MMM-yyyy", Locale.UK)
            format.timeZone = Utils.UK.TIME_ZONE

            val endDateMillis =
                Utils.parseIso8610Timestamp(checkoutResponse.freeTrialDuration!!.endDate)
            val calendar = Utils.getCurrentTimeInUK()
            calendar.timeInMillis = endDateMillis
            calendar.add(Calendar.DAY_OF_MONTH, 1)

            mFreeTrialPeriod.set(
                HtmlCompat.fromHtml(
                    context.getString(
                        R.string.dp_free_trial_period,
                        format.format(Date(Utils.parseIso8610Timestamp(checkoutResponse.freeTrialDuration!!.startDate))),
                        format.format(Date(endDateMillis))
                    ),
                    HtmlCompat.FROM_HTML_MODE_LEGACY
                )
            )

            mNextPayment.set(
                HtmlCompat.fromHtml(
                    context.getString(
                        R.string.dp_free_trial_next_payment,
                        format.format(calendar.time)
                    ),
                    HtmlCompat.FROM_HTML_MODE_LEGACY
                )
            )

            mTotal.set(
                context.getString(
                    R.string.dp_total,
                    Utils.addPoundIfNecessary(checkoutResponse.dpPrice)
                )
            )
        }
    }

    override fun addOnPropertyChangedCallback(onPropertyChangedCallback: Observable.OnPropertyChangedCallback) {
        //Implementation Not required
    }

    override fun removeOnPropertyChangedCallback(onPropertyChangedCallback: Observable.OnPropertyChangedCallback) {
        //Implementation Not required
    }

    fun initThreeDSForDP(
        paymentCard: PaymentDetailsResponse.PaymentCards,
        isFreeTrial: Boolean,
        selectedDPId: String
    ) {
        paymentCard.cardId?.let {
            val disposable = AsdaServiceFactory.get().checkoutDeliveryPass(
                selectedDPId,
                isFreeTrial,
                it,
                Authentication.getInstance().csrfToken
            )
                .subscribe({ checkoutDeliveryPassResponse ->
                    doCollectData(
                        checkoutDeliveryPassResponse,
                        paymentCard,
                        context
                    )
                }, { throwable ->
                    if (throwable is RxFailure && throwable.integer != -2) {
                        Log.i(TAG, "purchasedp API failed. " + throwable.integer)
                    }
                })
            safeAdd(disposable)
        }
    }

    @VisibleForTesting
    fun doCollectData(
        checkoutDeliveryPassResponse: CheckoutDeliveryPassResponse,
        paymentCard: PaymentDetailsResponse.PaymentCards,
        context: Context
    ) {
        if (isValidForCollectData(checkoutDeliveryPassResponse, paymentCard, context)) {
            val mThreeDS = ThreeDS()
            val cardinalInitCallback = CardinalInitCallbackImpl()
            mThreeDS.collectData(
                checkoutDeliveryPassResponse.paymentDetails?.jwt!!,
                Utils.isDebug(),
                cardinalInitCallback
            )
            if (Utils.isDebug()) {
                Log.d(TAG, "JWT: " + checkoutDeliveryPassResponse.paymentDetails?.jwt!!)
            }
        }
    }

    @VisibleForTesting
    fun isValidForCollectData(
        checkoutDeliveryPassResponse: CheckoutDeliveryPassResponse,
        paymentCard: PaymentDetailsResponse.PaymentCards,
        context: Context
    ): Boolean {
        return (!checkoutDeliveryPassResponse.paymentDetails?.jwt.isNullOrEmpty()
              && Utils.isThreeDs2XEnabled(context, paymentCard.cardBrand)
              && isThreeDsEnabledForDp(context))
    }

    @VisibleForTesting
    fun isThreeDsEnabledForDp(context: Context): Boolean {
        return FeatureSettings.isThreedsEnabledForDp(context)
    }

    fun loadContentForDPId(selectedDPId: String, isFreeTrial: Boolean) {
        dpCheckoutLoadContentState.postValue(
            OneTimeLiveDataEvent<DPCheckoutLoadContentState>(
                DPCheckoutLoadContentState.Loading
            )
        )
        val d = UserAccountDetails.getInstance().getUserCreditCardsDP(selectedDPId, isFreeTrial)
            .subscribe({ model ->
                dpCheckoutLoadContentState.postValue(
                    OneTimeLiveDataEvent<DPCheckoutLoadContentState>(
                        DPCheckoutLoadContentState.Loaded(
                            model,
                            selectedDPId,
                            isFreeTrial
                        )
                    )
                )
            },
                { throwable ->
                    dpCheckoutLoadContentState.postValue(
                        OneTimeLiveDataEvent<DPCheckoutLoadContentState>(
                            DPCheckoutLoadContentState.Error(throwable, selectedDPId, isFreeTrial)
                        )
                    )
                })

        safeAdd(d)
    }

    fun confirmOrder(
        cardNickname: String, csc: String, paRes: String?, autoRenew: Boolean,
        mPassPlainCvv: Boolean
    ) {
        dpConfirmOrderState.postValue(OneTimeLiveDataEvent<DPConfirmOrderState>(DPConfirmOrderState.Loading))
        val loginResponse = (Authentication.getInstance() as Authentication).loginResponse ?: return

        val disposable = getEncryptCreditCardService(context)!!
            .flatMap { encryptCreditCardService ->
                encryptCreditCardService.rxEncryptCvv(csc)
            }
            .flatMap { encryptedCc ->
                var deviceChannel = ""
                if (ThreeDsUtils.isCardinalInitSuccess) {
                    deviceChannel = ThreeDsUtils.DEVICE_CHANNEL
                }
                return@flatMap AsdaServiceFactory.get().rxConfirmDeliveryPassOrder(
                    cardNickname,
                    csc,
                    paRes,
                    autoRenew,
                    Authentication.getInstance().isDPFreeTrialEligible && (mDPModel.isMonthlyFreeTrial || mDPModel.isOneTimeFreeTrial),
                    encryptedCc.encryptedCc,
                    encryptedCc.encryptedCvv,
                    encryptedCc.keyId,
                    encryptedCc.phase,
                    encryptedCc.integrityCheck,
                    mPassPlainCvv,
                    Authentication.getInstance().csrfToken,
                    loginResponse.token,
                    deviceChannel
                )
            }
            .onErrorResumeNext { throwable: Throwable ->
                return@onErrorResumeNext handleConfirmOrderErrorResumeNext(
                    throwable,
                    cardNickname, csc, paRes, autoRenew, mPassPlainCvv, loginResponse
                )
            }
            .subscribe({ confirmDeliveryPassResponse ->
                dpConfirmOrderState.postValue(
                    OneTimeLiveDataEvent<DPConfirmOrderState>(
                        DPConfirmOrderState.Loaded(
                            confirmDeliveryPassResponse,
                            cardNickname,
                            csc,
                            paRes,
                            autoRenew
                        )
                    )
                )
            }, { throwable ->
                dpConfirmOrderState.postValue(
                    OneTimeLiveDataEvent<DPConfirmOrderState>(
                        DPConfirmOrderState.Error(throwable, cardNickname, csc, paRes, autoRenew)
                    )
                )
            })
        safeAdd(disposable)
    }

    private fun handleConfirmOrderErrorResumeNext(
        throwable: Throwable,
        cardNickname: String,
        csc: String,
        paRes: String?,
        autoRenew: Boolean,
        mPassPlainCvv: Boolean,
        loginResponse: LoginResponse
    ): ObservableSource<out ConfirmDeliveryPassResponse>? {
        if (throwable is RxFailure) {
            if (throwable.integer == pangea.ENCRYPTION_PROCESS_ERROR) {
                var deviceChannel = ""
                if (ThreeDsUtils.isCardinalInitSuccess) {
                    deviceChannel = ThreeDsUtils.DEVICE_CHANNEL
                }
                return AsdaServiceFactory.get().rxConfirmDeliveryPassOrder(
                    cardNickname, csc,
                    paRes, autoRenew,
                    Authentication.getInstance().isDPFreeTrialEligible && (mDPModel.isMonthlyFreeTrial || mDPModel.isOneTimeFreeTrial),
                    "", "", "", "", "",
                    mPassPlainCvv, Authentication.getInstance().csrfToken,
                    loginResponse.token, deviceChannel
                )
            } else {
                return io.reactivex.Observable.error(throwable);
            }
        }
        return io.reactivex.Observable.error(throwable);
    }


    fun checkout(
        dpPromotionId: String, cardNickname: String, csc: String, paRes: String?,
        autoRenew: Boolean
    ) {
        dpCheckoutState.postValue(OneTimeLiveDataEvent<DPCheckoutState>(DPCheckoutState.Loading))
        val isFreeTrial =
            (Authentication.getInstance() as Authentication).isDPFreeTrialEligible && (mDPModel.isMonthlyFreeTrial || mDPModel.isOneTimeFreeTrial)

        val callback = object : NetworkCallback<CheckoutDeliveryPassResponse>() {

            override fun onFailure(error: Int?, data: CheckoutDeliveryPassResponse) {
                dpCheckoutState.postValue(
                    OneTimeLiveDataEvent<DPCheckoutState>(
                        DPCheckoutState.Error(
                            error!!,
                            data
                        )
                    )
                )
            }

            override fun onSuccess(checkoutDeliveryPassResponse: CheckoutDeliveryPassResponse) {
                dpCheckoutState.postValue(
                    OneTimeLiveDataEvent<DPCheckoutState>(
                        DPCheckoutState.Loaded(
                            checkoutDeliveryPassResponse,
                            dpPromotionId,
                            cardNickname,
                            csc,
                            paRes,
                            autoRenew
                        )
                    )
                )
            }
        }
        AsdaServiceFactory.get()
            .checkoutDeliveryPass(
                dpPromotionId,
                isFreeTrial,
                Authentication.getInstance().csrfToken,
                callback
            )
    }

    companion object {
        private val TAG = "DPCheckoutViewModel"
    }

}
