package com.asda.android.app.orders.ods

import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.data.*
import com.asda.android.restapi.service.data.recipes.RecipeCartAddResponse
import java.util.ArrayList

const val REFUND_IS_COMPLETE = "processed"
const val REFUND_IS_CANCELLED = "cancelled"

fun mapBFFODStoWismoOrder(bffResponse: BffOdsOrderModel): WismoOrderResponse {
    val wismoOrder = WismoOrderResponse()
    bffResponse?.errors?.run {
        if (!isEmpty()) {
            wismoOrder.errorCode = get(0)?.message
            wismoOrder.errorDescription = get(0)?.message
        }
    }
    val order = bffResponse.data?.order
    order?.run {
        wismoOrder.status = status
        if (payload?.isNotEmpty() ?: false) {
            wismoOrder.payload = mapBffOdstoWismoPayload(payload?.get(0))
            wismoOrder.payload?.orderStatus = payload?.get(0)?.statusText
            wismoOrder.payload?.isExpress = payload?.get(0)?.isExpressOrder ?: false
            wismoOrder.payload?.isCancellable = payload?.get(0)?.isCancellable ?: false
        }
    }
    return wismoOrder
}

fun mapBffOdstoWismoPayload(bffPayload: BffPayload?): WismoOrderResponse.Payload {
    val payload = WismoOrderResponse.Payload()

    val reservationDetails = bffPayload?.deliveryReservationDetails?.get(0)
    payload.slotStartTime = reservationDetails?.startTime
    payload.slotEndTime = reservationDetails?.endTime
    payload.orderStatus = convertStatus(bffPayload?.status, bffPayload?.deliveryReservationDetails?.get(0)?.dispenseType
            ?: "")
    payload.dateOfDelivery = convertTime(reservationDetails?.fulfillmentDate)
    payload.bookedUntilTime = reservationDetails?.slotCutOffTime
    payload.etaTrackingEnd = reservationDetails?.etaEnd
    payload.etaTrackingStart = reservationDetails?.etaStart
    payload.refundAvailabilityPeriod = bffPayload?.orderCustomAttributes?.refundAvailabilityPeriod
    payload.refundAvailable = bffPayload?.orderCustomAttributes?.refundAvailable
    payload.fulfillmentType = mapfulfillmentType(reservationDetails?.dispenseType ?: "")
    payload.webOrderID = bffPayload?.orderNo
    payload.storeID = bffPayload?.accessPointInfo?.get(0)?.contactInfo?.storeFrontId?.storeId
    payload.showGcDetails = bffPayload?.showGiftCardDetail ?: false
    payload.orderStatusText = bffPayload?.statusText
    payload.status = bffPayload?.status
    payload.amendedOrderID = bffPayload?.orderCustomAttributes?.amendedOrderID
    payload.vanRegistrationId = bffPayload?.deliveryTracking?.vanRegistrationId
    payload.driverName = bffPayload?.deliveryTracking?.driverName
    payload.numberOfStopsAway = bffPayload?.deliveryTracking?.numberOfStopsAway.toString()

    val list: MutableList<ViewOrderResponse.Order.OrderItem?> = ArrayList()
    val subList: MutableList<WismoOrderResponse.SubstitutedItem> = ArrayList()
    val unavailableList: MutableList<WismoOrderResponse.UnavailableItem> = ArrayList()

    val shouldShowUnavailableItemList = when (bffPayload?.statusText) {
        ORDER_STATUS_PLACED -> false
        ORDER_STATUS_PROCESSING -> false
        ORDER_STATUS_AUTHORIZED -> false
        else -> true
    }

    for (orderLine in bffPayload?.orderLines.orEmpty()) {
        constructBffShoppingItem(orderLine, shouldShowUnavailableItemList).let { list.add(it) }
        if (orderLine?.pickedItem != null) {
            subList.add(constructBffSubstituteItem(orderLine))
        }
        if (orderLine?.orderLineCustomAttributes?.nilPickQty != "0") {
            unavailableList.add(mapUnavailableBffItemList(orderLine))
        }
    }

    if (subList.size > 0) {
        payload.substitutedItemList = subList.toTypedArray()
    }
    if (shouldShowUnavailableItemList && unavailableList.size > 0) {
        payload.unavailableItemList = unavailableList.toTypedArray()
    }

    payload.refundItemList = mapBffRefundItemfromODS(bffPayload)

    list.sortBy { it?.deptName }
    payload.shoppingItemList = list.toTypedArray()

    payload.customerAddress?.setAddress(bffPayload?.shipToAddress?.get(0)?.address?.addressLineOne)

    payload.payment = constructBffPaymentInfo(bffPayload)
    payload.payment?.colleagueDiscountAmount?.let { payload.colleagueDiscountEligible = true }

    payload.giftCards = constructBffGiftCardPaymentInfo(bffPayload)

    when (bffPayload?.deliveryReservationDetails?.get(0)?.dispenseType) {
        DISPENSE_TYPE_DELIVERY -> payload.customerAddress = constructBffDeliveryAddressForWismo(bffPayload)
        DISPENSE_TYPE_CNC, DISPENSE_TYPE_RCNC -> payload.cncStoreAddress = constructBffCncAddressforWismo(bffPayload)
    }

    val orderCustomAttributes = bffPayload?.orderCustomAttributes
    payload.orderAuthStatus = orderCustomAttributes?.authStatus
    payload.authStatusMsg = orderCustomAttributes?.authStatusMsg
    payload.allowCardEdit = orderCustomAttributes?.allowCardEdit?.toBoolean() ?: false
    payload.recipes = constructBffRecipeList(bffPayload)

    payload.containsRecipe = payload.recipes?.size ?: 0 > 0
    payload.isAmendable = bffPayload?.isAmendable

    return payload
}

fun constructBffShoppingItem(orderLine: BffOrderLine?, shouldShowUnavailableItemList: Boolean): ViewOrderResponse.Order.OrderItem? {

    var item: ViewOrderResponse.Order.OrderItem? = null
    if (orderLine != null) {
        item = ViewOrderResponse.Order.OrderItem()
        val orderProduct = orderLine.orderProduct
        val orderCustomAttributes = orderLine.orderLineCustomAttributes

        item.cost = orderCustomAttributes?.totalPrice
        item.desc = orderProduct?.description

        val offerLogistics = orderProduct?.offerLogistics

        item.id = offerLogistics?.skuId
        item.cin = offerLogistics?.cin
        item.imageURL = offerLogistics?.itemImageURL
        item.promoType = orderProduct?.offerAttribute?.offerType
        item.pricePerWt = if (orderLine?.unitOfMeasurement?.equals(KILOGRAMS)
                        ?: false) ProductDetails.PRICE_PER_WEIGHT_WEIGHTED else orderProduct?.pricePerUOM
        item.pricePerUOM = orderProduct?.pricePerUOM
        val nilPick = orderCustomAttributes?.nilPickQty?.toInt() ?: 0
        var orderedQty = orderLine.orderedQty?.measurementValue?.toInt()
        if (shouldShowUnavailableItemList) {
            orderedQty = orderLine.orderedQty?.measurementValue?.toInt()?.minus(nilPick)
        }
        item.qty = if (orderLine.unitOfMeasurement?.equals("KG", true) ?: false)
            orderLine.weight.toString() + orderLine.orderedQty?.unitOfMeasure.toString()
        else orderedQty?.toString()
        item.maxQty = orderProduct?.maxQty?.toString()
        item.bundledItemCount = orderProduct?.bundleItemCount ?: 0
        item.deptName = orderLine.deptName
        //TODO : below calculation can be removed once V4 api sends the pice in the response.
        item.price = orderLine.orderedQty?.measurementValue?.times((orderLine.unitPrice?.currencyAmount) as Float).toString()
        item.wasPrice = orderCustomAttributes?.wasPrice
        item.weight = orderCustomAttributes?.skuSize
        item.substitutionOption = orderLine.customerPreferences?.isSubstitutable ?: false
        item.associatedWithDish = orderCustomAttributes?.associated_with_dish?.toBoolean() ?: false
        item.numberOfRecipes = orderCustomAttributes?.number_of_recipes?.toInt() ?: 0
    }
    return item
}

fun constructBffSubstituteItem(orderLine: BffOrderLine?): WismoOrderResponse.SubstitutedItem {
    val substitutedItem = WismoOrderResponse.SubstitutedItem()
    val pickedItem = orderLine?.pickedItem

    substitutedItem.id = pickedItem?.id
    substitutedItem.description = pickedItem?.description
    substitutedItem.quantity = pickedItem?.substitutedQuantity.toString()
    substitutedItem.price = pickedItem?.unitPrice?.toString()
    substitutedItem.totalPrice = orderLine?.orderLineCustomAttributes?.substituted_price

    val subItemList: MutableList<WismoOrderResponse.SubItem> = ArrayList()
    for (subItem in pickedItem?.substitutedItemList.orEmpty()) {
        subItemList.add(mapBffSubItem(subItem))
    }
    substitutedItem.sub = subItemList.toTypedArray()

    return substitutedItem
}

fun mapBffSubItem(subItem: BffSubstitutedItem?): WismoOrderResponse.SubItem {
    val item = WismoOrderResponse.SubItem()
    item.subdescription = subItem?.description
    item.imageUrl = subItem?.imageURL
    item.pricePerUOM = subItem?.pricePerUOM
    item.subprice = subItem?.unitPrice?.times(subItem?.quantity ?: 0).toString()
    item.subquantity = subItem?.quantity.toString()
    item.totalPrice = subItem?.priceCharged?.toString()
    return item
}

fun mapUnavailableBffItemList(orderLine: BffOrderLine?): WismoOrderResponse.UnavailableItem {

    val nilPick = orderLine?.orderLineCustomAttributes?.nilPickQty?.toInt() ?: 0
    val orderedQty = orderLine?.orderedQty
    val unavailableItem = WismoOrderResponse.UnavailableItem()

    val orderProduct = orderLine?.orderProduct
    val offerLogistics = orderProduct?.offerLogistics

    unavailableItem.description = orderProduct?.description
    unavailableItem.id = offerLogistics?.skuId
    unavailableItem.imageURL = offerLogistics?.itemImageURL
    unavailableItem.pricePerWt = orderProduct?.pricePerUOM
    unavailableItem.totalPrice = orderLine?.orderLineCustomAttributes?.totalPrice
    unavailableItem.ordered = orderedQty?.measurementValue?.toString()
    unavailableItem.delivered = orderedQty?.measurementValue?.toInt()?.minus(nilPick).toString()

    return unavailableItem
}

fun mapBffRefundItemfromODS(bffPayload: BffPayload?): WismoOrderResponse.Refund? {

    val refundList = WismoOrderResponse.Refund()

    val approvedList = ArrayList<WismoOrderResponse.RefundItem>()
    val pendingList = ArrayList<WismoOrderResponse.RefundItem>()
    val cancelledList = ArrayList<WismoOrderResponse.RefundItem>()

    val odsRefundList = bffPayload?.returnDetails?.items.orEmpty()

    for (odsRefundItem in odsRefundList) {
        val refundItem = WismoOrderResponse.RefundItem()
        refundItem.name = odsRefundItem?.product_name
        refundItem.desc = odsRefundItem?.product_name
        refundItem.itemName = odsRefundItem?.product_name
        refundItem.price = odsRefundItem?.refund_amount.toString()
        refundItem.quantityRefunded = odsRefundItem?.refund_quantity.toString()
                .plus(" of ")
                .plus(odsRefundItem?.delivered_qty?.toString())
        refundItem.refundReason = odsRefundItem?.refund_reason
        refundItem.refundDate = odsRefundItem?.submitted_date
        refundItem.imageURL = odsRefundItem?.image_url

        when {
            odsRefundItem?.refund_status.equals(REFUND_IS_COMPLETE, false) -> approvedList.add(refundItem)
            odsRefundItem?.refund_status.equals(REFUND_IS_CANCELLED, false) -> cancelledList.add(refundItem)
            else -> pendingList.add(refundItem)
        }
    }

    refundList.approved = approvedList.toTypedArray()
    refundList.pending = pendingList.toTypedArray()
    refundList.cancelled = cancelledList.toTypedArray()

    return refundList
}

fun constructBffPaymentInfo(odsPayload: BffPayload?): WismoOrderResponse.Payment? {
    val payment = WismoOrderResponse.Payment()
    val odsPayment = odsPayload?.paymentMethods

    odsPayment?.let {
        for (paymentMethod in it) {
            paymentMethod?.personInfoBillTo?.let {
                payment.cardHolderName = it.name?.firstName
            }
            if (paymentMethod?.paymentType != GIFTCARD_PAYMENT_TYPE) {
                payment.orderCardNickName = paymentMethod?.cardId
                payment.cardType = paymentMethod?.paymentType
                payment.maskedCardNumber = paymentMethod?.last4DigitsOfCard
                payment.expirationMonth = paymentMethod?.instrumentExpiry?.substring(MONTH_START_INDEX)
                payment.expirationYear = paymentMethod?.instrumentExpiry?.substring(YEAR_START_INDEX, MONTH_START_INDEX)
                payment.amount = paymentMethod?.amount?.toDouble()
            }
        }
    }
    payment.costBeforePromo = odsPayload?.orderSummary?.amountBeforePromo?.currencyAmount?.toString()
    payment.carrierBagCharge = odsPayload?.orderSummary?.carrierBagCharge?.currencyAmount?.toString()
    payment.deliveryCharge = odsPayload?.orderSummary?.deliveryCharge?.currencyAmount?.toString()
    payment.evoucherTotalAmount = odsPayload?.orderSummary?.evoucherTotalAmount?.currencyAmount?.toString()
    payment.totalCost = odsPayload?.orderSummary?.totalAmount?.currencyAmount?.toString()
    payment.deliverySurcharge = odsPayload?.orderSummary?.minBasketCharge?.currencyAmount?.toString()
    payment.gcTotal = odsPayload?.orderSummary?.amountOnGiftCards?.currencyAmount?.toDouble()
    payment.incentiveDiscount = odsPayload?.orderSummary?.totalMultiSaveDiscount?.currencyAmount?.toString()

    val customAttributes = odsPayload?.orderCustomAttributes

    payment.paymentStatus = customAttributes?.authStatus
    payment.failureReason = customAttributes?.authFailureReason
    payment.paymentStatusMsg = customAttributes?.paymentStatusMsg
    payment.colleagueDiscountAmount = RestUtils.addPoundIfNecessary((odsPayload?.orderSummary?.colleagueDiscountAmount?.currencyAmount
            ?: 0.0).toString())
    payment.minOrderValue = customAttributes?.minOrderAmount
    return payment
}

fun constructBffGiftCardPaymentInfo(odsPayload: BffPayload?): List<WismoOrderResponse.GiftCardPayment?> {

    val giftCardPaymentList = ArrayList<WismoOrderResponse.GiftCardPayment>()

    val paymentMethod = odsPayload?.paymentMethods

    paymentMethod?.let {
        for (paymentCard in it) {
            if (paymentCard?.paymentType == GIFTCARD_PAYMENT_TYPE) {
                val giftCardPayment = WismoOrderResponse.GiftCardPayment()
                giftCardPayment.maskedGcNum = paymentCard.last4DigitsOfCard
                giftCardPayment.gcNum = paymentCard.cardNumber
                giftCardPayment.panelCode = paymentCard.panelCode
                giftCardPayment.amtApplied = paymentCard.amount?.toDouble()
                giftCardPaymentList.add(giftCardPayment)
            }
        }
    }

    return giftCardPaymentList
}


fun constructBffDeliveryAddressForWismo(bffPayload: BffPayload?): WismoOrderResponse.DeliveryAddress? {
    val wismoDeliveryAddress = WismoOrderResponse.DeliveryAddress()

    val odsAddress = bffPayload?.shipToAddress?.get(0)

    wismoDeliveryAddress.setAddress(
            if (bffPayload?.shipToAddress?.isEmpty() ?: true)
                ""
            else bffPayload?.shipToAddress?.get(0)?.address?.addressLineOne
    )
    wismoDeliveryAddress.setAddress(odsAddress?.address?.addressLineOne)
    wismoDeliveryAddress.address2 = odsAddress?.address?.addressLineTwo
    wismoDeliveryAddress.city = odsAddress?.address?.city
    wismoDeliveryAddress.postalCode = odsAddress?.address?.postalCode
    wismoDeliveryAddress.country = odsAddress?.address?.country
    wismoDeliveryAddress.latitude = odsAddress?.address?.geoPoint?.latitude.toString()
    wismoDeliveryAddress.longitude = odsAddress?.address?.geoPoint?.longitude.toString()
    wismoDeliveryAddress.phoneNumber = odsAddress?.storeFrontId?.phone?.completeNumber

    val primaryContact = bffPayload?.buyerInfo?.primaryContact

    wismoDeliveryAddress.firstName = primaryContact?.name?.firstName
    wismoDeliveryAddress.lastName = primaryContact?.name?.lastName
    wismoDeliveryAddress.middleName = primaryContact?.name?.middleName

    wismoDeliveryAddress.storeId = odsAddress?.storeFrontId?.storeId
    wismoDeliveryAddress.mobilePhoneNumber = primaryContact?.mobile?.completeNumber
    return wismoDeliveryAddress
}

fun constructBffCncAddressforWismo(bffPayload: BffPayload?): WismoOrderResponse.CncStoreAddress? {
    val cncStoreResponse = WismoOrderResponse.CncStoreAddress()

    val bffContactInfo = bffPayload?.accessPointInfo?.get(0)?.contactInfo
    val storeFrontId = bffContactInfo?.storeFrontId

    cncStoreResponse.setAddressLine1(bffContactInfo?.address?.addressLineOne)
    cncStoreResponse.setAddressLine2(bffContactInfo?.address?.addressLineTwo)
    cncStoreResponse.city = bffContactInfo?.address?.city
    cncStoreResponse.postalCode = bffContactInfo?.address?.postalCode
    cncStoreResponse.country = bffContactInfo?.address?.country
    cncStoreResponse.setStoreName(storeFrontId?.storeName)
    cncStoreResponse.latitude = bffContactInfo?.address?.geoPoint?.latitude.toString()
    cncStoreResponse.longitude = bffContactInfo?.address?.geoPoint?.longitude.toString()
    cncStoreResponse.phoneNumber = bffContactInfo?.storeFrontId?.phone?.completeNumber

    return cncStoreResponse
}

fun constructBffRecipeList(bffPayload: BffPayload?): MutableList<RecipeCartAddResponse.Recipe>? {
    val recipeList: MutableList<RecipeCartAddResponse.Recipe> = ArrayList()

    for (bffRecipe in bffPayload?.orderRecipeInfo.orEmpty()) {

        val ingredientList: MutableList<RecipeCartAddResponse.Ingredients> = ArrayList()

        for (bffIngredient in bffRecipe?.orderRecipeIngredients.orEmpty()) {
            val ingredient = RecipeCartAddResponse.Ingredients()
            ingredient.ingredientId = bffIngredient?.ingredientId
            ingredient.ingredientName = bffIngredient?.ingredientName
            ingredient.quantity = bffIngredient?.itemQty.toString()
            ingredient.skuId = bffIngredient?.skuId
            ingredient.qtyNeededForRecipe = bffIngredient?.itemQty.toString()
            ingredient.recipeId = bffRecipe?.recipeId
            ingredient.scene7AssetId = bffRecipe?.s7AssetId
            ingredientList.add(ingredient)
        }

        val recipe = RecipeCartAddResponse.Recipe()

        recipe.name = bffRecipe?.recipeTitle
        recipe.ingredients = ingredientList
        recipe.recipeId = bffRecipe?.recipeId
        recipe.source = bffRecipe?.recipeSource
        recipe.servingSize = bffRecipe?.servingSize.toString()
        recipe.scene7AssetId = bffRecipe?.s7AssetId
        recipe.costPerServing = bffRecipe?.costPerServing.toString()
        recipe.costOfRecipe = bffRecipe?.costOfRecipe.toString()
        recipe.readyIn = bffRecipe?.readyIn
        recipeList.add(recipe)
    }
    return recipeList
}
