package com.asda.android.app.recurringslot

import android.app.Dialog
import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AlertDialog
import androidx.lifecycle.ViewModelProviders
import androidx.recyclerview.widget.LinearLayoutManager
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.account.view.AccountFragment
import com.asda.android.app.account.view.BaseAccountActivity
import com.asda.android.app.account.view.WelcomeFragment
import com.asda.android.app.bookslotv3.service.BookSlotRepository
import com.asda.android.app.bookslotv3.subscription.SubscriptionAdapter
import com.asda.android.app.bookslotv3.utils.SlotUtils
import com.asda.android.app.bookslotv3.view.activity.DeliveryImpactActivity
import com.asda.android.app.dp.view.DeliveryPassPurchaseActivity
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.databinding.AccountRecurringSlotsBinding
import com.asda.android.restapi.constants.ExternalUrls
import com.asda.android.restapi.service.NetworkCallback
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.base.AsdaServiceImpl
import com.asda.android.restapi.service.data.RecurringSlotsViewResponse
import com.asda.android.restapi.service.data.bookslotv3.ServiceEligibilityData
import com.asda.android.restapi.service.data.recurringslots.*
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.CartManager
import com.asda.android.singleprofile.base.SingleProfile
import io.reactivex.observers.ResourceSingleObserver
import com.asda.android.base.core.view.ViewUtil
import com.asda.android.utils.view.ViewUtils.openWebPage

class RecurringSlotDetailsFragment : FeaturedFragment() {
    private lateinit var mModel: RecurringSlotAccountViewModel
    private var slotSubscriptionRepository: SlotSubscriptionRepository? = null
    private lateinit var fragmentBinder: AccountRecurringSlotsBinding
    private val CALLED_FROM_SKIP = "CALLED_FROM_SKIP"
    private val CALLED_FROM_CANCEL = "CALLED_FROM_CANCEL"

    override fun onAttach(context: Context) {
        super.onAttach(context)
        pageViewEvent = PageViewEvent(Anivia.PV_RECURRING_SLOT_DETAILS, Anivia.SECTION_ACCOUNT)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val binding = AccountRecurringSlotsBinding.inflate(inflater, container, false)

        if (RestUtils.isTablet(context) && parentFragment is AccountFragment) { // If on split screen need to use wrap_content width
            val params = binding.content.layoutParams
            params.width = ViewGroup.LayoutParams.WRAP_CONTENT
            binding.content.layoutParams = params
        }
        mModel = ViewModelProviders.of(this).get(RecurringSlotAccountViewModel::class.java)
        mModel.attach(this)
        binding.model = mModel
        fragmentBinder = binding

        return fragmentBinder.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        slotSubscriptionRepository = SlotSubscriptionRepository()
        fetchRecurringSlotForAllUsers()
    }

    private fun fetchRecurringSlotForAllUsers() {
        SingleProfile.getProfileResponse()?.profileId?.let { profileId ->
            slotSubscriptionRepository?.getSubscriptionForATGUsers(profileId, true)?.subscribe(object : ResourceSingleObserver<SubscriptionsResponse>() {
                override fun onSuccess(subscriptionsResponse: SubscriptionsResponse) {
                    dismissProgress()
                    if (isAdded && !mModel.setData(subscriptionsResponse)) {
                        closeRecurringSlotScreen()
                    }
                }

                override fun onError(e: Throwable) {
                    dismissProgress()
                    if (isAdded) {
                        showDialog(null, null)
                    }
                }
            })
        }
    }

    private fun fetchSubscriptionForMigratedUsers() {
        SingleProfile.getProfileResponse()?.profileId?.let { profileId ->
            slotSubscriptionRepository?.getSubscriptions(profileId, true)?.subscribe(object : ResourceSingleObserver<SubscriptionsResponse>() {
                override fun onSuccess(subscriptionsResponse: SubscriptionsResponse) {
                    dismissProgress()
                    if (isAdded && !mModel.setData(subscriptionsResponse)) {
                        closeRecurringSlotScreen()
                    }
                }

                override fun onError(e: Throwable) {
                    dismissProgress()
                    if (isAdded) {
                        showDialog(null, null)
                    }
                }
            })
        }
    }

    override fun hasActionBar(): Boolean {
        val activity = activity
        return activity is AsdaActivity || activity is DeliveryPassPurchaseActivity || activity is BaseAccountActivity || parentFragment is AccountFragment || activity is DeliveryImpactActivity
    }

    override fun getActionBarTitle(): String? {
        return if (activity != null) getString(R.string.recurring_slot_title) else ""
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            pop()
            return true
        }
        return false
    }

    override fun getInternalTag(): String? {
        return TAG
    }

    //region SKIP SLOT
    fun onClickSkip(id: String?, stopId: String?, date: String?) {
        context?.let {
            val builder = AlertDialog.Builder(it)
            builder.setMessage(getString(R.string.skip_recurring, date))
                  .setPositiveButton(R.string.ok) { _, _ -> doSkip(id, stopId) }
                  .setNegativeButton(R.string.cancel, null)
                  .create()
                  .show()
        }
    }

    private fun doSkip(id: String?, stopId: String?) {
        showProgress(R.string.please_wait)
        context?.let {
            //As per API docs, this API will be used for CXO and NON CXO
            skipForATGUsers(id, stopId, it)
        }
    }

    fun setAdapter(subscriptionAdapter: SubscriptionAdapter?) {
        subscriptionAdapter?.let { _adapter ->
            fragmentBinder.rsRecurringSlots.apply {
                layoutManager = LinearLayoutManager(context)
                adapter = _adapter
            }
            _adapter.notifyDataSetChanged()
        }
    }

    private fun skipForATGUsers(id: String?, stopId: String?, context: Context) {
        AsdaServiceFactory.get().skipRecurringSlot(id, stopId, Authentication.getInstance().csrfToken, object : NetworkCallback<RecurringSlotsViewResponse>() {
            override fun onFailure(error: Int?, data: RecurringSlotsViewResponse) {
                dismissProgress()
                if (isAdded) {
                    showDialog(error, data)
                }
            }

            override fun onSuccess(data: RecurringSlotsViewResponse) {
                dismissProgress()
                if (isAdded) {
                    if (FeatureSettings.isCXOMigrationEnabled(context)) {
                        serviceEligibilityCallToUpdateStoreId(CALLED_FROM_SKIP)
                    } else {
                        RecurringSlotsViewResponse.broadcast(data)
                        CartManager.get(context).refreshCart(context)
                    }
                    mModel?.subscriptionAdapter?.updateToSkipState(stopId)
                }
            }
        })
    }

    private fun serviceEligibilityCallToUpdateStoreId(calledFrom: String) {
        BookSlotRepository().makeServiceEligibilityCall(SlotUtils().getDefaultPostCodeFromSingleProfile(), includeAccessPointList = false)?.subscribe(object : ResourceSingleObserver<ServiceEligibilityData>() {
            override fun onSuccess(serviceEligibilityData: ServiceEligibilityData) {
                if ((serviceEligibilityData.data?.service_eligibility?.isNullOrEmpty() == false
                            && serviceEligibilityData.data?.service_eligibility?.get(0)?.accesspoint_list?.isNullOrEmpty() == false)
                      && !serviceEligibilityData.data?.service_eligibility?.get(0)?.accesspoint_list?.get(0)?.fulfill_store_id.isNullOrEmpty()) {
                    AsdaServiceImpl.setStoreId(serviceEligibilityData.data?.service_eligibility?.get(0)?.accesspoint_list?.get(0)?.fulfill_store_id)
                    SlotSubscriptionManager.refreshSubscriptionEligibilityForATGUsers()
                }
                if (calledFrom == CALLED_FROM_SKIP) {
                    refreshCXOCart()
                    SlotSubscriptionManager.refreshSubscriptionEligibility()
                } else { // CALLED_FROM_CANCEL
                    refreshCXOCart()
                    closeRecurringSlotScreen()
                }
            }

            override fun onError(e: Throwable) {
                //Do nothing
            }
        })
    }

    //endregion SKIP SLOT

    override fun onCreateDialog(dialog: Int): Dialog? {
        when (dialog) {
            DIALOG_CANCEL -> {
                context?.let {
                    val builder = AlertDialog.Builder(it)
                    return builder.setMessage(getString(R.string.dp_cancel_recurring_slot))
                          .setPositiveButton(R.string.keep_slot, null)
                          .setNegativeButton(R.string.cancel_slot) { _, _ -> cancelSlot() }
                          .create()
                }
            }
        }
        return null
    }

    //region CANCEL SLOT

    private fun cancelSlot() {
        showProgress(R.string.please_wait)
        context?.let {
            if (FeatureSettings.isCXOMigrationEnabled(it) && FeatureSettings.isSlotSubscriptionMigratedUser(it)) {
                onSubscriptionCancel()
            } else {
                cancelForATGUsers(it)
            }
        }
    }

    private fun cancelForATGUsers(context: Context) {
        AsdaServiceFactory.get().cancelRecurringSlot(mModel.id, Authentication.getInstance().csrfToken, object : NetworkCallback<RecurringSlotsViewResponse>() {
            override fun onFailure(error: Int?, data: RecurringSlotsViewResponse) {
                dismissProgress()
                if (isAdded) {
                    showDialog(error, data)
                }
            }

            override fun onSuccess(data: RecurringSlotsViewResponse) {
                dismissProgress()
                if (isAdded) {
                    if (FeatureSettings.isCXOMigrationEnabled(context)) {
                        serviceEligibilityCallToUpdateStoreId(CALLED_FROM_CANCEL)
                    } else {
                        RecurringSlotsViewResponse.broadcast(data)
                        CartManager.get(context).refreshCart(context)
                        closeRecurringSlotScreen()
                    }

                }
            }
        })
    }

    private fun refreshCXOCart() {
        com.asda.android.cxo.CartManager.instance?.refreshCart()
    }

    private fun onSubscriptionCancel() {
        val subscription = SlotSubscription(mModel.subscriptionInfo)
        subscription.subscription_info?.status = SubscriptionSlotStatus.CANCEL_INITIATED.name

        val singleProfileId = SingleProfile.getProfileResponse()?.profileId
        if (singleProfileId.isNullOrEmpty()) return

        slotSubscriptionRepository?.cancelSubscription(singleProfileId,
              subscription.subscription_info?.subscription_id
                    ?: "", SubscriptionCreation(SubscriptionCreationData(subscription.subscription_info)))
              ?.subscribe(object : ResourceSingleObserver<SubscriptionCreation>() {
                  override fun onSuccess(t: SubscriptionCreation) {
                      if (isAdded) {
                          SlotSubscriptionManager.refreshSubscriptionEligibility()
                          serviceEligibilityCallToUpdateStoreId(CALLED_FROM_CANCEL)
                      }
                  }

                  override fun onError(e: Throwable) {
                      dismissProgress()
                  }
              })
    }

    private fun closeRecurringSlotScreen() {
        if (!RestUtils.isTablet(context) && activity != null) {
            activity?.finish()
        } else if (activity is AsdaActivity) {
            val fragmentSwitcher = (activity as AsdaActivity).fragmentSwitcher
            if (fragmentSwitcher != null) {
                val fragment = fragmentSwitcher.topFragment
                if (fragment is AccountFragment) {
                    fragment.pop()
                    fragment.onPushed(WelcomeFragment.newInstance(), true)
                    fragment.reset()
                }
            }
        }
    }

    fun onClickCancel() {
        showDialog(DIALOG_CANCEL)
    }
    //endregion CANCEL SLOT

    fun onClickFAQ() {
        push(RecurringSlotInfoFragment.newInstance())
    }

    fun onClickLearnMore(faqUrl: String) {
        context?.let { _context ->
            openWebPage(faqUrl, _context, TAG)
        }
    }

    companion object {

        const val TAG = "RecurringSlotDetailsFragment"
        private const val DIALOG_CANCEL = 1

        fun newInstance(): RecurringSlotDetailsFragment {
            return RecurringSlotDetailsFragment()
        }
    }
}
