package com.asda.android.app.settings.featureflag

import android.content.Context
import androidx.annotation.VisibleForTesting
import com.asda.android.BuildConfig
import com.asda.android.R
import com.asda.android.abtestingframework.implementation.sitespect.metrics.SitespectP13NMetrics
import com.asda.android.abtestingframework.implementation.sitespect.variations.Variations
import com.asda.android.app.main.AsdaApplication
import com.asda.android.firebase.RemoteConfig
import com.asda.android.restapi.app.siteconfig.SiteConfigDataSource
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.Authentication
import com.asda.android.utils.Utils

object FeatureSettings {
    const val SETTING_FORCE_OFF = 0
    const val SETTING_FORCE_ON = 1
    const val SETTING_HONOR_REMOTE_SETTING = 2 // This could be sitespect or killswitch or siteconfig.

    interface RemoteSetting {
        fun isFeatureEnabled(): Boolean
    }

    @VisibleForTesting
    fun getFeatureSetting(context: Context, featureKey: String, defaultSetting: Int): Int {
        if (Utils.isDevBuild()) {
            // NOTE: While the defaultSetting is used here, the setting for the feature in FeatureFlag item in settings.xml will override this value.
            return SharedPreferencesUtil.getSetting(context, featureKey, defaultSetting)
        } else {
            // for production, always return remote setting (sitespect or killswitch or siteconfig)
            return SETTING_HONOR_REMOTE_SETTING
        }
    }

    @VisibleForTesting
    fun isFeatureEnabled(
          context: Context,
          featureKey: String,
          defaultSetting: Int = SETTING_FORCE_OFF,
          remoteSetting: RemoteSetting?): Boolean {
        var enabled = false
        when (getFeatureSetting(context, featureKey, defaultSetting)) {
            SETTING_FORCE_ON -> enabled = true
            SETTING_FORCE_OFF -> enabled = false
            SETTING_HONOR_REMOTE_SETTING -> enabled = remoteSetting?.isFeatureEnabled() ?: false
        }
        return enabled
    }

    fun isCriteoEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_criteo),
              defaultSetting = SETTING_FORCE_ON,
              remoteSetting = getRemoteSettingCriterion(context))
    }

    @VisibleForTesting
    fun getRemoteSettingCriterion(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns sitespect value
            return SharedPreferencesUtil.getIsCriteoEnabled(context)
        }
    }

    fun isRelatedSearchesEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_related_searches),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getIsRelatedSearchesEnabled())
    }

    @VisibleForTesting
    fun getIsRelatedSearchesEnabled() = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns sitespect value
            return if (Authentication.getInstance() != null && Authentication.getInstance().isLoggedIn) {
                SiteConfigDataSource.getSiteConfigValue()?.isRelatedSearchEnabled == true
            } else {
                false
            }
        }
    }

    fun isEACSceneSevenEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_eac_scene_seven),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getisEACSceneSevenEnabled(context))
    }

    @VisibleForTesting
    fun getisEACSceneSevenEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns sitespect/killswitch value
            return RemoteConfig.getValue(context.getString(R.string.scene7ForEACEnabled)).toBoolean()
        }
    }

    fun isFMOEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_fmo),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getIsFmoenabled(context))
    }

    @VisibleForTesting
    fun getIsFmoenabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns sitespect value
            return SharedPreferencesUtil.getIsFMOEnabled(context)
        }
    }

    fun isSolrEACEngineEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_solr_eac),
              defaultSetting = SETTING_FORCE_ON,
              remoteSetting = getIsSolrEACEngineEnabled())
    }

    @VisibleForTesting
    fun getIsSolrEACEngineEnabled() = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            return if (Authentication.getInstance() != null && Authentication.getInstance().isLoggedIn) {
                AsdaApplication.getComponent().sitespectAbTestEngine.hasVariation(Variations.EACSolrVariation)
            } else {
                false
            }
        }
    }

    fun isSolrSearchEngineEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_solr_search),
              defaultSetting = SETTING_FORCE_ON,
              remoteSetting = getIsSolrRemoteSetting())
    }

    @VisibleForTesting
    fun getIsSolrRemoteSetting() = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            return if (Authentication.getInstance() != null && Authentication.getInstance().isLoggedIn) {
                AsdaApplication.getComponent().sitespectAbTestEngine.hasVariation(Variations.SearchSolrVariation)
            } else {
                false
            }
        }
    }

    @VisibleForTesting
    fun isFirebaseAnalyticsEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_firebase_analytics),
              defaultSetting = SETTING_FORCE_ON,
              remoteSetting = getFirebaseAnalyticsEnabled(context))
    }

    @VisibleForTesting
    fun getFirebaseAnalyticsEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns killswitch value
            return SharedPreferencesUtil.getFirebaseAnalyticsMobileConfigEnabledFlag(context)
        }
    }

    @VisibleForTesting
    fun isFirebasePerformanceEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_firebase_performance_metrics),
              defaultSetting = SETTING_FORCE_ON,
              remoteSetting = getisFirebasePerformanceEnabled(context))
    }

    @VisibleForTesting
    fun getisFirebasePerformanceEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns killswitch value
            return SharedPreferencesUtil.getFirebasePerformanceEnabledFlag(context)
        }
    }

    fun isCXOCancellationEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_cxo_cancellation),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getisCXOCancellationEnabled(context))
    }

    @VisibleForTesting
    fun getisCXOCancellationEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getIsCXOCancellationEnabled(context)
        }
    }

    fun isBottomNavOrderTestEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_bottom_nav_order),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getisBottomNavOrderTestEnabled())
    }

    @VisibleForTesting
    fun getisBottomNavOrderTestEnabled() = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            return SiteConfigDataSource.getSiteConfigValue()?.isBottomNavOrderTestEnabled == true
        }
    }

    fun isAllOffersP13NEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_all_offers_p13n),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getIsAllOffersP13Enabled())
    }

    @VisibleForTesting
    fun getIsAllOffersP13Enabled() = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns sitespect value
            return if (Authentication.getInstance() != null) {
                AsdaApplication.getComponent().sitespectAbTestEngine.hasCampaign(SitespectP13NMetrics.P13N_ALL_OFFERS_CAMPAIGN_ID)
                      && (AsdaApplication.getComponent().sitespectAbTestEngine.hasVariation(SitespectP13NMetrics.P13N_ALL_OFFERS_VARIATION_ID)
                      || AsdaApplication.getComponent().sitespectAbTestEngine.hasVariation(SitespectP13NMetrics.P13N_ALL_OFFERS_ORCH_VARIATION_ID))
            } else {
                false
            }
        }
    }

    fun isBFFforODSEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_bff_for_ods),
              defaultSetting = SETTING_HONOR_REMOTE_SETTING,
              remoteSetting = getIsBFFforODSEnabled(context))
    }

    @VisibleForTesting
    fun getIsBFFforODSEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.isBFFforODSEnabled(context)
        }
    }

    fun isCXOMigrationEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_cxo_migration),
              defaultSetting = SETTING_HONOR_REMOTE_SETTING,
              remoteSetting = getisCXOMigrationEnabled(context))
    }

    @VisibleForTesting
    fun getisCXOMigrationEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            return Authentication.getInstance().userViewResponse?.isCXOMigratedUser == true && SharedPreferencesUtil.getIsCXOMigrationEnabled(context)
        }
    }

    fun isP13NBYGRecipeEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_byg_recipe_p13n),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getisP13NBYGRecipeEnabled())
    }

    @VisibleForTesting
    fun getisP13NBYGRecipeEnabled() = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns sitespect value
            return AsdaApplication.getComponent().sitespectAbTestEngine.hasCampaign(SitespectP13NMetrics.P13N_BYG_RECIPE_CAMPAIGN_ID)
                  && AsdaApplication.getComponent().sitespectAbTestEngine.hasVariation(SitespectP13NMetrics.P13N_BYG_RECIPE_VARIATION_ID)
        }
    }

    @VisibleForTesting
    fun isSelfIsolationEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_self_isolation),
              defaultSetting = SETTING_FORCE_ON,
              remoteSetting = getisSelfIsolationEnabled(context))
    }

    @VisibleForTesting
    fun getisSelfIsolationEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns firebase value
            return SharedPreferencesUtil.getIsSelfIsolationEnabled(context)
        }
    }

    @VisibleForTesting
    fun isSubstitutionMessageEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_substitution_policy_update),
              defaultSetting = SETTING_FORCE_ON,
              remoteSetting = getisSubstitutionMessageEnabled(context))
    }

    @VisibleForTesting
    fun getisSubstitutionMessageEnabled(context: Context) = object : RemoteSetting {
        override fun isFeatureEnabled(): Boolean {
            // Returns firebase value
            return SharedPreferencesUtil.getIsSubstitutionPolicyUpdateEnabled(context)
        }
    }


    @VisibleForTesting
    fun isSlotBlockedByAmends(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_slot_blocked_by_amends),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getisSlotBlockedByAmends(context))
    }

    @VisibleForTesting
    fun getisSlotBlockedByAmends(context: Context) = object : RemoteSetting {
        //Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getIsSlotBlockedByAmends(context)
        }
    }

    @VisibleForTesting
    fun isCovid19BannerOnBookSlotPageEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_c19_banner_on_bookslot),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getisCovid19BannerOnBookSlotPageEnabled(context))
    }

    @VisibleForTesting
    fun getisCovid19BannerOnBookSlotPageEnabled(context: Context) = object : RemoteSetting {
        //Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getIsCOVID19BannerEnabledOnBookSlot(context)
        }
    }

    @VisibleForTesting
    fun isDeliveryPassBlocked(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_delivery_pass_blocked),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getisDeliveryPassBlocked(context))
    }

    @VisibleForTesting
    fun getisDeliveryPassBlocked(context: Context) = object : RemoteSetting {
        //Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getDeliveryPassBlocked(context)
        }
    }

    fun isSlotSubscriptionMigratedUser(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_slot_subscription_migrated),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getisSlotSubscriptionMigratedUser(context))
    }

    @VisibleForTesting
    fun getisSlotSubscriptionMigratedUser(context: Context) = object : RemoteSetting {
        //Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getIsSlotSubscriptionMigrated(context) &&
                  Authentication.getInstance()?.userViewResponse?.slotSubscriptionMigratedUser == true
        }
    }

    fun isPaymentAuthBannerVisible(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_payment_auth_banner_setting),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getisPaymentAuthBannerVisible(context))
    }

    @VisibleForTesting
    fun getisPaymentAuthBannerVisible(context: Context) = object : RemoteSetting {
        //Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getPaymentAuthBannerVisibility(context)
        }
    }

    fun isThreedsEnabledForDp(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_three_ds_dp),
              defaultSetting = SETTING_FORCE_ON,
              remoteSetting = getIsThreedsEnabledForDp(context))
    }

    fun isThreedsEnabledForEditDp(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_three_ds_edit_dp),
              defaultSetting = SETTING_FORCE_ON,
              remoteSetting = getIsThreedsEnabledForEditDp(context))
    }


    @VisibleForTesting
    fun getIsThreedsEnabledForEditDp(context: Context) = object : RemoteSetting {
        //Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getThreeDSV2EnabledFlagForEditDp(context)
        }
    }

    @VisibleForTesting
    fun getIsThreedsEnabledForDp(context: Context) = object : RemoteSetting {
        //Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getThreeDSV2EnabledFlagForDp(context)
        }
    }

    fun isThreedsEnabledAmex(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_three_ds_amex),
              defaultSetting = SETTING_FORCE_ON,
              remoteSetting = getisThreedsEnabledAmex(context))
    }

    @VisibleForTesting
    fun getisThreedsEnabledAmex(context: Context) = object : RemoteSetting {
        //Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getAmexCardsEnabledFlag(context)
        }
    }

    //This flag is added to start passing single profile Id instead ATG profile id
    fun isProfileIdAcceptedInBookSlot(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_profile_id_accepted_for_bookslot),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  //Firebase config value
                  override fun isFeatureEnabled(): Boolean {
                      return SharedPreferencesUtil.getIsProfileIdAcceptedForBookSlot(context)
                  }
              })
    }

    fun isSlotEligibilityMinPurchaseEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_service_eligibility_light_weight),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  //Firebase config value
                  override fun isFeatureEnabled(): Boolean {
                      return SharedPreferencesUtil.getIsSlotEligibilityLighWeightEnabled(context)
                  }
              })
    }

    fun isP13TrolleyFeatureEnabled(context: Context, campaignId: String, variationId: String): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_p13n_trolley_feature),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  override fun isFeatureEnabled(): Boolean {
                      // Returns sitespect value
                      return if (Authentication.getInstance() != null) {
                          AsdaApplication.getComponent().sitespectAbTestEngine.hasCampaign(campaignId.toInt())
                                && AsdaApplication.getComponent().sitespectAbTestEngine.hasVariation(variationId.toInt())
                      } else {
                          false
                      }
                  }
              })
    }

    @VisibleForTesting
    fun isTrolleyTempoEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_trolley_tempo),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  //Firebase config value
                  override fun isFeatureEnabled(): Boolean {
                      return SharedPreferencesUtil.isTrolleyTempoEnabled(context) && (SiteConfigDataSource.getSiteConfigValue()?.isTrolleyTempoEnabled == true)
                  }
              })
    }


    fun isP13NOffersOnAisleEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_p13n_offers_aisle_feature),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  override fun isFeatureEnabled(): Boolean {
                      // Returns sitespect value
                      return SiteConfigDataSource.getSiteConfigValue()?.isP13NOffersEnabled == true
                  }
              })
    }

    fun isTaxonomyTempoEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_cms_taxonomy),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  //Firebase config value
                  override fun isFeatureEnabled(): Boolean {
                      return SharedPreferencesUtil.isTaxonomyTempoEnabled(context) && SiteConfigDataSource.getSiteConfigValue()?.isTaxonomyTempoEnabled == true
                  }
              })
    }

    fun isCMSNewHomeEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_cms_home_feature),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  //Firebase config value
                  override fun isFeatureEnabled(): Boolean {
                      return SharedPreferencesUtil.isCMSNewHomeEnabled(context) && SiteConfigDataSource.getSiteConfigValue()?.isHomeTempoEnabled == true
                  }
              })
    }

    fun isCMSSearchEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_cms_search_feature),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  //Firebase config value
                  override fun isFeatureEnabled(): Boolean {
                      return SharedPreferencesUtil.isSearchTempoEnabled(context) && SiteConfigDataSource.getSiteConfigValue()?.isSearchTempoEnabled == true
                  }
              })
    }

    fun isPdpTempEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_pdp_new),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  //Firebase config value
                  override fun isFeatureEnabled(): Boolean {
                      return SharedPreferencesUtil.isPDPNewEnabled(context) && SiteConfigDataSource.getSiteConfigValue()?.isPDPTempoEnabled == true
                  }
              })
    }

    fun isCMSBeforeYouGoEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_byg_page_feature),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  //Firebase config value
                  //TODO: Add Siteconfig check for CMS/Tempo BYG screen
                  override fun isFeatureEnabled(): Boolean {
                      return SharedPreferencesUtil.isCMSBeforeYouGoEnabled(context) && SiteConfigDataSource.getSiteConfigValue()?.isTrolleyTempoEnabled == true
                  }
              })
    }

    fun isRecurringSlotDisabledForNormalDP(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_recurring_slot_disabled_for_normal_dp),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getisRecurringSlotDisabledForNormalDP(context))
    }

    @VisibleForTesting
    fun getisRecurringSlotDisabledForNormalDP(context: Context) = object : RemoteSetting {
        //Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.isRecurringSlotDisabledForNormalDP(context)
        }
    }

    fun isExpressHDEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_express_hd),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getisExpressHDEnabled(context))
    }

    @VisibleForTesting
    fun getisExpressHDEnabled(context: Context) = object : RemoteSetting {
        //Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getIsExpressHDEnabled(context)
        }
    }

    fun isFavoriteBffApiEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_favorite_api),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getIsFavouriteBffApiEnabled(context)) && SiteConfigDataSource.getSiteConfigValue()?.isFavouritesBannerEnabled == true
    }

    @VisibleForTesting
    fun getIsFavouriteBffApiEnabled(context: Context) = object : RemoteSetting {
        //Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getIsBffFavoriteApiEnabled(context)
        }
    }

    fun isSponsoredLabelEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_sponsored_label),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = getIsSponsoredLabel(context))
    }

    @VisibleForTesting
    fun getIsSponsoredLabel(context: Context) = object : RemoteSetting {
        //Firebase config value
        override fun isFeatureEnabled(): Boolean {
            return SharedPreferencesUtil.getIsSponsoredLabelEnabled(context)
        }
    }

    fun isSbaAdsEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_sba_add_label),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  //Firebase config value
                  override fun isFeatureEnabled(): Boolean {
                      return SharedPreferencesUtil.getIsSbaAdsEnabled(context) && SiteConfigDataSource.getSiteConfigValue()?.isSbaAdsEnabled == true
                  }
              })
    }


    fun isSearchInGridV1Enabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_search_ingrid_v1_label),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  //Firebase config value
                  override fun isFeatureEnabled(): Boolean {
                      return SharedPreferencesUtil.getIsSearchInGridV1Enabled(context)
                  }
              })
    }

    fun isSlotsDateTimeAddressForReservedStatusEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_use_addr_date_time_for_reserved_slot_indicator),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  //Firebase config value
                  override fun isFeatureEnabled(): Boolean {
                      return SharedPreferencesUtil.getIsSlotsDateTimeAddressForReservedStatusEnabled(context)
                  }
              })
    }

    fun isEGiftCardsEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_settings_e_gift_cards_feature),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  //Firebase config value
                  override fun isFeatureEnabled(): Boolean {
                      return SharedPreferencesUtil.isEGiftCardsEnabled(context)
                  }
              })
    }

    fun isSlotSecondaryStoreEnabled(context: Context): Boolean {
        return isFeatureEnabled(
              context = context,
              featureKey = context.getString(R.string.debug_is_slot_secondary_store_enabled),
              defaultSetting = SETTING_FORCE_OFF,
              remoteSetting = object : RemoteSetting {
                  //Firebase config value
                  override fun isFeatureEnabled(): Boolean {
                      return SharedPreferencesUtil.getIsSlotSecondaryStoreEnabled(context)
                  }
              })
    }

    fun isCmsLinkSaveEnabled(context: Context) = isFeatureEnabled(
          context = context,
          featureKey = context.getString(R.string.debug_settings_link_Save),
          remoteSetting = object : RemoteSetting {
              //Firebase config value
              override fun isFeatureEnabled(): Boolean {
                  return SharedPreferencesUtil.isCMSLinkSaveEnabled(context) &&
                        SiteConfigDataSource.getSiteConfigValue()?.isLinkSaveTempoEnabled == true
              }
          })
}
