package com.asda.android.app.shop

import android.content.Context
import android.util.Log
import androidx.annotation.VisibleForTesting
import androidx.appcompat.app.AppCompatActivity
import com.asda.android.R
import com.asda.android.app.shoppinglists.ShoppingListManager
import com.asda.android.base.core.view.ui.ItemQuantityController
import com.asda.android.cxo.CartManager
import com.asda.android.cxo.formatter.CXOErrors
import com.asda.android.designlibrary.view.dialog.AsdaAlertDialog
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.data.Cart
import com.asda.android.restapi.service.data.cart.CartResponse
import com.asda.android.restapi.utils.parseFloat
import org.json.JSONObject

class CXOHelper {

    private val TAG = "CXOHelper"
    private val skuIdSeparator = "|"
    private val ZERO_ORDER_TOTAL = "0.00"
    var cxoCartItemMap = HashMap<String, com.asda.android.restapi.service.data.cart.Cart.CartItems>()

    fun initItemQuantityController(cartItem: Cart.CartItem?, shelfListItem: ShelfListItem, alwaysShowPopup: Boolean): ItemQuantityController {
        return ItemQuantityController(
              if (cartItem != null) RestUtils.poundStringToBigDecimal(cartItem.price) else shelfListItem.pricePerUnit,
              if (cartItem != null) cartItem.avgWeight else shelfListItem.avgWeight,
              if (cartItem != null) cartItem.pricePerWt else shelfListItem.pricePerWt,
              (if (cartItem != null) cartItem.maxQty else shelfListItem.maxQty).parseFloat(0.01f, 99f),
              cartItem != null || alwaysShowPopup)
    }

    fun initCXOItemQuantityController(cartItem: com.asda.android.restapi.service.data.cart.Cart.CartItems?, shelfListItem: ShelfListItem, alwaysShowPopup: Boolean, isShoppingList: Boolean): ItemQuantityController {
        var price = shelfListItem.pricePerUnit
        var avgWeight = shelfListItem.avgWeight
        var salesUnit = shelfListItem.pricePerWt
        var maxQty = shelfListItem.maxQty
        if (!isShoppingList) {
            cartItem?.let {
                price = RestUtils.poundStringToBigDecimal(it.catalogInfo?.catalogPriceInfo?.priceInfo?.price)
                avgWeight = it.catalogInfo?.catalogPriceInfo?.priceInfo?.avgWeight?.toString()
                salesUnit = it.catalogInfo?.catalogItem?.salesUnit
                maxQty = it.catalogInfo?.inventory?.availabilityInfo?.maxQuantity?.toString()
            }
        }
        return ItemQuantityController(price, avgWeight, salesUnit, maxQty.parseFloat(0.01f, 99f),
              cartItem != null || alwaysShowPopup)
    }

    fun getCxoItemFromShelfItem(item: ShelfListItem, isShoppingList: Boolean, cartItem: Cart.CartItem?): com.asda.android.restapi.service.data.cart.Cart.CartItems? {
        return if (isShoppingList) {
            cartItem?.let { c ->
                com.asda.android.restapi.service.data.cart.Cart.CartItems(skuId = c.id, quantity = c.quantityString.toDouble())
            }
        } else {
            com.asda.android.restapi.service.data.cart.Cart.CartItems(skuId = item.id, quantity = item.qty?.toDouble())
        }
    }


    fun getItemInCartCXO(item: ShelfListItem, isShoppingList: Boolean): com.asda.android.restapi.service.data.cart.Cart.CartItems? {
        return when {
            isShoppingList -> {
                if (getItemInCartCXOFromItemId(item.id) == null) null else getItemInCartCXOFromItemId(item.id)
            }
            else ->
                getItemInCartCXOFromItemId(item.id)
        }
    }

    @VisibleForTesting
    fun getShoppingListManager(): ShoppingListManager {
        return ShoppingListManager.getInstance()
    }


    fun getItemInCartCXOFromItemId(itemId: String): com.asda.android.restapi.service.data.cart.Cart.CartItems? {
        var cartItem: com.asda.android.restapi.service.data.cart.Cart.CartItems? = null
        val itemMap: HashMap<String, com.asda.android.restapi.service.data.cart.Cart.CartItems> = if (cxoCartItemMap.size == 0 && CartManager.instance != null) {
            CartManager.instance!!.cartItemMap
        } else {
            cxoCartItemMap
        }
        if (itemMap.size > 0) {
            cartItem = itemMap[itemId]
        }
        return cartItem
    }

    fun showErrorDialog(context: Context?, errors: MutableList<com.asda.android.restapi.service.data.cart.Cart.Error>) {
        errors.let {
            if (it.isNotEmpty()) {
                var leftText = ""
                var rightText = ""
                var reason = ""
                context?.let { context ->
                    leftText = context.getString(R.string.cancel)
                    rightText = context.getString(R.string.ok)
                    reason = CXOErrors.getResId((it[0].code))?.resId?.let { it1 -> context.getString(it1) }
                          ?: context.getString(CXOErrors.CART_GENERIC_ERROR.resId)
                }

                showDialog(leftText, rightText, reason, context)
            }
        }
    }

    fun showQuantityCheckDialog(context: Context?) {
        var leftText = ""
        var rightText = ""
        var reason = ""
        context?.let { context ->
            leftText = context.getString(R.string.got_it)
            rightText = context.getString(R.string.cart_express_full_title)
            reason = context.getString(R.string.cart_express_full)
        }

        showSingleActionDialog(leftText, rightText, reason, context)
    }

    fun handleCartError(error: Throwable, context: Context?) {
        var leftText = ""
        var rightText = ""
        var reason = ""
        context?.let {
            leftText = it.getString(R.string.cancel)
            rightText = it.getString(R.string.ok)
            var errorCode = CXOErrors.CART_GENERIC_ERROR.code
            var errorDescription = it.getString(R.string.error_message)
            reason = it.getString(R.string.error_message)
            if (error is com.jakewharton.retrofit2.adapter.rxjava2.HttpException) {
                try {
                    val response = error.response()?.errorBody()
                    if (response != null) {
                        val responseBytes = response.bytes()
                        if (!responseBytes.isEmpty()) {
                            val responseBodyStr = String(responseBytes, Charsets.UTF_8)
                            val json = JSONObject(responseBodyStr)
                            val jsonArray = json.getJSONArray("errors")
                            if (!json.isNull("errors") && jsonArray.length() >= 0) {
                                val jsonObject = jsonArray.getJSONObject(0)
                                if (jsonObject.has("code"))
                                    errorCode = jsonObject.getString("code")
                                else if (jsonObject.has("error_code"))
                                    errorCode = jsonObject.getString("error_code")
                                if (jsonObject.has("error_description"))
                                    errorDescription = jsonArray.getJSONObject(0).getString("error_description")
                                else if (jsonObject.has("message"))
                                    errorDescription = jsonArray.getJSONObject(0).getString("message")
                            }
                        }
                        reason = CXOErrors.getResId(errorCode)?.resId?.let { it1 -> context.getString(it1) }
                              ?: context.getString(CXOErrors.CART_GENERIC_ERROR.resId)
                    }
                } catch (ex: Exception) {
                    Log.w(TAG, ex)
                }
            }
        }
        showDialog(leftText, rightText, reason, context)
    }

    private fun showDialog(leftText: String, rightText: String, reason: String, context: Context?) {
        val dialog = AsdaAlertDialog
              .newInstance(leftText, rightText, reason)
        dialog.let { alertDialog ->
            alertDialog.onRightButtonClick { alertDialog.dismiss() }
            alertDialog.onLeftButtonClick { alertDialog.dismiss() }
            (context as AppCompatActivity).supportFragmentManager.let { manager ->
                alertDialog.show(manager, TAG)
            }
        }
    }

    fun showSingleActionDialog(buttonText: String, title: String, reason: String, context: Context?) {
        val dialog = AsdaAlertDialog
              .newInstance(buttonText, title, reason, true)
        dialog.let { alertDialog ->
            alertDialog.onLeftButtonClick { alertDialog.dismiss() }
            (context as AppCompatActivity).supportFragmentManager.let { manager ->
                alertDialog.show(manager, TAG)
            }
        }
    }

    fun getCXOCartContent(): String? {
        val cartItems = CartManager.instance?.cartResponse?.data?.cart?.cartItems
        val cartContents = mutableSetOf<String>()
        cartItems?.let {
            for (item in it) {
                item.skuId?.let { itemId ->
                    cartContents.add(itemId)
                }
            }
        }
        return cartContents.joinToString(skuIdSeparator)

    }

    fun getCXOCartOrderTotal(cartResponse: CartResponse): String? {
        return cartResponse.data?.cart?.cartPriceSummary?.itemCost?.let { itemCost ->
            itemCost.itemCostAfterDiscount?.toString()
        } ?: run {
            ZERO_ORDER_TOTAL
        }
    }

    fun toggleCartFragment(isOpened : Boolean) {
        CartManager.instance?.toggleCartFragment(isOpened)
    }

}
