package com.asda.android.app.shop

import android.animation.Animator
import android.animation.AnimatorListenerAdapter
import android.annotation.SuppressLint
import android.annotation.TargetApi
import android.app.Activity
import android.app.Dialog
import android.app.ProgressDialog
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.*
import android.view.animation.AccelerateDecelerateInterpolator
import android.widget.LinearLayout
import android.widget.RadioGroup
import android.widget.TextView
import androidx.appcompat.widget.AppCompatRadioButton
import androidx.lifecycle.ViewModelProviders
import com.asda.android.R
import com.asda.android.base.core.ActivityRequestCodes
import com.asda.android.app.dynamic.ProductPageUtils.createCustomRefinement
import com.asda.android.app.dynamic.ProductPageUtils.createDynamicFilter
import com.asda.android.app.view.AsdaFullScreenActivity
import com.asda.android.restapi.service.data.CustomNutritionFilterResponse
import com.asda.android.service.base.Authentication
import com.asda.android.utils.Utils
import com.asda.android.utils.view.startOnRestrictedAccess
import com.asda.android.utils.view.ViewUtil
import io.reactivex.observers.DisposableObserver


private const val TAG = "FiltersIntroActivity"

class CustomFiltersIntroActivity : AsdaFullScreenActivity() {
    private var viewModel: CustomFiltersViewModel? = null

    private var introText: TextView? = null
    private var container: LinearLayout? = null
    private var radioGroup: RadioGroup? = null
    private var progressDialog: ProgressDialog? = null
    private var contentPath: String? = null
    private var dialog: Dialog? = null
    private var screenName: String? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (!(Authentication.getInstance() as Authentication).isLoggedIn) {
            startOnRestrictedAccess(this@CustomFiltersIntroActivity, R.string.custom_filters, "", null)
        }
        Utils.addNewRefToLeakDetector(TAG, this)
        setContentView(R.layout.shop_custom_filters_intro)
        viewModel = ViewModelProviders.of(this).get(CustomFiltersViewModel::class.java)
        setupView()
        setHeaderText(getString(R.string.refine_title))
        contentPath = intent?.getStringExtra(DynamicFilterAdapter.EXTRA_CONTENT_PATH)
        screenName = intent?.getStringExtra(DynamicFilterAdapter.EXTRA_SCREEN_NAME)
        getNutritionFilters()
    }


    private fun setupView() {
        container = findViewById(R.id.custom_filter_container)
        introText = findViewById(R.id.filters_intro_text)
        radioGroup = findViewById(R.id.filter_radio_group)
    }

    private fun getNutritionFilters() {
        if (ShopFilters.instance.customNutritionFilters.isEmpty()) {
            Authentication.getInstance().userViewResponse?.filterList?.let {
                if (it.isNotEmpty()) {
                    ShopFilters.instance.customNutritionFilters.clear()
                    ShopFilters.instance.customNutritionFilters.addAll(it)
                    setFilters(ShopFilters.instance.customNutritionFilters)
                }
            }
        } else {
            setFilters(ShopFilters.instance.customNutritionFilters)
        }
        container?.invalidate()
    }

    override fun onResume() {
        if (!(Authentication.getInstance() as Authentication).isLoggedIn) {
            finish()
        } else {
            getNutritionFilters()
        }
        super.onResume()
    }

    @SuppressLint("ClickableViewAccessibility")
    private fun setFilters(filters: List<CustomNutritionFilterResponse.Filter>) {
        introText?.visibility = View.GONE
        radioGroup?.removeAllViews()
        filters.let {
            for (filter: CustomNutritionFilterResponse.Filter in it) {
                val button = AppCompatRadioButton(this@CustomFiltersIntroActivity)
                val params = RadioGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)
                button.text = filter.filterName
                button.tag = filter
                button.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.ic_baseline_more_vert, 0)
                val padding: Int = ViewUtil.dpToPixels(15, this@CustomFiltersIntroActivity)
                button.setPadding(padding, padding, padding, padding)
                button.layoutParams = params
                radioGroup?.addView(button)
                button.setOnTouchListener { view, event ->
                    val DRAWABLE_RIGHT = 2
                    if (event.action == MotionEvent.ACTION_UP && event.rawX >= button.right - button.compoundDrawables[DRAWABLE_RIGHT].bounds.width()) {
                        showDialog(filter, view.left + view.width * 2,
                              view.top + view.height * 2)
                        return@setOnTouchListener true
                    }
                    false
                }
            }
        }

        radioGroup?.setOnCheckedChangeListener { radioGroup, i ->
            radioGroup?.let {
                for (id: Int in 0..it.childCount) {
                    if (it.getChildAt(id)?.id == i) {
                        it.getChildAt(id)?.tag?.let {
                            ShopFilters.instance.selectedCustomNutritionFilter = it as CustomNutritionFilterResponse.Filter
                            addSelectedFilter(it)
                        }
                    }
                }
            }
            prepareReturnIntent()
        }
    }

    private fun addSelectedFilter(filter: CustomNutritionFilterResponse.Filter) {
        val categoryNutrition = AsdaFilter(DynamicFilterAdapter.NUTRITION_FACET)
        filter.filters?.forEach { customFilter: CustomNutritionFilterResponse.FilterDetails ->
            val refinement = createCustomRefinement(customFilter.name, contentPath, customFilter.dimensionId, categoryNutrition.name, false)
            val asdaFilter = createDynamicFilter(refinement, DynamicFilterAdapter.VIEW_TYPE_FILTER_ITEM_NUTRITION)
            ShopFilters.instance.addSelectedNutritionFilter(asdaFilter)
        }

    }

    public override fun getMenuResourceId(): Int {
        return R.menu.custom_filter
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            R.id.action_add_item -> {
                ShopFilters.instance.customNutritionFilter = null
                gotoCustomFilterActivity()
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    private fun gotoCustomFilterActivity() {
        try {
            dialog?.let {
                if (it.isShowing) {
                    it.dismiss()
                }
            }
        } catch (ex: Exception) {
            Log.w(TAG, ex)
        }
        val i = Intent(this, CustomFiltersActivity::class.java)
        i.putExtra(DynamicFilterAdapter.EXTRA_CONTENT_PATH, contentPath)
        i.putExtra(DynamicFilterAdapter.EXTRA_SCREEN_NAME, screenName)
        startActivityForResult(i, ActivityRequestCodes.REQUEST_CODE_APPLY_FILTERS)
    }

    private fun showDialog(filter: CustomNutritionFilterResponse.Filter?, x: Int, y: Int) {
        val dialogView = View.inflate(this, R.layout.filter_edit_layout, null)

        dialog = Dialog(this)
        dialog?.requestWindowFeature(Window.FEATURE_NO_TITLE)
        dialog?.setContentView(dialogView)

        ViewUtil.findViewById<View>(dialogView, R.id.filter_edit).setOnClickListener { _ ->
            filter?.let { ShopFilters.instance.customNutritionFilter = it }
            gotoCustomFilterActivity()
        }

        ViewUtil.findViewById<View>(dialogView, R.id.filter_delete).setOnClickListener { _ ->
            filter?.filterName?.let {
                try {
                    dialog?.let {
                        if (it.isShowing) {
                            it.dismiss()
                        }
                    }
                    progressDialog = ProgressDialog(this@CustomFiltersIntroActivity)
                    progressDialog?.setMessage(baseContext.getString(R.string.loading))
                    progressDialog?.show()
                } catch (ex: Exception) {
                    Log.w(TAG, ex)
                }

                safeAdd(viewModel?.deleteFilter(it)?.subscribeWith<DisposableObserver<CustomNutritionFilterResponse>>(object : DisposableObserver<CustomNutritionFilterResponse>() {
                    override fun onNext(customNutritionFilterResponse: CustomNutritionFilterResponse) {
                        customNutritionFilterResponse.filterList?.let {
                            if (!it.isEmpty()) {
                                ShopFilters.instance.customNutritionFilters.clear()
                                ShopFilters.instance.customNutritionFilters.addAll(it)
                                setFilters(ShopFilters.instance.customNutritionFilters)
                                Authentication.getInstance().userViewResponse?.filterList = it
                            }
                        }
                    }

                    override fun onError(e: Throwable) {
                        if (Utils.isDebug()) {
                            Log.e(TAG, "Delete Custom Filter onError" + e.message)
                        }
                        try {
                            progressDialog?.let {
                                if (it.isShowing) {
                                    it.dismiss()
                                }
                            }
                        } catch (ex: Exception) {
                            Log.w(TAG, ex)
                        }
                    }

                    override fun onComplete() {
                        if (Utils.isDebug()) {
                            Log.d(TAG, "Delete Custom Filter onComplete")
                        }
                        try {
                            progressDialog?.let {
                                if (it.isShowing) {
                                    it.dismiss()
                                }
                            }
                        } catch (ex: Exception) {
                            Log.w(TAG, ex)
                        }
                    }
                }))
            }
        }

        val window = dialog?.window
        val wlp = window?.attributes
        wlp?.gravity = Gravity.TOP or Gravity.CENTER_HORIZONTAL
        wlp?.flags = wlp?.flags?.let {
            it and WindowManager.LayoutParams.FLAG_DIM_BEHIND.inv()
        }
        wlp?.x = x
        wlp?.y = y
        window?.attributes = wlp

        container?.setOnClickListener { _ -> revealShow(dialogView, false, dialog) }

        dialog?.setOnShowListener { _ -> revealShow(dialogView, true, null) }

        dialog?.setOnKeyListener { _, i, _ ->
            if (i == KeyEvent.KEYCODE_BACK) {
                revealShow(dialogView, false, dialog)
                return@setOnKeyListener true
            }

            false
        }

        dialog?.show()
    }


    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    private fun revealShow(rootView: View, reveal: Boolean, dialog: Dialog?) {
        val view = rootView.findViewById<View>(R.id.filter_edit_container)

        if (reveal) {
            val cx = view.width
            val cy = 0

            // get the final radius for the clipping circle
            val finalRadius = Math.hypot(view.width.toDouble(), view.height.toDouble()).toFloat()

            // get the final radius for the clipping circle
            // int finalRadius = Math.max(view.getWidth(), view.getHeight()) / 2;

            // create the animator for this view (the start radius is zero)
            val anim = ViewAnimationUtils.createCircularReveal(view, cx, cy, 0f, finalRadius)

            // make the view visible and start the animation
            //   oldView.setVisibility(View.GONE);
            view.visibility = View.VISIBLE
            anim.interpolator = AccelerateDecelerateInterpolator()
            anim.duration = 500
            anim.start()

        } else {
            val cx = view.width
            val cy = 0

            // get the initial radius for the clipping circle
            val initialRadius = Math.hypot(view.width.toDouble(), view.height.toDouble()).toFloat()

            val anim = ViewAnimationUtils.createCircularReveal(view, cx, cy, initialRadius, 0f)

            anim.addListener(object : AnimatorListenerAdapter() {
                override fun onAnimationEnd(animation: Animator) {
                    super.onAnimationEnd(animation)
                    try {
                        dialog?.dismiss()
                    } catch (ex: Exception) {
                        Log.w(TAG, ex)
                    }
                    view.visibility = View.GONE
                }
            })
            anim.interpolator = AccelerateDecelerateInterpolator()
            anim.duration = 500
            anim.start()
        }

    }

    private fun createReturnIntent(applyFilters: Boolean): Intent {
        val returnIntent = Intent()
        returnIntent.putExtra(DynamicFilterActivity.EXTRA_APPLY_FILTERS, applyFilters)
        returnIntent.putExtra(DynamicFilterActivity.EXTRA_NUTRITION_ENABLED, true)
        returnIntent.putExtra(DynamicFilterActivity.EXTRA_HAS_CHANGE, true)
        setResult(Activity.RESULT_OK, returnIntent)
        return returnIntent
    }

    override fun onBackPressed() {
        prepareReturnIntent()
    }

    private fun prepareReturnIntent() {
        createReturnIntent(true)
        finish()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == ActivityRequestCodes.REQUEST_CODE_APPLY_FILTERS && resultCode == Activity.RESULT_OK) {
            createReturnIntent(true)
            finish()
        }
    }

    companion object {
        val EXTRA_FILTERS = "data_filters"
    }

}
