package com.asda.android.app.shop

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.text.method.LinkMovementMethod
import android.util.Log
import android.view.View
import android.view.ViewGroup
import android.view.accessibility.AccessibilityEvent
import android.widget.*
import androidx.annotation.StringRes
import androidx.annotation.VisibleForTesting
import androidx.core.content.ContextCompat
import androidx.core.view.AccessibilityDelegateCompat
import androidx.core.view.ViewCompat
import com.asda.android.R
import com.asda.android.abtestingframework.implementation.sitespect.metrics.SitespectP13NMetrics
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.ErrorEvent
import com.asda.android.analytics.events.exception.CaughtExceptionEvent
import com.asda.android.app.base.SiteSpectPreferences
import com.asda.android.base.core.ActivityRequestCodes
import com.asda.android.app.dynamic.ProductPageUtils
import com.asda.android.app.dynamic.manager.CMSUtils
import com.asda.android.app.main.AsdaApplication
import com.asda.android.app.shop.DynamicFilterAdapter.*
import com.asda.android.app.shop.FilterActivity.*
import com.asda.android.base.core.view.adapter.HeaderRecyclerViewAdapter
import com.asda.android.recipes.RecipeUtils
import com.asda.android.recipes.RecipesManager
import com.asda.android.recipes.datasource.PAGE_SIZE
import com.asda.android.restapi.app.siteconfig.SiteConfigDataSource
import com.asda.android.restapi.service.NetworkCallback
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.restapi.service.base.RecipeAPI
import com.asda.android.restapi.service.base.ServiceGenerator
import com.asda.android.restapi.service.data.SiteConfigResponse
import com.asda.android.restapi.service.data.ViewListResponse
import com.asda.android.restapi.service.data.ViewListResponse.Facet
import com.asda.android.restapi.service.data.cms.Refinement
import com.asda.android.restapi.service.data.recipes.SearchRecipeResponse
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.FavoritesManager
import com.asda.android.utils.view.ViewUtil
import io.reactivex.SingleObserver
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.disposables.Disposable
import io.reactivex.observers.ResourceObserver
import io.reactivex.schedulers.Schedulers

@StringRes
fun getSortTitle(type: Int): Int {
    return when (type) {
        ShelfListAdapter.SORT_NONE -> R.string.shop_sort_items_relevance
        ShelfListAdapter.SORT_DEPARTMENT -> R.string.shop_sort_items_department
        ShelfListAdapter.SORT_LOW_HIGH -> R.string.shop_sort_items_price_low_to_high
        ShelfListAdapter.SORT_HIGH_LOW -> R.string.shop_sort_items_price_high_to_low
        ShelfListAdapter.SORT_FAVORITES -> R.string.shop_sort_items_favourites
        else -> R.string.shop_sort_items_relevance
    }
}


const val SORT_RELEVANCE = "relevance+desc"
const val SORT_RELEVANCE_BASE = "relevance"
const val RECIPE_FAVORITE_DESC_SORT_ORDER = "descending"
const val RECIPE_DESC_SORT_ORDER = "desc"
const val RECIPE_ASC_SORT_ORDER = "asc"
const val RECIPE_FAVORITE_ASC_SORT_ORDER = "ascending"
const val PATTERN = "^[a-zA-Z\\s]+$"
const val FILTER_KEY_SORT_BY = "Sort by"


/**
 * Sort Modes
 *
 * @see ShelfListAdapter
 * int SORT_NONE = 0;
 * int SORT_LOW_HIGH = 1;
 * int SORT_HIGH_LOW = 2;
 * int SORT_FAVORITES = 3;
 * int SORT_DEPARTMENT = 4;
 */

class FilterAdapter(val context: Context?) : androidx.recyclerview.widget.RecyclerView.Adapter<FilterAdapter.ViewHolder>() {
    val TAG = "FilterAdapter"

    constructor(context: Context?, mode: String?, searchQuery: String?, sortBy: String?, activeFacets: Array<AsdaFilter>?, isSpecialOfferActive: Boolean, screenName: String) : this(context) {
        this.mode = mode
        this.specialOffer = isSpecialOfferActive
        this.searchQuery = searchQuery.orEmpty()
        this.screenName = screenName
        SORT_BY_TITLE = context?.getString(R.string.sort_by)
        SPECIAL_OFFERS_TITLE = context?.getString(R.string.special_offers)

        sortBy?.let {
            if (it.isNotEmpty()) {
                this.sortBy = it
            }
        }

        activeFacets?.let {
            this.activeFacets.addAll(it)
        }
        initAdapterFacets()
    }

    constructor(context: Context?, mode: String?, searchQuery: String?, sortBy: String?, sortIndex: Int?, sortOrder: String?, activeFacets: Array<AsdaFilter>?, screenName: String) : this(context) {
        this.mode = mode
        this.searchQuery = searchQuery.orEmpty()
        this.screenName = screenName
        SORT_BY_TITLE = context?.getString(R.string.sort_by)

        sortBy?.let {
            if (it.isNotEmpty()) {
                this.sortBy = it
            }
        }

        sortOrder?.let {
            if (it.isNotEmpty()) {
                this.sortOrder = it
            }
        }

        sortIndex?.let {
            this.sortMode = it
        }

        activeFacets?.let {
            this.activeFacets.addAll(it)
        }
        initAdapterFacets()
    }

    constructor(context: Context?, mode: String?, sortBy: Int, activeFacets: Array<AsdaFilter>?, screenName: String) : this(context) {
        this.mode = mode
        this.sortMode = sortBy
        this.screenName = screenName
        activeFacets?.let {
            this.activeFacets.addAll(it)
        }
        addFavoriteSortOptions()
        getFavoritesDepartment()
        addFavoriteFilters()
    }

    private var SORT_BY_TITLE: String? = null
    private var SPECIAL_OFFERS_TITLE: String? = null

    private val activeFacets = ArrayList<AsdaFilter>()

    private val mHiddenCategories = ArrayList<String>()

    private val compositeDisposable: CompositeDisposable = CompositeDisposable()

    var mode: String? = null

    var sortMode: Int = 1

    var sortBy = SORT_RELEVANCE

    var sortOrder = if (isFavoritesMode()) {
        RECIPE_FAVORITE_ASC_SORT_ORDER
    } else {
        RECIPE_ASC_SORT_ORDER
    }

    var searchQuery: String = ""

    var hasChanged: Boolean = false

    var maxSortIndex: Int = 5

    var specialOffer: Boolean = false

    var originFacets = ArrayList<AsdaFilter>()

    var loadingState: Boolean = true

    var nutritionEnabled: Boolean = false

    var screenName: String? = null

    val categoryFacets = ArrayList<String>()

    private fun initAdapterFacets() {
        loadFilters()
    }

    @VisibleForTesting
    fun addOriginalFacets(filters: ArrayList<AsdaFilter>, originalFacets: Array<AsdaFilter>) {
        if (isRecipesMode()) {
            for (facet in filters) {
                facet.category?.let {
                    if (it == facet.name) {
                        categoryFacets.add(it)
                    }
                }
            }
        }
        for (facet in originalFacets) {
            facet.category?.let {
                addFacets(categoryFacets, filters, facet)
            }
        }
    }

    private fun addFacets(categoryFacets: ArrayList<String>, filters: ArrayList<AsdaFilter>, facet: AsdaFilter) {
        facet.category?.let {
            if (!categoryFacets.contains(it)) {
                val categoryFacet = Facet()
                categoryFacet.facetName = it
                categoryFacet.displayName = it

                val categoryFilter = AsdaFilter(categoryFacet)

                filters.add(categoryFilter)
                filters.add(facet)
                categoryFacets.add(it)
            } else {
                if (!filters.contains(facet)) {
                    filters.add(facet)
                } else {
                    return
                }
            }
        }
    }

    private fun addSortOptions(listener: OnLoadingListener?) {
        SiteConfigDataSource.getSiteConfigResponse()?.subscribe(object : SingleObserver<SiteConfigResponse> {
            override fun onSubscribe(d: Disposable) {
                compositeDisposable.add(d)
                listener?.onLoad()
            }

            override fun onSuccess(configResponse: SiteConfigResponse) {
                val options = if (isRecipesMode()) {
                    configResponse.config?.recipeSortOptions?.options?.filter { it.id != context?.getString(R.string.recipe_cost_per_serving) }
                } else {
                    if (isFavoritesMode()) {
                        configResponse.config?.account?.favourite?.sort?.options?.filter { it.id != context?.getString(R.string.recipe_cost_per_serving) }
                    } else {
                        configResponse.config?.sortOptions?.options?.toList()
                    }
                }
                options?.let {
                    val sortFacetCategory = Facet()
                    sortFacetCategory.displayName = SORT_BY_TITLE
                    sortFacetCategory.facetName = SORT_BY_TITLE

                    val sortCategory = AsdaFilter(sortFacetCategory)
                    originFacets.add(sortCategory)
                    if (isRecipesMode()) {
                        val recipeSortOrder = AsdaFilter(isRecipesMode())
                        originFacets.add(recipeSortOrder)
                    }
                    maxSortIndex = 0

                    for (option: SiteConfigResponse.Option in it) {
                        val sortFacet = Facet()
                        sortFacet.displayName = option.displayName
                        sortFacet.facetName = SORT_BY_TITLE
                        sortFacet.value = option.id?.let {
                            var sortValue = it
                            if (sortValue.contains("-")) {
                                sortValue = sortValue.replace("-", "+")
                            }
                            if (sortValue.contains(":store_id")) {
                                sortValue = sortValue.replace(":store_id", "")
                            }
                            sortValue
                        }
                        sortFacet.isSortOption = true

                        val sortFilter = AsdaFilter(sortFacet)
                        originFacets.add(sortFilter)
                        maxSortIndex++
                    }
                } ?: run {
                    if (isRecipesMode()) {
                        maxSortIndex = 0
                    }
                }
                listener?.onComplete()
            }

            override fun onError(e: Throwable) {
                listener?.onComplete()
            }
        })
    }

    @VisibleForTesting
    fun addSpecialOfferFacet(filters: ArrayList<AsdaFilter>) {
        val specialOfferFacet = Facet()
        specialOfferFacet.displayName = SPECIAL_OFFERS_TITLE
        specialOfferFacet.facetName = SPECIAL_OFFERS_TITLE

        val specialOfferCategory = AsdaFilter(specialOfferFacet)

        filters.add(specialOfferCategory)

        val specialOffersFacet = Facet()
        specialOffersFacet.displayName = SPECIAL_OFFERS_TITLE
        specialOffersFacet.facetName = SPECIAL_OFFERS_TITLE
        specialOffersFacet.isSpecialOffer = true

        val specialOfferFilter = AsdaFilter(specialOffersFacet)

        filters.add(specialOfferFilter)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val view: View
        val viewHolder: ViewHolder
        when (viewType) {
            VIEW_TYPE_CATEGORY -> {
                view = inflateView(context, R.layout.shelf_filter_list_entry_category, parent, false)
                viewHolder = ViewHolder(view, viewType)

                viewHolder.containerView?.setOnClickListener { _ ->
                    if (viewHolder.text?.tag is String) {
                        val categoryName = viewHolder.text.tag as String
                        toggleCategory(categoryName)
                    }
                }
            }
            VIEW_TYPE_FILTER_ITEM_NUTRITION -> {
                view = inflateView(context, R.layout.shelf_filter_list_entry_filter, parent, false)
                viewHolder = ViewHolder(view, viewType)

                viewHolder.checkBox?.setOnClickListener { _ ->
                    hasChanged = true
                    val position = viewHolder.adapterPosition
                    if (position != androidx.recyclerview.widget.RecyclerView.NO_POSITION) {
                        val selectedFacet = originFacets[position]
                        if (getSelectedNutritionFilters().contains(selectedFacet)) {
                            activeFacets.remove(selectedFacet)
                            getSelectedNutritionFilters().remove(selectedFacet)
                        } else {
                            activeFacets.add(selectedFacet)
                            getSelectedNutritionFilters().add(selectedFacet)
                        }
                    }
                }
            }
            VIEW_TYPE_FILTER_ITEM_RECIPES -> {
                view = inflateView(context, R.layout.shelf_filter_list_entry_filter, parent, false)
                viewHolder = ViewHolder(view, viewType)

                viewHolder.checkBox?.setOnClickListener { _ ->
                    hasChanged = true
                    val position = viewHolder.adapterPosition
                    if (position != androidx.recyclerview.widget.RecyclerView.NO_POSITION) {
                        val selectedFacet = originFacets[position]
                        if (selectedFacet.isSpecialOffer) {
                            specialOffer = !specialOffer
                        } else {
                            if (activeFacets.contains(selectedFacet)) {
                                activeFacets.remove(selectedFacet)
                            } else {
                                if (isFavoritesMode()) {
                                    var departmentTitle = context?.getString(R.string.department)
                                    departmentTitle?.let {
                                        for (filter in activeFacets) {
                                            if (filter.category == it) {
                                                activeFacets.remove(filter)
                                            }
                                        }
                                    }
                                }
                                activeFacets.add(selectedFacet)
                            }
                        }
                    }
                }
            }
            VIEW_TYPE_FILTER_ITEM, VIEW_TYPE_FILTER_ITEM_SPECIAL_OFFER -> {
                view = inflateView(context, R.layout.shelf_filter_list_entry_sort, parent, false)
                viewHolder = ViewHolder(view, viewType)

                viewHolder.radioButton?.setOnClickListener { _ ->
                    hasChanged = true
                    val position = viewHolder.adapterPosition
                    if (position != androidx.recyclerview.widget.RecyclerView.NO_POSITION) {
                        val selectedFacet = originFacets[position]
                        if (selectedFacet.isSpecialOffer) {
                            specialOffer = !specialOffer
                            notifyDataSetChanged()
                        } else {
                            if (activeFacets.contains(selectedFacet)) {
                                activeFacets.remove(selectedFacet)
                            } else {
                                if (isFavoritesMode()) {
                                    var departmentTitle = context?.getString(R.string.department)
                                    departmentTitle?.let {
                                        for (filter in activeFacets) {
                                            if (filter.category == it) {
                                                activeFacets.remove(filter)
                                            }
                                        }
                                    }
                                }
                                activeFacets.add(selectedFacet)
                            }
                            notifyDataSetChanged()
                            if (DynamicFilterAdapter.NUTRITION_FACET != selectedFacet.category || mode != FILTER_MODE_FAVORITES) {
                                loadFilters()
                            }
                        }
                    }
                }
            }
            VIEW_TYPE_NUTRITION_HEADER -> {
                view = inflateView(context, R.layout.shelf_filter_nutrition_header, parent, false)
                viewHolder = ViewHolder(view, viewType)
                viewHolder.text?.setOnClickListener { _ ->
                    val i = Intent(context, CustomFiltersIntroActivity::class.java)
                    if (context is Activity) {
                        i.putExtra(EXTRA_SELECT_FILTER, true)
                        i.putExtra(EXTRA_SCREEN_NAME, screenName)
                        context.startActivityForResult(i, ActivityRequestCodes.REQUEST_CODE_APPLY_FILTERS)
                    }
                }
                if (ShopFilters.instance.customNutritionFilters.isEmpty()) {
                    Authentication.getInstance().userViewResponse?.filterList?.let {
                        if (it.isNotEmpty()) {
                            ShopFilters.instance.customNutritionFilters.addAll(it)
                            viewHolder.customFilter?.text = context?.getString(R.string.custom_filters_count, it.size.toString())
                        } else {
                            viewHolder.customFilter?.text = context?.getString(R.string.custom_filters)
                        }
                    }
                } else {
                    viewHolder.customFilter?.text = context?.getString(R.string.custom_filters_count, ShopFilters.instance.customNutritionFilters.size.toString())
                }
                viewHolder.switch?.setOnCheckedChangeListener(checkChangeListener(view, viewHolder))
            }
            VIEW_TYPE_UNAVAILABLE_FILTER_NOTE -> {
                view = inflateView(context, R.layout.shelf_filter_custom_view, parent, false)
                viewHolder = ViewHolder(view, viewType)
            }
            VIEW_TYPE_SORT_ITEM -> {
                view = inflateView(context, R.layout.shelf_filter_list_entry_sort, parent, false)
                viewHolder = ViewHolder(view, viewType)
                viewHolder.radioButton?.setOnClickListener {
                    hasChanged = true
                    val position = viewHolder.adapterPosition
                    if (position >= 0 && position < originFacets.size) {
                        originFacets[position].singleDimensionId?.let {
                            sortBy = it
                        }
                        if (isFavoritesMode()) {
                            sortMode = position
                            if (sortMode != 1) {
                                activeFacets.clear()
                            }
                        }
                        notifyDataSetChanged()
                    }
                }
            }
            VIEW_TYPE_RECIPE_SORT_ORDER -> {
                view = inflateView(context, R.layout.recipe_sort_order_layout, parent, false)
                viewHolder = ViewHolder(view, viewType)
                viewHolder.recipeSortOrder?.setOnCheckedChangeListener { _, checkedId ->
                    when (checkedId) {
                        R.id.ascending -> {
                            sortOrder = if (isFavoritesMode()) {
                                RECIPE_FAVORITE_ASC_SORT_ORDER
                            } else {
                               RECIPE_ASC_SORT_ORDER
                            }
                        }
                        R.id.descending -> {
                            sortOrder = if (isFavoritesMode()) {
                              RECIPE_FAVORITE_DESC_SORT_ORDER
                            } else {
                               RECIPE_DESC_SORT_ORDER
                            }
                        }
                    }
                }
            }
            else -> {
                view = inflateView(context, R.layout.shelf_filter_list_entry_sort, parent, false)
                viewHolder = ViewHolder(view, viewType)
                viewHolder.radioButton?.setOnClickListener {
                    hasChanged = true
                    val position = viewHolder.adapterPosition
                    if (position >= 0 && position < originFacets.size) {
                        originFacets[position].singleDimensionId?.let {
                            sortBy = it
                        }
                        if (isFavoritesMode()) {
                            sortMode = position
                        }
                        notifyDataSetChanged()
                    }
                }
            }
        }

        return viewHolder
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        val viewType = holder.itemViewType
        val facet = originFacets[position]

        holder.text?.let { textView ->
            ViewCompat.setAccessibilityDelegate(textView, object : AccessibilityDelegateCompat() {
            override fun onPopulateAccessibilityEvent(host: View, event: AccessibilityEvent) {
                super.onPopulateAccessibilityEvent(host, event)
                if (event.eventType == AccessibilityEvent.TYPE_VIEW_ACCESSIBILITY_FOCUSED) {
                    if (mHiddenCategories.contains(facet.category))
                        holder.let { it.text?.announceForAccessibility(context?.getString(R.string.ally_filter_collapsed))}
                    else holder.let { it.text?.announceForAccessibility(context?.getString(R.string.ally_filter_expand)) }
                }
            }
        })
        }

        if (viewType != VIEW_TYPE_CATEGORY) {
            holder.containerView?.visibility =
                    if (facet.isVisible) {
                        View.VISIBLE
                    } else {
                        View.GONE
                    }
        }
        when (viewType) {
            VIEW_TYPE_CATEGORY -> {
                if (position == 0 && maxSortIndex > 0) {
                    holder.text?.text = context?.getString(R.string.sort_by)
                } else {
                    facet.name?.let {
                        if (NUTRITION_FACET == it) {
                            holder.text?.text = context?.getString(R.string.filter_by, NUTRITION_FILTER_NAME)
                        } else {
                            holder.text?.text = context?.getString(R.string.filter_by, it)
                        }
                    }
                }
                holder.text?.tag = facet.category
                context?.let {
                    val collapseArrow = ContextCompat.getDrawable(it,
                            if (mHiddenCategories.contains(facet.category)) {
                                R.drawable.ic_arrow_up
                            } else {
                                R.drawable.ic_arrow_down
                            })
                    holder.text?.setCompoundDrawablesWithIntrinsicBounds(null, null, collapseArrow, null)
                }
            }
            VIEW_TYPE_FILTER_ITEM, VIEW_TYPE_FILTER_ITEM_SPECIAL_OFFER -> {
                val isSelected = activeFacets.contains(facet)
                val displayName: String? = if (isSelected || viewType == VIEW_TYPE_FILTER_ITEM_SPECIAL_OFFER) {
                    facet.name
                } else {
                    facet.refinement?.count?.let {
                        facet.name + "(" + it + ")"
                    } ?: run {
                        facet.name
                    }
                }
                holder.text?.text = displayName
                if (facet.isSpecialOffer) {
                    holder.radioButton?.isChecked = specialOffer
                } else {
                    holder.radioButton?.isChecked = activeFacets.contains(facet)
                }

            }
            VIEW_TYPE_UNAVAILABLE_FILTER_NOTE -> {
                holder.text?.text = context?.getString(R.string.unavailable_filters)
            }
            VIEW_TYPE_NUTRITION_HEADER -> {
                val rightArrow = context?.resources?.getDrawable(R.drawable.ic_arrow_right)
                holder.nutritionNote?.text = RestUtils.fromHtml(context?.getString(R.string.filters_note))
                holder.nutritionNote?.movementMethod = LinkMovementMethod.getInstance()
                holder.text?.setCompoundDrawablesWithIntrinsicBounds(null, null, rightArrow, null)
                holder.text?.text = context?.getString(R.string.selected_none)
                holder.switch?.isChecked = ShopFilters.instance.selectedCustomNutritionFilter != null
                if (ShopFilters.instance.selectedCustomNutritionFilter != null) {
                    holder.text?.text = context?.getString(R.string.selected_filter, ShopFilters.instance.selectedCustomNutritionFilter?.filterName)
                }
            }
            VIEW_TYPE_FILTER_ITEM_NUTRITION -> {
                holder.checkBox?.tag = facet
                holder.text?.text = ShopFilters.instance.nutritionMap[facet.name]?.second
                val isSelected = ShopFilters.instance.selectedNutritionFilters.contains(facet)
                if (ShopFilters.instance.nutritionMap.containsKey(facet.name) && !facet.isActive) {
                    holder.checkBox?.isEnabled = false
                    holder.checkBox?.isChecked = false
                    holder.text?.alpha = 0.4F
                    context?.resources?.getColor(R.color.grey_53)?.let {
                        holder.text?.setTextColor(it)
                    }
                } else {
                    holder.checkBox?.isEnabled = true
                    holder.checkBox?.isChecked = isSelected
                    holder.text?.alpha = 1F
                    holder.text?.setBackgroundColor(Color.WHITE)
                }
            }
            VIEW_TYPE_FILTER_ITEM_RECIPES -> {
                holder.checkBox?.tag = facet
                val isSelected = activeFacets.contains(facet)
                holder.text?.text = if (isSelected || viewType == VIEW_TYPE_FILTER_ITEM_SPECIAL_OFFER) {
                    facet.name
                } else {
                    facet.refinement?.count?.let {
                        facet.name + "(" + it + ")"
                    } ?: run {
                        facet.name
                    }
                }
                holder.checkBox?.isChecked = isSelected
            }
            VIEW_TYPE_SORT_ITEM -> {
                holder.text?.text = facet.name
                if (isFavoritesMode()) {
                    if (holder.adapterPosition != androidx.recyclerview.widget.RecyclerView.NO_POSITION) {
                        holder.radioButton?.isChecked = holder.adapterPosition == sortMode
                    }
                } else {
                    holder.radioButton?.isChecked = facet.singleDimensionId?.let {
                        it == sortBy
                    } ?: run {
                        false
                    }
                }
            }
            VIEW_TYPE_RECIPE_SORT_ORDER -> {
                if (isRecipesMode()) {
                    holder.recipeSortOrder?.visibility = View.VISIBLE
                    when (sortOrder) {
                        RECIPE_ASC_SORT_ORDER, RECIPE_FAVORITE_ASC_SORT_ORDER -> (holder.recipeSortOrder?.getChildAt(0) as RadioButton).isChecked = true
                        RECIPE_DESC_SORT_ORDER, RECIPE_FAVORITE_DESC_SORT_ORDER -> (holder.recipeSortOrder?.getChildAt(1) as RadioButton).isChecked = true
                    }
                }
            }
            else -> {
                holder.text?.text = facet.name
                holder.radioButton?.isChecked = facet.singleDimensionId?.let {
                    it == sortBy
                } ?: run {
                    false
                }
            }
        }
    }

    override fun getItemCount(): Int {
        return originFacets.size
    }

    override fun getItemViewType(position: Int): Int {
        return originFacets[position].type
    }

    private fun checkChangeListener(view: View?, viewHolder: ViewHolder) = CompoundButton.OnCheckedChangeListener { _, isChecked ->
        val i = Intent(view?.context, CustomFiltersIntroActivity::class.java)
        if (isChecked) {
            if (ShopFilters.instance.selectedCustomNutritionFilter == null && view?.context is Activity) {
                i.putExtra(EXTRA_SELECT_FILTER, true)
                i.putExtra(EXTRA_SCREEN_NAME, screenName)
                (view.context as Activity).startActivityForResult(i, ActivityRequestCodes.REQUEST_CODE_APPLY_FILTERS)
            }
        } else {
            viewHolder.text?.text = view?.context?.getString(R.string.selected_none)
            clearAll()
        }
    }

    class ViewHolder(itemView: View, viewType: Int) : HeaderRecyclerViewAdapter.BaseViewHolder(itemView, viewType) {
        val nutritionNote: TextView?
        val text: TextView?
        val customFilter: TextView?
        val checkBox: CompoundButton?
        val radioButton: RadioButton?
        val containerView: View?
        val switch: Switch?
        val recipeSortOrder: RadioGroup?

        init {
            when (viewType) {
                VIEW_TYPE_CATEGORY -> {
                    text = itemView as TextView
                    containerView = text
                    customFilter = null
                    nutritionNote = null
                    checkBox = null
                    radioButton = null
                    switch = null
                    recipeSortOrder = null
                }
                VIEW_TYPE_FILTER_ITEM, VIEW_TYPE_FILTER_ITEM_SPECIAL_OFFER -> {
                    text = ViewUtil.findViewById(itemView, R.id.text)
                    containerView = ViewUtil.findViewById(itemView, R.id.container_view)
                    nutritionNote = null
                    customFilter = null
                    radioButton = ViewUtil.findViewById(itemView, R.id.sort_radio_button)
                    checkBox = null
                    switch = null
                    recipeSortOrder = null
                }
                VIEW_TYPE_FILTER_ITEM_NUTRITION, VIEW_TYPE_FILTER_ITEM_RECIPES -> {
                    text = ViewUtil.findViewById(itemView, R.id.text)
                    containerView = ViewUtil.findViewById(itemView, R.id.container_view)
                    nutritionNote = null
                    customFilter = null
                    checkBox = ViewUtil.findViewById(itemView, R.id.filter_option_checkbox)
                    radioButton = null
                    switch = null
                    recipeSortOrder = null
                }
                VIEW_TYPE_NUTRITION_HEADER -> {
                    containerView = ViewUtil.findViewById(itemView, R.id.container_view)
                    nutritionNote = ViewUtil.findViewById(itemView, R.id.nutrition_note)
                    text = ViewUtil.findViewById(itemView, R.id.selected_filters)
                    customFilter = ViewUtil.findViewById(itemView, R.id.custom_filters)
                    switch = ViewUtil.findViewById(itemView, R.id.custom_filter_switch)
                    radioButton = null
                    checkBox = null
                    recipeSortOrder = null
                }
                VIEW_TYPE_UNAVAILABLE_FILTER_NOTE -> {
                    text = ViewUtil.findViewById(itemView, R.id.text_container)
                    nutritionNote = null
                    customFilter = null
                    containerView = text
                    switch = null
                    checkBox = null
                    radioButton = null
                    recipeSortOrder = null
                }
                VIEW_TYPE_RECIPE_SORT_ORDER -> {
                    text = null
                    nutritionNote = null
                    customFilter = null
                    containerView = null
                    switch = null
                    checkBox = null
                    radioButton = null
                    recipeSortOrder = ViewUtil.findViewById(itemView, R.id.sort_order_group)
                }
                else -> {
                    //Sort
                    nutritionNote = ViewUtil.findViewById(itemView, R.id.nutrition_note)
                    text = ViewUtil.findViewById(itemView, R.id.text)
                    containerView = ViewUtil.findViewById(itemView, R.id.container_view)
                    radioButton = ViewUtil.findViewById(itemView, R.id.sort_radio_button)
                    customFilter = ViewUtil.findViewById(itemView, R.id.custom_filters)
                    checkBox = null
                    switch = null
                    recipeSortOrder = null
                }
            }
        }
    }

    fun clearAll() {
        // Make sure we don't unnecessarily clearall and this is causing an IllegalStateException
        // as this seems to be called too early some times.
        if (activeFacets.isEmpty() && sortBy == SORT_RELEVANCE_BASE) {
            val errorMsg = "Calling clearAll when activeFacets is empty"
            Log.e(TAG, errorMsg)
            Tracker.get().trackEvent(ErrorEvent(Anivia.UNEXPECTED_ERROR_EVENT, errorMsg))
            return
        }

        try {
            hasChanged = true
            when {
                isRecipesMode() -> {
                    sortOrder = if (!isFavoritesMode()) {
                        RECIPE_ASC_SORT_ORDER
                    } else {
                        RECIPE_FAVORITE_ASC_SORT_ORDER
                    }
                    sortMode = 2
                    sortBy = SORT_RELEVANCE_BASE

                    val facets = activeFacets.iterator()
                    while (facets.hasNext()) {
                        val facet = facets.next()
                        if (facet.name != com.asda.android.restapi.service.data.recipes.Facet.DISPLAY_NAME) {
                            facets.remove()
                        }
                    }
                }
                isFavoritesMode() -> {
                    activeFacets.clear()
                    sortMode = 1
                }
                else -> {
                    activeFacets.clear()
                    sortBy = SORT_RELEVANCE
                }
            }
            categoryFacets.clear()
            getSelectedNutritionFilters().clear()
            specialOffer = false
            loadFilters()
            ShopFilters.instance.selectedCustomNutritionFilter = null
            notifyDataSetChanged()
        } catch (exception:Exception) {
            Tracker.get().trackException(CaughtExceptionEvent(exception))
        }
    }

    private fun isFavoritesMode() = mode == FILTER_MODE_FAVORITES || mode == FILTER_MODE_FAVORITES_RECIPES

    private fun isRecipesMode() = mode == FILTER_MODE_SEARCH_RECIPES || mode == FILTER_MODE_FAVORITES_RECIPES

    fun toggleCategory(categoryName: String?) {
        if (!categoryName.isNullOrEmpty()) {
            if (mHiddenCategories.contains(categoryName)) {
                showCategory(categoryName)
            } else {
                hideCategory(categoryName)
            }
            notifyDataSetChanged()
        }
    }

    private fun hideCategory(categoryName: String?) {
        if (!categoryName.isNullOrEmpty()) {
            for (facet in originFacets) {
                if (categoryName == facet.category) {
                    facet.isVisible = false
                    if (!mHiddenCategories.contains(categoryName)) {
                        mHiddenCategories.add(categoryName!!)
                    }
                }
            }
        }
    }

    private fun showCategory(categoryName: String?) {
        if (!categoryName.isNullOrEmpty() && mHiddenCategories.contains(categoryName)) {
            for (facet in originFacets) {
                if (categoryName == facet.category) {
                    facet.isVisible = true
                    if (mHiddenCategories.contains(categoryName)) {
                        mHiddenCategories.remove(categoryName)
                    }
                }
            }
        }
    }

    private fun addSelectedFilters(filters: ArrayList<AsdaFilter>, selectedFacets: Array<AsdaFilter>) {
        addOriginalFacets(filters, selectedFacets)
    }

    @VisibleForTesting
    fun addFavoriteFilters() {
        val filters = ArrayList<AsdaFilter>()
        val nutritionCategory = AsdaFilter(NUTRITION_FACET)
        nutritionCategory.type = VIEW_TYPE_CATEGORY
        val nutritionHeader = AsdaFilter(NUTRITION_FACET)
        nutritionHeader.type = VIEW_TYPE_NUTRITION_HEADER
        filters.add(nutritionCategory)
        filters.add(nutritionHeader)
        if (ShopFilters.instance.favoritesAvailableFilters.size > 0) {
            filters.addAll(ShopFilters.instance.favoritesAvailableFilters)
        } else {
            clearAll()
        }
        val unavailableHeader = AsdaFilter(NUTRITION_FACET)
        unavailableHeader.type = VIEW_TYPE_UNAVAILABLE_FILTER_NOTE
        if (ShopFilters.instance.favoritesUnavailableFilters.size > 0) {
            filters.add(unavailableHeader)
            filters.addAll(ShopFilters.instance.favoritesUnavailableFilters)
            ShopFilters.instance.addAllNutritionFilters(filters)
        }
        originFacets.addAll(filters)
        notifyDataSetChanged()
    }

    @VisibleForTesting(otherwise = VisibleForTesting.PRIVATE)
    fun loadFilters() {
        if (isRecipesMode()) {
            ServiceGenerator.createService(AsdaServiceSettings.BASE_URL, "", RecipeAPI::class.java)?.let { recipeApi ->
                loadingState = true
                val facetParam = RecipeUtils.createFacetParam(getActiveRecipeFacets(), isFavoritesMode())
                val observable = when {
                    mode == FILTER_MODE_FAVORITES_RECIPES -> recipeApi.getFavoriteRecipes(RecipesManager.instance?.authentication?.profileId
                            ?: "", 1, PAGE_SIZE, sortBy, sortOrder, facetParam)
                            .map { SearchRecipeResponse(null, null, null, it.payload?.facets) }
                    searchQuery.isEmpty() -> recipeApi.listRecipes(null, PAGE_SIZE, sortBy, sortOrder, facetParam, RecipesManager.instance?.asdaService?.shipDate)
                    else -> recipeApi.searchRecipes(searchQuery
                            , "1", "1", sortBy, sortOrder, facetParam)
                }
                observable.subscribeOn(Schedulers.io())
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe(object : ResourceObserver<SearchRecipeResponse>() {
                            override fun onComplete() {
                                dispose()
                            }

                            override fun onNext(response: SearchRecipeResponse) {
                                originFacets.clear()
                                addSortOptions(object : OnLoadingListener {
                                    override fun onLoad() {
                                        //no need to handle

                                    }

                                    override fun onComplete() {
                                        val filters = ArrayList<AsdaFilter>()
                                        addSelectedFilters(filters, getActiveFacets())
                                        response.facets?.let {
                                            val asdaFilters = ArrayList<AsdaFilter>()
                                            for (facet in it) {
                                                if (facet.dimensionName != context?.getString(R.string.facet_recipe_cost_per_serving)) {
                                                    val filter = AsdaFilter(facet)
                                                    asdaFilters.add(filter)
                                                }
                                            }
                                            addOriginalFacets(filters, asdaFilters.toTypedArray())
                                        }
                                        val filterByCategory: HashMap<String?, ArrayList<AsdaFilter>> = HashMap()
                                        for (filter in filters) {
                                            if (!filterByCategory.containsKey(filter.category)) {
                                                when (isFavoritesMode()) {
                                                    true -> filterByCategory[filter.category] = ArrayList()
                                                    false -> {
                                                        if (filter.category?.matches((PATTERN).toRegex()) == true) filterByCategory[filter.category] = ArrayList()
                                                    }
                                                }
                                            }
                                            filterByCategory[filter.category]?.add(filter)
                                        }

                                        filterByCategory.get(FILTER_KEY_SORT_BY)?.let { originFacets.addAll(it) }
                                        for (category in categoryFacets) {
                                            if (category != FILTER_KEY_SORT_BY && (filterByCategory[category]?.let { originFacets.containsAll(it) } == false)) {
                                                filterByCategory[category]?.let { originFacets.addAll(it) }
                                            }
                                        }
                                        loadingState = false
                                        notifyDataSetChanged()
                                    }
                                })
                            }

                            override fun onError(e: Throwable) {
                                dispose()
                                loadingState = false
                            }
                        })
            }
        } else {
            val callback = object : NetworkCallback<ViewListResponse>() {

                override fun onFailure(error: Int?, data: ViewListResponse) {
                    //ignore
                }

                override fun onSuccess(data: ViewListResponse) {
                    originFacets.clear()
                    addSortOptions(object : OnLoadingListener {

                        override fun onLoad() {
                            loadingState = true
                        }

                        override fun onComplete() {
                            val filters = ArrayList<AsdaFilter>()
                            addSpecialOfferFacet(filters)
                            addSelectedFilters(filters, getActiveFacets())
                            data.facets?.let {
                                val asdaFilters = ArrayList<AsdaFilter>()
                                for (facet in it) {
                                    val filter = AsdaFilter(facet)
                                    asdaFilters.add(filter)
                                }
                                addOriginalFacets(filters, asdaFilters.toTypedArray())
                            }
                            var indexOfNutrition = -1
                            for (filter in filters) {
                                filter.category?.let {
                                    if (it == NUTRITION_FACET) {
                                        if (indexOfNutrition == -1) {
                                            indexOfNutrition = filters.indexOf(filter)
                                        }
                                        if (filter.type == VIEW_TYPE_FILTER_ITEM_NUTRITION && !ShopFilters.instance.nutritionFilters.contains(filter)) {
                                            ShopFilters.instance.nutritionFilters.add(filter)
                                        }
                                    }
                                }
                            }
                            if (indexOfNutrition >= 0) {
                                addNutritionHeader(indexOfNutrition + 1, filters)
                            }

                            addSelectedNutritionFilters(filters)
                            if (indexOfNutrition >= 0 || ShopFilters.instance.selectedNutritionFilters.size >= 1) {
                                addUnavailableFilters(filters)
                            }

                            ShopFilters.instance.sortFilter.addAll(filters)
                            originFacets.addAll(filters)
                            loadingState = false
                            notifyDataSetChanged()
                        }
                    })
                }
            }

            val facetParam = ProductPageUtils.createFacetParam(getActiveFacets())
            val testingEngine = AsdaApplication.getComponent().sitespectAbTestEngine
            AsdaServiceFactory.get().itemSearch(testingEngine.isP13NSearchEnabled, searchQuery, sortBy, 1, facetParam, callback)
            if (Authentication.getInstance().isLoggedIn &&
                  SiteSpectPreferences().getSiteSpectUserID(context).isNotEmpty()) {
                if (testingEngine.isP13NSearchEnabled) {
                    testingEngine.trackMetrics(SitespectP13NMetrics.P13N_SEARCH_VARIATION_GROUP,
                            CMSUtils.getP13NImpressionMetric(""),
                            1, null, false).subscribe()
                } else {
                    testingEngine.trackMetrics(SitespectP13NMetrics.P13N_SEARCH_CONTROL_GROUP,
                            CMSUtils.getP13NImpressionMetric(""),
                            1, null, false).subscribe()
                }
            }
        }
    }

    @VisibleForTesting
    fun addNutritionHeader(position: Int, filters: ArrayList<AsdaFilter>) {
        if (position >= 0 && position <= filters.size) {
            val nutritionHeader = AsdaFilter(NUTRITION_FACET)
            nutritionHeader.type = VIEW_TYPE_NUTRITION_HEADER
            filters.add(position, nutritionHeader)
        }
    }

    @VisibleForTesting
    fun addUnavailableFilters(filters: ArrayList<AsdaFilter>) {
        val unavailableFilters = ArrayList<AsdaFilter>()
        val unavailableHeader = AsdaFilter(NUTRITION_FACET)
        unavailableHeader.type = VIEW_TYPE_UNAVAILABLE_FILTER_NOTE
        if (!unavailableFilters.contains(unavailableHeader)) {
            unavailableFilters.add(unavailableHeader)
        }
        for ((key) in ShopFilters.instance.nutritionMap) {
            val refinement = Refinement()
            refinement.label = key
            refinement.category = NUTRITION_FACET
            val unavailableFilter = AsdaFilter(refinement)
            unavailableFilter.type = VIEW_TYPE_FILTER_ITEM_NUTRITION
            if (!filters.contains(unavailableFilter)) {
                unavailableFilter.isActive = false
                unavailableFilters.add(unavailableFilter)
            }
        }

        if (unavailableFilters.size > 0) {
            filters.addAll(unavailableFilters)
            ShopFilters.instance.addAllNutritionFilters(unavailableFilters)
        }
    }

    fun addSelectedNutritionFilters(filters: ArrayList<AsdaFilter>) {
        var lastIndexAdded = -1
        for (selectedFilter in getSelectedNutritionFilters()) {
            var hasBeenAdded = false
            if (selectedFilter.type != -1) {
                selectedFilter.category?.let {
                    for (filter in filters) {
                        if (filter.type == VIEW_TYPE_CATEGORY && it == filter.category) {
                            val categoryIndex = filters.indexOf(filter)
                            if (lastIndexAdded == -1) {
                                //Plus 2 because we have a header before the actual list.
                                lastIndexAdded = categoryIndex + 2
                            }
                            if (!filters.contains(selectedFilter)) {
                                filters.add(lastIndexAdded++, selectedFilter)
                            }
                            hasBeenAdded = true
                            break
                        }
                    }
                }
            }

            if (!hasBeenAdded) {
                selectedFilter.category?.let {
                    val categoryFilter = AsdaFilter(it)
                    filters.add(categoryFilter)
                    addNutritionHeader(filters.size, filters)
                    if (!filters.contains(selectedFilter)) {
                        filters.add(selectedFilter)
                    }
                    lastIndexAdded = filters.size
                }
            }
        }
    }

    fun getActiveFacets(): Array<AsdaFilter> {
        return activeFacets.toArray(Array(activeFacets.size, { AsdaFilter() }))
    }

    fun getActiveRecipeFacets(): Array<com.asda.android.restapi.service.data.recipes.Facet> {
        return activeFacets.map { com.asda.android.restapi.service.data.recipes.Facet(it.category, it.singleDimensionId, it.refinement?.label.toString(), it.name, it.name, it.refinement?.count) }.toTypedArray()
    }

    fun getSelectedNutritionFilters(): ArrayList<AsdaFilter> {
        return ShopFilters.instance.selectedNutritionFilters
    }

    fun hasActiveFilters(): Boolean {
        return activeFacets.isNotEmpty() || getSelectedNutritionFilters().size > 0 || sortBy != SORT_RELEVANCE
    }

    fun onDestroy() {
        compositeDisposable.dispose()
    }

    @VisibleForTesting
    fun addFavoriteSortOptions() {
        val sortCategory = AsdaFilter(FILTER_KEY_SORT_BY)
        originFacets.add(sortCategory)
        val sortDepartment = AsdaFilter(FILTER_KEY_SORT_BY, context?.getString(R.string.department), FavoritesManager.DEPT_SORT, true)
        originFacets.add(sortDepartment)
        val sortFrequency = AsdaFilter(FILTER_KEY_SORT_BY, context?.getString(R.string.purchase_frequency), FavoritesManager.FREQUENCY_SORT, true)
        originFacets.add(sortFrequency)
        val sortAlphabetical = AsdaFilter(FILTER_KEY_SORT_BY, context?.getString(R.string.alphabetical), FavoritesManager.TITLE_SORT, true)
        originFacets.add(sortAlphabetical)
        val sortPriceLowHigh = AsdaFilter(FILTER_KEY_SORT_BY, context?.getString(R.string.price_low_high), FavoritesManager.PRICE_ASC_SORT, true)
        originFacets.add(sortPriceLowHigh)
        val sortPriceHighLow = AsdaFilter(FILTER_KEY_SORT_BY, context?.getString(R.string.price_high_low), FavoritesManager.PRICE_DESC_SORT, true)
        originFacets.add(sortPriceHighLow)
        val pricePurchaseDate = AsdaFilter(FILTER_KEY_SORT_BY, context?.getString(R.string.purchase_date), FavoritesManager.DATE_SORT, true)
        originFacets.add(pricePurchaseDate)
    }

    private fun getFavoritesDepartment() {
        val favoriteManager: FavoritesManager = FavoritesManager.get(context)
        val disposable = favoriteManager.departments.subscribe { response ->
            val departmentName = context?.getString(R.string.department)
            departmentName?.let {
                val departmentCategory = AsdaFilter(departmentName)
                originFacets.add(departmentCategory)
                for (item in response) {
                    val departmentFilter = AsdaFilter(it, item.key, item.value, false)
                    originFacets.add(departmentFilter)
                }
                notifyDataSetChanged()
            }
        }
        compositeDisposable.add(disposable)
    }

    @VisibleForTesting
    fun inflateView(context:Context?, resId:Int, parent:ViewGroup, attachToRoot:Boolean):View{
        return ViewUtil.inflate(context, resId, parent, attachToRoot)
    }
}
