package com.asda.android.app.shop

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.fragment.app.Fragment
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Observer
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.analytics.info.AnalyticsBannerInfo
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.ActivityRequestCodes
import com.asda.android.base.core.view.adapter.CustomFragmentPagerAdapter
import com.asda.android.base.core.view.ui.visible
import com.asda.android.recipes.RecipesManager
import com.asda.android.recipes.view.fragments.EXTRA_SEARCH_TERM
import com.asda.android.recipes.view.fragments.SearchRecipeListFragment
import com.asda.android.restapi.search.model.AsdaSearchData
import com.asda.android.restapi.search.model.SearchData
import com.asda.android.restapi.service.RestUtils
import com.asda.android.search.AsdaSearchConfig
import com.asda.android.search.constants.Constants.SEARCH_INFO
import com.asda.android.search.model.SearchResultsInfoModel
import com.asda.android.search.view.SearchResultsFragment
import com.google.android.material.tabs.TabLayout


private const val EXTRA_SEARCH_DATA = "extra_search_data"
private const val EXTRA_SPECIAL_OFFERS = "extra_special_offers"
private const val EXTRA_SHOPPING_LIST = "extra_shopping_list"

class ShelfRecipesTabbedFragment : AbstractShopFragment() {
    private val TAG = "ShelfRecipesTabbedFragment"
    private var headerText: CharSequence? = null
    private var viewPager: androidx.viewpager.widget.ViewPager? = null
    private val searchEventTriggered = MutableLiveData<Boolean>(false)
    private val filtersRequestCode = 100
    private var refineButton: TextView? = null

    override fun getInternalTag() = TAG

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val containerView = inflater.inflate(R.layout.shelf_recipes_tabbed, container, false)
        refineButton = activity?.findViewById<TextView>(R.id.filter_button_breadcrumbs)
        viewPager = containerView.findViewById(R.id.pager) as androidx.viewpager.widget.ViewPager?
        viewPager?.let { viewPager ->
            activity?.let {
                val tabLayout = containerView?.findViewById(R.id.sliding_tabs) as TabLayout?
                val adapter = ViewPagerAdapter(getCurrentFragmentManager() ?: childFragmentManager, it,
                      arguments?.getParcelable(EXTRA_SEARCH_DATA), arguments?.getBoolean(EXTRA_SPECIAL_OFFERS),
                      arguments?.getBoolean(EXTRA_SHOPPING_LIST), this,
                      headerText, searchEventTriggered)
                viewPager.adapter = adapter
                tabLayout?.setupWithViewPager(viewPager)
                tabLayout?.visibility = View.VISIBLE
                tabLayout?.addOnTabSelectedListener(object :
                      TabLayout.OnTabSelectedListener {
                    override fun onTabSelected(tab: TabLayout.Tab) {
                        if (tab.position == 1 && searchEventTriggered.value == false) {
                            val pageViewEvent = PageViewEvent(Anivia.PV_SEARCH_RESULT, Anivia.SECTION_RECIPES_LIST)
                            pageViewEvent.putString(Anivia.SEARCH_TERM, RecipesManager.instance?.getSearchTerm())
                            pageViewEvent.putString(Anivia.SEARCH_RESULTS, RecipesManager.instance?.getSearchResultsSize())
                            pageViewEvent.putString(Anivia.RECIPES_KEY, RecipesManager.instance?.getRecipeIds())
                            RecipesManager.instance?.tracker?.trackPageView(pageViewEvent)
                            searchEventTriggered.postValue(true)
                        }

                        context?.let {
                            if (RestUtils.isTablet(it) && FeatureSettings.isCMSSearchEnabled(it)) {
                                refineButton?.visible()
                                when (val fragment = adapter.getItem(tab.position)) {
                                    is SearchResultsFragment -> {
                                        setUpRefineButton()
                                    }
                                    is SearchRecipeListFragment -> {
                                        refineButton?.setOnClickListener {
                                            setUpRecipesRefineButton(fragment)
                                        }
                                    }
                                }
                            }
                        }
                    }

                    override fun onTabUnselected(tab: TabLayout.Tab) {
                        //ignore tab unselected

                    }

                    override fun onTabReselected(tab: TabLayout.Tab) {
                        //ignore tab reselected
                    }

                })
            }
        }

        return containerView
    }

    fun setUpRefineButton() {
        refineButton?.visible()
        refineButton?.setOnClickListener { _ ->
            activity?.let {
                AsdaSearchConfig.productManager.launchFilter(it, filtersRequestCode, sourceScreenName = "Search Results")
            }
        }
    }

    fun setUpRecipesRefineButton(fragment: SearchRecipeListFragment) {
        refineButton?.visible()
        refineButton?.setOnClickListener { view ->
            activity?.let {
                FilterActivity.showFilterForRecipesSearch(it, ActivityRequestCodes.REQUEST_CODE_FILTER,
                      fragment.sortMode, fragment.recipeSortOrder, arguments?.getString(EXTRA_SEARCH_TERM) ?: "",
                      fragment.activeFilters?.map { AsdaFilter(it) }?.toTypedArray(), view)
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        val fragmentManager = getCurrentFragmentManager() ?: childFragmentManager
        fragmentManager.fragments.forEach {
            if (it is SearchResultsFragment || it is SearchRecipeListFragment) {
                it.onActivityResult(requestCode, resultCode, data)
            }
        }
    }

    fun setHeaderText(text: CharSequence?) {
        headerText = text
    }

    override fun onDestroyView() {
        // Clean up the adapter to remove the fragments
        Handler().post {
            viewPager?.adapter = null
        }
        super.onDestroyView()
    }

    companion object {
        @JvmStatic
        fun newInstance(searchData: AsdaSearchData, isSpecialOffers: Boolean, isShoppingList: Boolean = false) = ShelfRecipesTabbedFragment().apply {
            arguments = Bundle().apply {
                putParcelable(EXTRA_SEARCH_DATA, searchData)
                putBoolean(EXTRA_SPECIAL_OFFERS, isSpecialOffers)
                putBoolean(EXTRA_SHOPPING_LIST, isShoppingList)
            }
        }
    }
}

private class ViewPagerAdapter(val manager: androidx.fragment.app.FragmentManager, private val activity: Activity,
                               private val searchData: AsdaSearchData?, private  val isSpecialOffers: Boolean? = false, val isShoppingList: Boolean? = false,
                               private val shelfRecipesTabbedFragment: ShelfRecipesTabbedFragment,private val headerText: CharSequence?,
                               private val searchEventTriggered: MutableLiveData<Boolean>) : CustomFragmentPagerAdapter(manager) {
    private val TAB_COUNT = 2

    override fun getCount() = TAB_COUNT

    override fun getItem(position: Int): Fragment = when (position) {
        0 -> {
            if (FeatureSettings.isCMSSearchEnabled(activity) && isShoppingList != true) {
                val searchResultsInfo = SearchResultsInfoModel(
                      searchData, isSpecialOffers ?: false)

                val fragment = SearchResultsFragment()
                val bundle = Bundle()
                bundle.putParcelable(SEARCH_INFO, searchResultsInfo)
                fragment.arguments = bundle
                fragment.onHandleMerchandisingEvent = { pushEvent ->
                    MerchandisingHandler.handleMerchandisingEvent(activity, manager,
                          pushEvent, null, AnalyticsBannerInfo("search redirect",
                          "mobile banner", "slot 0").buildAnalyticsBannerInfoString())
                }
                fragment.onSearchRequested = {
                    shelfRecipesTabbedFragment.onSearchRequested(it)
                }
                fragment
            } else {
                ShelfListSearchFragment.newInstance(searchData ?: AsdaSearchData(
                      "",
                      SearchData.SearchType.SEARCH_TYPE_EXACT,
                      ""
                ),
                      isSpecialOffers ?: false).apply {
                    headerText?.let { setHeaderText(it) }
                }
            }
        }
        1 -> SearchRecipeListFragment.newInstance(searchData?.searchText?.toString() ?: "").apply {
            REQUEST_CODE_FILTER = ActivityRequestCodes.REQUEST_CODE_FILTER

            refineButtonId = if (RestUtils.isTablet(activity)) {
                R.id.filter_button_breadcrumbs
            } else {
                R.id.filter_button
            }

            searchEventTriggered.observe(this, Observer<Boolean> {
                triggerSearch.postValue(it)
            })

            onRefineClicked = { view ->
                FilterActivity.showFilterForRecipesSearch(activity, this, ActivityRequestCodes.REQUEST_CODE_FILTER,
                      sortMode, recipeSortOrder, arguments?.getString(EXTRA_SEARCH_TERM) ?: "",
                      activeFilters?.map { AsdaFilter(it) }?.toTypedArray(), view)
            }
        }
        else -> Fragment()
    }

    override fun setPrimaryItem(container: ViewGroup, position: Int, fragment: Any) {
        super.setPrimaryItem(container, position, fragment)
            when (fragment) {
                is ShelfListSearchFragment -> fragment.setupRefinePanelView()
                is SearchRecipeListFragment -> {
                    if (FeatureSettings.isCMSSearchEnabled(activity)) {
                        shelfRecipesTabbedFragment.setUpRecipesRefineButton(fragment)
                    }else {
                        fragment.setupRefineButton()
                    }
                }
                is SearchResultsFragment -> shelfRecipesTabbedFragment.setUpRefineButton()
            }
    }

    override fun getPageTitle(position: Int): CharSequence = when (position) {
        0 -> activity.getString(R.string.tab_search_items)
        1 -> activity.getString(R.string.tab_search_recipes)
        else -> ""
    }
}
