package com.asda.android.app.shop.productdetail.helper

import android.content.Context
import androidx.annotation.VisibleForTesting
import com.asda.android.analytics.info.CartItemAnalyticsInfo
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.app.shop.*
import com.asda.android.app.shoppinglists.ShoppingListManager
import com.asda.android.base.core.view.ui.ItemQuantityController
import com.asda.android.base.interfaces.IQuantityController
import com.asda.android.base.interfaces.ShopContext
import com.asda.android.products.ui.AsdaProductsConfig
import com.asda.android.restapi.app.product.model.SearchInfoModel
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.data.Cart
import com.asda.android.restapi.service.data.catalog.IroProductDetailsPdp
import com.asda.android.restapi.service.data.catalog.IroProductDetailsPlp
import com.asda.android.service.base.CartManager
import com.asda.android.utils.Utils
import java.math.BigDecimal

class ProductHelper {

    private fun validateQuantity(context: Context, shelfListItem: ShelfListItem, specialOffersTagIsClickable: Boolean, isBigIcon: Boolean, mode: Int, shopContext: ShopContext, quantityController: IQuantityController): Pair<BigDecimal, BigDecimal> {
        val cartItem: Cart.CartItem?
        val cxoCartItem: com.asda.android.restapi.service.data.cart.Cart.CartItems?
        val qtyInCart: BigDecimal
        if (isCXOEnabled(context)) {
            cxoCartItem = CXOHelper().getItemInCartCXO(shelfListItem, shopContext.isShoppingList)
            qtyInCart = if (cxoCartItem != null) cxoCartItem.quantity?.let { BigDecimal.valueOf(it) }
                  ?: BigDecimal.ZERO else BigDecimal.ZERO
        } else {
            cartItem = AsdaProductsConfig.productManager.getCartItemFrom(context, shelfListItem, mode, shopContext.isShoppingList)
            qtyInCart = if (cartItem != null) Utils.getCartQuantity(cartItem.quantityString) else BigDecimal.ZERO
        }
        return Pair(qtyInCart, quantityController.getCurrentQuantity() ?: BigDecimal.ZERO)
    }

    fun validateQuantity(context: Context, item: IroProductDetailsPdp.Items, specialOffersTagIsClickable: Boolean, isBigIcon: Boolean, mode: Int, shopContext: ShopContext, quantityController: IQuantityController): Pair<BigDecimal, BigDecimal> {
        val shelfListItem = buildShelfObjectPdp(item, specialOffersTagIsClickable)
        return validateQuantity(context, shelfListItem, specialOffersTagIsClickable, isBigIcon, mode, shopContext, quantityController)
    }

    fun validateQuantity(context: Context, item: IroProductDetailsPlp.Items, specialOffersTagIsClickable: Boolean, isBigIcon: Boolean, mode: Int, shopContext: ShopContext, quantityController: IQuantityController): Pair<BigDecimal, BigDecimal> {
        val shelfListItem = buildShelfObject(item, specialOffersTagIsClickable, isBigIcon)
        return validateQuantity(context, shelfListItem, specialOffersTagIsClickable, isBigIcon, mode, shopContext, quantityController)
    }

    fun getCartAnalytics(context: Context, position: Int, activeSort: Int, searchInfoModel: SearchInfoModel, firstCartAdd: Boolean, shelfListItem: ShelfListItem, pageName: String? = null): CartItemAnalyticsInfo {
        val cartItemAnalyticsInfoBuilder = CartItemAnalyticsInfo.CartItemAnalyticsInfoBuilder()
              .setPosition(position.toString())
              .setSearchResults(searchInfoModel.searchResultsCount)
              .setSearchTerm(searchInfoModel.searchTerm)
              .setSortMethod(context.getString(getSortTitle(activeSort)))
              .setFilterAnalytics(Utils.formatAnalyticsFilters(getShopFilter().selectedNutritionFilters))
              .setStrategyId(shelfListItem.strategyId)
              .setStrategyTitle(shelfListItem.strategyTitle)
              .setPlacement(shelfListItem.placement)
              .setSearchType(searchInfoModel.searchType)
              .setPreviousSearchTerm(searchInfoModel.previousSearchTerm)
              .isFirstCartAdd(firstCartAdd)
              .setPageName(pageName)

        getShopFilter().selectedCustomNutritionFilter?.filterName?.let {
            cartItemAnalyticsInfoBuilder.setCustomFilter(it)
        }
        return cartItemAnalyticsInfoBuilder.build()
    }

    fun getCartItemFrom(context: Context, item: ShelfListItem, mode: Int, isShoppingList: Boolean): Cart.CartItem? {
        var cartItem: Cart.CartItem?
        if (isShoppingList) {
            cartItem = getShoppingListManager().getItemByStoresId(item.id)
            if (cartItem != null || isShoppingList && (mode == ShelfListAdapter.LIST_MODE_STATIC || mode == ShelfListBaseFragment.LIST_MODE_SEARCH)) {
                return cartItem
            }
        }

        if (mode == ShelfListAdapter.LIST_MODE_ADD_TO_TROLLEY_SELECTION) {
            cartItem = Cart.CartItem.CartItemBuilder()
                  .setId(item.id)
                  .setDesc(item.title)
                  .setImageURL(item.imageURL)
                  .setDeptName(item.deptName)
                  .setPrice(item.priceToDisplay)
                  .setPricePerWt(item.pricePerWt)
                  .setPricePerUOM(item.pricePerUOM)
                  .setAvgWeight(item.avgWeight)
                  .setMaxQty(item.maxQty)
                  .setQuantityString(item.qty)
                  .setPromoMissingCount(item.promoMissingCount.toString() + "")
                  .setCost(ItemQuantityController.calcCost(RestUtils.poundStringToBigDecimal(item.priceToDisplay), item.avgWeight, item.pricePerWt, item.qty))
                  .build()
        } else {
            cartItem = getCartManager(context)?.getItemInCart(item.id)
        }
        return cartItem
    }

    @VisibleForTesting
    fun getShoppingListManager(): ShoppingListManager {
        return ShoppingListManager.getInstance()
    }

    @VisibleForTesting
    fun getCartManager(context: Context): CartManager? {
        return CartManager.get(context)
    }

    @VisibleForTesting
    fun getShopFilter(): ShopFilters {
        return ShopFilters.instance
    }

    @VisibleForTesting
    fun isCXOEnabled(context: Context): Boolean {
        return FeatureSettings.isCXOMigrationEnabled(context)
    }

    @VisibleForTesting
    fun buildShelfObject(item: IroProductDetailsPlp.Items, specialOffersTagIsClickable: Boolean, isBigIcon: Boolean): ShelfListItem {
        return ShelfListItem.buildShelfObject(item, specialOffersTagIsClickable, isBigIcon)
    }

    @VisibleForTesting
    fun buildShelfObjectPdp(item: IroProductDetailsPdp.Items, specialOffersTagIsClickable: Boolean): ShelfListItem {
        return ShelfListItem.buildShelfObjectPdp(item, specialOffersTagIsClickable)
    }
}
