package com.asda.android.service.base


import android.util.Log
import com.asda.android.app.dynamic.manager.ContentManager
import com.asda.android.app.main.AsdaApplication
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.app.shop.model.ShelfListItem.buildShelfObject
import com.asda.android.restapi.service.NetworkCallback
import com.asda.android.restapi.service.base.AsdaServiceSettings
import com.asda.android.restapi.service.base.RxFailure
import com.asda.android.restapi.service.data.Merchandising
import com.asda.android.restapi.service.data.catalog.IroProductDetailsPlp
import com.asda.android.restapi.service.data.cms.*
import io.reactivex.Observable
import io.reactivex.ObservableEmitter
import java.util.*

object IroProductListService {
    private val TAG = "IroProductListService"

    class ProductListData {
        var mTitle: String? = null
        var mList = ArrayList<ShelfListItem>()
        var mMenuList = ArrayList<ContentNav.ContentNavigationItem>()
        var mBanner: Merchandising.Banner? = null
    }

    fun getProductList(pageId: String?, queryString: String?): Observable<ProductListData> {
        return Observable.create<ProductListData> { emitter ->

            val asyncCallback = object : NetworkCallback<Content>() {

                override fun onFailure(error: Int?, data: Content?) {
                    if (emitter.isDisposed) return
                    if (AsdaServiceSettings.DEBUG) {
                        Log.e(TAG, "Failed to get product sku list from PageService.")
                    }
                    emitter.onError(RxFailure(error, null))
                }

                override fun onSuccess(data: Content?) {
                    if (data == null || emitter.isDisposed) return
                    val productListData = ProductListData()
                    ContentManager.produce({ content: Content ->
                        onProductListSuccess(emitter, content, productListData)
                    }, data)
                    if (productListData.mBanner == null && productListData.mList.isEmpty() && productListData.mMenuList.isEmpty()) {
                        // We should still emit any empty result
                        emitter.onNext(productListData)
                    }
                }
            }
            AsdaServiceFactory.get().getPage(pageId, queryString, AsdaApplication.getComponent().getCampaignIdProvider().getCampaignId(pageId), asyncCallback, null)
        }
    }

    private fun getBannerFromPageService(productListData: ProductListData, content: BannerBase, emitter: ObservableEmitter<ProductListData>) {
        Log.i(TAG, "Received banner from PageService.")
        productListData.mBanner = content.content()
        emitter.onNext(productListData)
    }

    private fun loadProductDetails(productListData: ProductListData, emitter: ObservableEmitter<ProductListData>) {
        if (productListData.mList.size > 0) {
            val builder = StringBuilder(productListData.mList[0].id)

            for (i in 1 until productListData.mList.size) {
                builder.append(",")
                builder.append(productListData.mList[i].id)
            }

            com.asda.android.service.base.AsdaServiceFactory.get().iroGetProductDetails(builder.toString(), "basic", object : NetworkCallback<IroProductDetailsPlp>() {
                override fun onFailure(error: Int?, data: IroProductDetailsPlp?) {
                    if (emitter.isDisposed) return
                    if (AsdaServiceSettings.DEBUG) {
                        Log.e(TAG, "Failed to get product details for PageService items. Error=$error Data=$data")
                    }
                    emitter.onError(RxFailure(error, null))
                }

                override fun onSuccess(data: IroProductDetailsPlp?) {
                    if (emitter.isDisposed) return
                    data?.getItems()?.let { items ->
                        onSuccess(productListData, emitter, items)
                    }
                    emitter.onComplete()
                }
            })
        } else {
            if (AsdaServiceSettings.DEBUG) {
                Log.e(TAG, "Product list cartridge is empty!")
            }
            emitter.onError(RxFailure(-1, null))
        }
    }

    private fun onProductListSuccess(emitter: ObservableEmitter<ProductListData>, content: Content, productListData: ProductListData) {
        if (emitter.isDisposed) return
        when (content) {
            is ProductSpotlight -> {
                Log.i(TAG, "Received product items from PageService. Calling item details.")
                productListData.mList = content.content()
                productListData.mTitle = content.title
                emitter.onNext(productListData)
                loadProductDetails(productListData, emitter)
            }
            is ContentNav -> {
                Log.i(TAG, "Received menu items from PageService.")
                productListData.mMenuList = content.content()
                emitter.onNext(productListData)
            }
            is BannerStatic -> {
                getBannerFromPageService(productListData, content, emitter)
            }
            is BannerAddToBasket -> {
                getBannerFromPageService(productListData, content, emitter)
            }
        }
    }

    private fun onSuccess(productListData: ProductListData, emitter: ObservableEmitter<ProductListData>, items: List<IroProductDetailsPlp.Items>) {
        if (items.isEmpty()) return
        productListData.mList.clear()
        for (i in items.indices) {
            val shelfItem = buildShelfObject(items[i], true, false)
            productListData.mList.add(shelfItem)
        }
        Log.i(TAG, "Received items details.")
        emitter.onNext(productListData)
    }
}
