package com.asda.giftcard.view

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.util.Log
import android.view.View
import android.widget.Button
import android.widget.EditText
import android.widget.ImageView
import android.widget.TextView
import androidx.annotation.VisibleForTesting
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.ApiErrorEvent
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.scheduler.SchedulerProvider
import com.asda.android.base.core.statemanagement.type.State
import com.asda.android.base.core.view.actitivies.BaseAppCompatActivity
import com.asda.android.base.core.view.ui.DialogFactory
import com.asda.android.cxo.datasource.OrderSummaryDataSource
import com.asda.android.restapi.service.data.cart.Cart
import com.asda.android.restapi.service.data.checkout.CheckoutResponse
import com.asda.android.restapi.service.data.giftcard.GiftCardResponse
import com.asda.android.service.base.Authentication
import com.asda.android.utils.Utils
import com.asda.giftcard.GiftCardFormatter
import com.asda.giftcard.GiftCardUtils
import com.asda.giftcard.INVALID_GIFT_CARD_NUMBER
import com.asda.giftcard.INVALID_GIFT_CARD_PIN
import com.asda.giftcard.models.GiftCardViewModel
import com.fasterxml.jackson.databind.ObjectMapper
import com.jakewharton.retrofit2.adapter.rxjava2.HttpException
import io.reactivex.observers.ResourceObserver
import io.reactivex.observers.ResourceSingleObserver
import org.json.JSONObject


const val EXTRA_ORDER_ID = "OrderId"
const val ADD_GIFT_CARD_RESPONSE = "apply_gift_card_response"
const val GIFT_CARD_TOTAL = "gift_card_total"
const val DP_ENABLED = "dpEnabled"
const val PREVIOUS_PAGE = "previousPageName"

class AddGiftCardActivity : BaseAppCompatActivity() {

    private val TAG = "AddGiftCardActivity"

    var viewModel: GiftCardViewModel? = null
    var applyGiftCard: Button? = null
    private var cancelGiftCard: Button? = null
    private var giftCardNumber: EditText? = null
    private var giftCardPin: EditText? = null
    private var giftCardNumberError: TextView? = null
    private var giftCardPinError: TextView? = null
    private var giftCardAlertIcon: ImageView? = null
    private var orderId: String = ""
    private var validGiftCardNumber = false
    private var validGiftCardPin = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.add_gift_card)

        supportActionBar?.title = getString(R.string.new_gift_card_title)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        val intent = intent
        intent?.let {
            orderId = it.getStringExtra(EXTRA_ORDER_ID)
        }

        viewModel = ViewModelProviders.of(this).get(GiftCardViewModel::class.java)

        initView()
        wireListeners()
    }

    @VisibleForTesting
    fun initView() {
        applyGiftCard = findViewById(R.id.apply_gift_card)
        cancelGiftCard = findViewById(R.id.cancel_gift_card)
        giftCardNumber = findViewById(R.id.gift_card_number)
        giftCardPin = findViewById(R.id.gift_card_pin)
        giftCardNumberError = findViewById(R.id.gift_card_number_error_message)
        giftCardPinError = findViewById(R.id.gift_card_pin_error_message)
        giftCardAlertIcon = findViewById(R.id.gift_card_alert_image)
    }

    private fun wireListeners() {
        applyGiftCard?.setOnClickListener { applyGiftCard() }

        cancelGiftCard?.setOnClickListener { finish() }

        giftCardNumber?.addTextChangedListener(object : TextWatcher {
            override fun afterTextChanged(text: Editable?) {
                validGiftCardNumber = text?.isNotEmpty() == true && text.length == 16
                toggleEnableApplyButton()
            }

            override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {}

            override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {}
        })

        giftCardPin?.addTextChangedListener(object : TextWatcher {
            override fun afterTextChanged(text: Editable?) {
                validGiftCardPin = text?.isNotEmpty() == true && text.length == 4
                toggleEnableApplyButton()
            }

            override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {}

            override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {}
        })

        OrderSummaryDataSource(SchedulerProvider()).checkOutResponse.observe(this, Observer {
            when (it.getState()) {
                State.SUCCESS -> {
                    it.getData()?.errors?.let { errors ->
                        errors.forEach { error ->
                            giftCardAlertIcon?.visibility = View.VISIBLE
                            giftCardNumberError?.visibility = View.VISIBLE
                            val errorMessage = GiftCardFormatter.getErrorMessage(applicationContext, error.code)
                            giftCardNumberError?.text = errorMessage
                            errorMessage?.let { message ->
                                if (resources.getStringArray(R.array.anivia_error_codes).contains(error.code)) {
                                    val apiEvent = ApiErrorEvent.createResponseEvent(Anivia.GIFT_CARD_APPLY_API_NAME, error.code, message, System.currentTimeMillis().toString(), TAG, message)
                                    Tracker.get().trackEvent(apiEvent)
                                }
                            }
                        }
                    } ?: run {
                        val resultIntent = Intent()
                        resultIntent.putExtra(ADD_GIFT_CARD_RESPONSE, it.getData())
                        setResult(Activity.RESULT_OK, resultIntent)
                        finish()
                    }
                }

                State.ERROR -> {
                    if (it.getError() is HttpException) {
                        val errorResponse = (it.getError() as HttpException).response()?.errorBody()?.string()
                        errorResponse?.let {
                            val objectMapper = ObjectMapper()
                            val errorObj = objectMapper.readValue(JSONObject(it).getJSONArray(applicationContext.getString(R.string.gift_card_error)).get(0).toString(), Cart.Error::class.java)
                            giftCardPinError?.text = GiftCardFormatter.getErrorMessage(applicationContext, errorObj.code)
                            giftCardPinError?.visibility = View.VISIBLE
                        }
                    } else {
                        giftCardPinError?.text = getString(R.string.oops_connectivity_error)
                    }
                    if (Utils.isDebug()) {
                        Log.d(TAG, "Error calling API", it.getError())
                    }
                }
            }
        })
    }

    private fun toggleEnableApplyButton() {
        applyGiftCard?.isEnabled = validGiftCardNumber && validGiftCardPin
    }

    @VisibleForTesting
    fun applyGiftCard() {
        if (GiftCardUtils.isValidGiftCardInfo(giftCardNumber?.text.toString(), giftCardPin?.text.toString())) {
               giftCardNumberError?.visibility = View.GONE
               giftCardPinError?.visibility = View.GONE
                if(FeatureSettings.isCXOMigrationEnabled(applicationContext)) {
                    executeAddGiftCardCXO()
                } else {
                    executeAddGiftCard()
                }
        } else {
            GiftCardUtils.errorCode?.let {
                if (it.contentEquals(INVALID_GIFT_CARD_NUMBER)) {
                    giftCardNumberError?.text = getString(R.string.invalid_gift_card_number)
                    giftCardNumberError?.visibility = View.VISIBLE
                } else {
                    giftCardNumberError?.visibility = View.GONE
                }

                if (it.contentEquals(INVALID_GIFT_CARD_PIN)) {
                    giftCardPinError?.text = getString(R.string.invalid_gift_card_pin)
                    giftCardPinError?.visibility = View.VISIBLE
                } else {
                    giftCardPinError?.visibility = View.GONE
                }
            }
        }
    }

    private fun executeAddGiftCardCXO() {
        val dialog = DialogFactory.createDialog(DialogFactory.LOADING_INDICATOR_NOT_CANCELABLE, getString(com.asda.android.cxo.R.string.applying_giftcard), this)
        dialog.show()
        OrderSummaryDataSource(SchedulerProvider()).applyGiftCard(giftCardNumber?.text.toString(), giftCardPin?.text.toString())?.subscribe(object : ResourceSingleObserver<CheckoutResponse>() {
            override fun onSuccess(response: CheckoutResponse) {
                try {
                    dialog?.dismiss()
                } catch (ignore: Throwable) {
                    Log.w(TAG, ignore)
                }
                if (response.errors?.isNotEmpty() == true) {
                    response.errors?.let {
                        it.forEach { error ->
                            giftCardAlertIcon?.visibility = View.VISIBLE
                            giftCardNumberError?.visibility = View.VISIBLE
                            var errorMessage = GiftCardFormatter.getErrorMessage(applicationContext, error.code)
                            giftCardNumberError?.text = errorMessage
                            errorMessage?.let {
                                if (resources.getStringArray(R.array.anivia_error_codes).contains(error.code)) {
                                    var apiEvent = ApiErrorEvent.createResponseEvent(Anivia.GIFT_CARD_APPLY_API_NAME, error.code, it, System.currentTimeMillis().toString(), TAG, it)
                                    Tracker.get().trackEvent(apiEvent)
                                }
                            }
                        }
                    }
                } else {
                    val resultIntent = Intent()
                    resultIntent.putExtra(ADD_GIFT_CARD_RESPONSE, response)
                    setResult(Activity.RESULT_OK, resultIntent)
                    finish()
                }
            }

            override fun onError(throwable: Throwable) {
                try {
                    dialog?.dismiss()
                } catch (ignore: Throwable) {
                    Log.w(TAG, ignore)
                }
                if (throwable is HttpException) {
                    val errorResponse = throwable.response().errorBody()?.string()
                    errorResponse?.let {
                        val objectMapper = ObjectMapper()
                        val errorObj = objectMapper.readValue(JSONObject(it).getJSONArray(applicationContext.getString(R.string.gift_card_error)).get(0).toString(), Cart.Error::class.java)
                        giftCardPinError?.text = GiftCardFormatter.getErrorMessage(applicationContext, errorObj.code)
                        giftCardPinError?.visibility = View.VISIBLE
                    }
                } else {
                    giftCardPinError?.text = getString(R.string.oops_connectivity_error)
                }

                dispose()
                if (Utils.isDebug()) {
                    Log.d(TAG, "Error calling API", throwable)
                }
            }


        })
    }

    private fun executeAddGiftCard() {
        viewModel?.applyGiftCard(Authentication.getInstance().csrfToken, giftCardNumber?.text.toString(), giftCardPin?.text.toString(), orderId)
              ?.safeSubscribe(object : ResourceObserver<GiftCardResponse>() {
                  override fun onComplete() {
                      dispose()
                  }

                  override fun onNext(response: GiftCardResponse) {
                      response.errors?.let {
                          it.forEach { error ->
                              giftCardAlertIcon?.visibility = View.VISIBLE
                              giftCardNumberError?.visibility = View.VISIBLE
                              var errorMessage = GiftCardFormatter.getErrorMessage(applicationContext, error.errorCode)
                              giftCardNumberError?.text = errorMessage
                              errorMessage?.let {
                                  if (resources.getStringArray(R.array.anivia_error_codes).contains(error.errorCode)) {
                                      var apiEvent = ApiErrorEvent.createResponseEvent(Anivia.GIFT_CARD_APPLY_API_NAME, error.errorCode, it, System.currentTimeMillis().toString(), TAG, it)
                                      Tracker.get().trackEvent(apiEvent)
                                  }
                              }
                          }
                      } ?: run {
                          val resultIntent = Intent()
                          resultIntent.putExtra(ADD_GIFT_CARD_RESPONSE, response)
                          setResult(Activity.RESULT_OK, resultIntent)
                          finish()
                      }
                  }

                  override fun onError(throwable: Throwable) {
                      if (throwable is HttpException) {
                          val errorResponse = throwable.response().errorBody()?.string()
                          errorResponse?.let {
                              val objectMapper = ObjectMapper()
                              val giftCardErrorObj = objectMapper.readValue(JSONObject(it).getJSONArray(applicationContext.getString(R.string.gift_card_error)).get(0).toString(), GiftCardResponse.Error::class.java)
                              var errorMessage = GiftCardFormatter.getErrorMessage(applicationContext, giftCardErrorObj.errorCode)
                              giftCardPinError?.text = errorMessage
                              giftCardPinError?.visibility = View.VISIBLE
                          }
                      } else {
                          giftCardPinError?.text = getString(R.string.oops_connectivity_error)
                      }

                      dispose()
                      if (Utils.isDebug()) {
                          Log.d(TAG, "Error calling API", throwable)
                      }
                  }
              })
    }

    override fun getFragmentContainer() = R.id.fragmentContainer

}
